@@ -24,10 +24,18 @@
 #include <glib.h>
 #include "callback.h"
 
+/** Filename for the chromosome of the genetic player */
+extern char *chromosomeFile;
+
 void ai_panic(const char *message);
 void ai_wait(void);
 void ai_chat(const char *message);
+void genetic_init(void);
 void greedy_init(void);
 void lobbybot_init(void);
 
+/** Chat when a player must discard resources */
+void ai_chat_discard(gint player_num, gint discard_num);
+/** Chat when the computer player moved the robber */
+void ai_chat_self_moved_robber(void);
 #endif
