@@ -43,9 +43,6 @@ typedef struct resource_values_s {
 } resource_values_t;
 
 static int quote_num;
-/* used to avoid multiple chat messages when more than one other player
- * must discard resources */
-static gboolean discard_starting;
 
 /* things we can buy, in the order that we want them. */
 typedef enum {
@@ -230,7 +227,7 @@ static void leftover_resources(const gint assets[NO_RESOURCE],
  * Probability of a dice roll
  */
 
-static float dice_prob(int roll)
+static float dice_prob(gint roll)
 {
 	switch (roll) {
 	case 2:
@@ -254,36 +251,36 @@ static float dice_prob(int roll)
 }
 
 /*
- * By default how valuable is this resource?
+ * By default how valuable is this terrain?
  */
 
-static float default_score_resource(Resource resource)
+static float default_score_terrain(Terrain terrain)
 {
 	float score;
 
-	switch (resource) {
+	switch (terrain) {
 	case GOLD_TERRAIN:	/* gold */
-		score = 1.25;
+		score = 1.25f;
 		break;
 	case HILL_TERRAIN:	/* brick */
-		score = 1.1;
+		score = 1.1f;
 		break;
 	case FIELD_TERRAIN:	/* grain */
-		score = 1.0;
+		score = 1.0f;
 		break;
 	case MOUNTAIN_TERRAIN:	/* rock */
-		score = 1.05;
+		score = 1.05f;
 		break;
 	case PASTURE_TERRAIN:	/* sheep */
-		score = 1.0;
+		score = 1.0f;
 		break;
 	case FOREST_TERRAIN:	/* wood */
-		score = 1.1;
+		score = 1.1f;
 		break;
 	case DESERT_TERRAIN:
 	case SEA_TERRAIN:
 	default:
-		score = 0;
+		score = 0.0f;
 		break;
 	}
 
@@ -311,7 +308,7 @@ static void reevaluate_iterator(Node * n, void *rock)
 			if (h && h->terrain < DESERT_TERRAIN) {
 				produce[h->terrain] +=
 				    mult *
-				    default_score_resource(h->terrain) *
+				    default_score_terrain(h->terrain) *
 				    dice_prob(h->roll);
 			}
 
@@ -340,9 +337,10 @@ static void reevaluate_resources(resource_values_t * outval)
 	 */
 	for (i = 0; i < NO_RESOURCE; i++) {
 		if (produce[i] == 0) {
-			outval->value[i] = default_score_resource(i);
+			outval->value[i] =
+			    default_score_terrain(resource_to_terrain(i));
 		} else {
-			outval->value[i] = 1.0 / produce[i];
+			outval->value[i] = 1.0f / produce[i];
 		}
 
 	}
@@ -373,7 +371,7 @@ static float resource_value(Resource resource,
 	if (resource < NO_RESOURCE)
 		return resval->value[resource];
 	else if (resource == GOLD_RESOURCE)
-		return default_score_resource(resource);
+		return default_score_terrain(GOLD_TERRAIN);
 	else
 		return 0.0;
 }
@@ -391,12 +389,13 @@ static float score_hex(Hex * hex, const resource_values_t * resval)
 
 	/* multiple resource value by dice probability */
 	score =
-	    resource_value(hex->terrain, resval) * dice_prob(hex->roll);
+	    resource_value(terrain_to_resource(hex->terrain),
+			   resval) * dice_prob(hex->roll);
 
 	/* if we don't have a 3 for 1 port yet and this is one it's valuable! */
 	if (!resval->info.any_resource) {
 		if (hex->resource == ANY_RESOURCE)
-			score += 0.5;
+			score += 0.5f;
 	}
 
 	return score;
@@ -407,14 +406,13 @@ static float score_hex(Hex * hex, const resource_values_t * resval)
  */
 static float default_score_hex(Hex * hex)
 {
-	int score;
+	float score;
 
 	if (hex == NULL)
 		return 0;
 
 	/* multiple resource value by dice probability */
-	score =
-	    default_score_resource(hex->terrain) * dice_prob(hex->roll);
+	score = default_score_terrain(hex->terrain) * dice_prob(hex->roll);
 
 	return score;
 }
@@ -423,7 +421,7 @@ static float default_score_hex(Hex * hex)
  * Give a numerical score to how valuable putting a settlement/city on this spot is
  *
  */
-static float score_node(Node * node, int city,
+static float score_node(const Node * node, gboolean city,
 			const resource_values_t * resval)
 {
 	int i;
@@ -437,7 +435,7 @@ static float score_node(Node * node, int city,
 		return -1;
 	if (is_node_spacing_ok(node) == FALSE)
 		return -1;
-	if (city == 0) {
+	if (!city) {
 		if (node->owner != -1)
 			return -1;
 	}
@@ -499,7 +497,7 @@ static Node *best_settlement_spot(gboolean during_setup,
 						continue;
 				}
 
-				score = score_node(n, 0, resval);
+				score = score_node(n, FALSE, resval);
 				if (score > bestscore) {
 					best = n;
 					bestscore = score;
@@ -533,7 +531,7 @@ static Node *best_city_spot(const resource_values_t * resval)
 				if ((n->owner == my_player_num())
 				    && (n->type == BUILD_SETTLEMENT)) {
 					float score =
-					    score_node(n, 1, resval);
+					    score_node(n, TRUE, resval);
 
 					if (score > bestscore) {
 						best = n;
@@ -599,7 +597,7 @@ static Edge *traverse_out(Node * n, node_seen_set_t * set, float *score,
 			/* no owner, how good is the other node ? */
 			cur_e = e;
 
-			cur_score = score_node(othernode, 0, resval);
+			cur_score = score_node(othernode, FALSE, resval);
 
 			/* umm.. can we build here? */
 			/*if (!can_settlement_be_built(othernode, my_player_num ()))
@@ -651,7 +649,8 @@ static Edge *best_road_to_road_spot(Node * n, float *score,
 						    score_node(other_node
 							       (e2,
 								othernode),
-							       0, resval);
+							       FALSE,
+							       resval);
 
 						if (nscore > bscore) {
 							bscore = nscore;
@@ -983,7 +982,7 @@ static void greedy_setup_road(void)
 {
 	Node *node;
 	Edge *e = NULL;
-	int i;
+	guint i;
 	resource_values_t resval;
 	float tmp;
 
@@ -1165,21 +1164,21 @@ static gboolean will_play_development_card(DevelType cardtype)
 static void greedy_turn(void)
 {
 	resource_values_t resval;
-	int i;
+	guint i;
 	gint need[NO_RESOURCE], assets[NO_RESOURCE];
 
 	/* play soldier card before the turn when an own resource is blocked */
 	Hex *hex = map_robber_hex(callbacks.get_map());
 	if (hex && !have_rolled_dice() && can_play_any_develop()) {
-		const DevelDeck *deck = get_devel_deck();
-		for (i = 0; i < deck->num_cards; i++) {
-			DevelType cardtype = deck_card_type(deck, i);
+		const Deck *deck = get_devel_deck();
+		for (i = 0; i < deck_count(deck); i++) {
+			DevelType cardtype = deck_get_guint(deck, i);
 			if (cardtype == DEVEL_SOLDIER
 			    && can_play_develop(i)) {
 				int j;
 				for (j = 0; j < 6; j++) {
-					if ((hex->nodes[j]->owner ==
-					     my_player_num())) {
+					if (hex->nodes[j]->owner ==
+					    my_player_num()) {
 						cb_play_develop(i);
 						return;
 					}
@@ -1258,12 +1257,12 @@ static void greedy_turn(void)
 
 	/* play development cards */
 	if (can_play_any_develop()) {
-		const DevelDeck *deck = get_devel_deck();
+		const Deck *deck = get_devel_deck();
 		gint num_victory_cards = 0;
 		gint victory_point_target, my_points;
 
-		for (i = 0; i < deck->num_cards; i++) {
-			DevelType cardtype = deck_card_type(deck, i);
+		for (i = 0; i < deck_count(deck); i++) {
+			DevelType cardtype = deck_get_guint(deck, i);
 
 			/* if it's a vp card, note this for later */
 			if (is_victory_card(cardtype)) {
@@ -1284,9 +1283,9 @@ static void greedy_turn(void)
 		victory_point_target = game_victory_points();
 		my_points = player_get_score(my_player_num());
 		if (num_victory_cards + my_points >= victory_point_target) {
-			for (i = 0; i < deck->num_cards; i++) {
+			for (i = 0; i < deck_count(deck); i++) {
 				DevelType cardtype =
-				    deck_card_type(deck, i);
+				    deck_get_guint(deck, i);
 
 				if (is_victory_card(cardtype)) {
 					cb_play_develop(i);
@@ -1298,124 +1297,6 @@ static void greedy_turn(void)
 	cb_end_turn();
 }
 
-#define randchat(array,nochat_percent)				\
-	do {							\
-		int p = (G_N_ELEMENTS(array)*1000/nochat_percent);	\
-		int n = (rand() % p) / 10;			\
-		if (n < G_N_ELEMENTS(array) )			\
-			ai_chat (array[n]);			\
-	} while(0)
-
-static const char *chat_turn_start[] = {
-	/* AI chat at the start of the turn */
-	N_("Ok, let's go!"),
-	/* AI chat at the start of the turn */
-	N_("I'll beat you all now! ;)"),
-	/* AI chat at the start of the turn */
-	N_("Now for another try..."),
-};
-
-static const char *chat_receive_one[] = {
-	/* AI chat when one resource is received */
-	N_("At least I get something..."),
-	/* AI chat when one resource is received */
-	N_("One is better than none..."),
-};
-
-static const char *chat_receive_many[] = {
-	/* AI chat when more than one resource is received */
-	N_("Wow!"),
-	/* AI chat when more than one resource is received */
-	N_("Ey, I'm becoming rich ;)"),
-	/* AI chat when more than one resource is received */
-	N_("This is really a good year!"),
-};
-
-static const char *chat_other_receive_many[] = {
-	/* AI chat when other players receive more than one resource */
-	N_("You really don't deserve that much!"),
-	/* AI chat when other players receive more than one resource */
-	N_("You don't know what to do with that many resources ;)"),
-	/* AI chat when other players receive more than one resource */
-	N_("Ey, wait for my robber and lose all this again!"),
-};
-
-static const char *chat_self_moved_robber[] = {
-	/* AI chat when it moves the robber */
-	N_("Hehe!"),
-	/* AI chat when it moves the robber */
-	N_("Go, robber, go!"),
-};
-
-static const char *chat_moved_robber_to_me[] = {
-	/* AI chat when the robber is moved to it */
-	N_("You bastard!"),
-	/* AI chat when the robber is moved to it */
-	N_("Can't you move that robber somewhere else?!"),
-	/* AI chat when the robber is moved to it */
-	N_("Why always me??"),
-};
-
-static const char *chat_discard_self[] = {
-	/* AI chat when it must discard resources */
-	N_("Oh no!"),
-	/* AI chat when it must discard resources */
-	N_("Grrr!"),
-	/* AI chat when it must discard resources */
-	N_("Who the hell rolled that 7??"),
-	/* AI chat when it must discard resources */
-	N_("Why always me?!?"),
-};
-
-static const char *chat_discard_other[] = {
-	/* AI chat when other players must discard */
-	N_("Say good bye to your cards... :)"),
-	/* AI chat when other players must discard */
-	N_("*evilgrin*"),
-	/* AI chat when other players must discard */
-	N_("/me says farewell to your cards ;)"),
-	/* AI chat when other players must discard */
-	N_("That's the price for being rich... :)"),
-};
-
-static const char *chat_stole_from_me[] = {
-	/* AI chat when someone steals from it */
-	N_("Ey! Where's that card gone?"),
-	/* AI chat when someone steals from it */
-	N_("Thieves! Thieves!!"),
-	/* AI chat when someone steals from it */
-	N_("Wait for my revenge..."),
-};
-
-static const char *chat_monopoly_other[] = {
-	/* AI chat when someone plays the monopoly card */
-	N_("Oh no :("),
-	/* AI chat when someone plays the monopoly card */
-	N_("Must this happen NOW??"),
-	/* AI chat when someone plays the monopoly card */
-	N_("Args"),
-};
-
-static const char *chat_largestarmy_self[] = {
-	/* AI chat when it has the largest army */
-	N_("Hehe, my soldiers rule!"),
-};
-
-static const char *chat_largestarmy_other[] = {
-	/* AI chat when another player that the largest army */
-	N_("First robbing us, then grabbing the points..."),
-};
-
-static const char *chat_longestroad_self[] = {
-	/* AI chat when it has the longest road */
-	N_("See that road!"),
-};
-
-static const char *chat_longestroad_other[] = {
-	/* AI chat when another player has the longest road */
-	N_("Pf, you won't win with roads alone..."),
-};
-
 static float score_node_hurt_opponents(Node * node)
 {
 	/* no building there */
@@ -1517,7 +1398,7 @@ static void greedy_steal_building(void)
 		}
 	}
 	cb_rob(victim);
-	randchat(chat_self_moved_robber, 15);
+	ai_chat_self_moved_robber();
 }
 
 /*
@@ -1913,7 +1794,7 @@ static void greedy_consider_quote(G_GNUC_UNUSED gint partner,
 	log_message(MSG_INFO, "Quoting.\n");
 }
 
-static void greedy_setup(unsigned num_settlements, unsigned num_roads)
+static void greedy_setup(gint num_settlements, gint num_roads)
 {
 	ai_wait();
 	if (num_settlements > 0)
@@ -1933,22 +1814,12 @@ static void greedy_roadbuilding(gint num_roads)
 		cb_end_turn();
 }
 
-static void greedy_discard_start(void)
-{
-	discard_starting = TRUE;
-}
-
 static void greedy_discard_add(gint player_num, gint discard_num)
 {
+	ai_chat_discard(player_num, discard_num);
 	if (player_num == my_player_num()) {
-		randchat(chat_discard_self, 10);
 		ai_wait();
 		greedy_discard(discard_num);
-	} else {
-		if (discard_starting) {
-			discard_starting = FALSE;
-			randchat(chat_discard_other, 10);
-		}
 	}
 }
 
@@ -1968,10 +1839,11 @@ static void greedy_gold_choose(gint gold_num, const gint * bank)
 	}
 
 	for (i = 0; i < gold_num; i++) {
+		gint j;
+
 		reevaluate_resources(&resval);
 
 		/* If the bank has been emptied, don't desire it */
-		gint j;
 		for (j = 0; j < NO_RESOURCE; j++) {
 			if (my_bank[j] == 0) {
 				resval.value[j] = 0;
@@ -1991,101 +1863,6 @@ static void greedy_gold_choose(gint gold_num, const gint * bank)
 		my_bank[r1]--;
 	}
 	cb_choose_gold(want);
-
-}
-
-static void greedy_error(const gchar * message)
-{
-	gchar *buffer;
-
-	buffer =
-	    g_strdup_printf(_(""
-			      "Received error from server: %s.  Quitting\n"),
-			    message);
-	cb_chat(buffer);
-	g_free(buffer);
-	cb_disconnect();
-}
-
-static void greedy_game_over(gint player_num, G_GNUC_UNUSED gint points)
-{
-	if (player_num == my_player_num()) {
-		/* AI chat when it wins */
-		ai_chat(N_("Yippie!"));
-	} else {
-		/* AI chat when another player wins */
-		ai_chat(N_("My congratulations"));
-	}
-	cb_disconnect();
-}
-
-/* functions for chatting follow */
-static void greedy_player_turn(gint player)
-{
-	if (player == my_player_num())
-		randchat(chat_turn_start, 70);
-}
-
-static void greedy_robber_moved(G_GNUC_UNUSED Hex * old, Hex * new)
-{
-	int idx;
-	gboolean iam_affected = FALSE;
-	for (idx = 0; idx < G_N_ELEMENTS(new->nodes); idx++) {
-		if (new->nodes[idx]->owner == my_player_num())
-			iam_affected = TRUE;
-	}
-	if (iam_affected)
-		randchat(chat_moved_robber_to_me, 20);
-}
-
-static void greedy_player_robbed(G_GNUC_UNUSED gint robber_num,
-				 gint victim_num,
-				 G_GNUC_UNUSED Resource resource)
-{
-	if (victim_num == my_player_num())
-		randchat(chat_stole_from_me, 15);
-}
-
-static void greedy_get_rolled_resources(gint player_num,
-					const gint * resources,
-					G_GNUC_UNUSED const gint * wanted)
-{
-	gint total = 0, i;
-	for (i = 0; i < NO_RESOURCE; ++i)
-		total += resources[i];
-	if (player_num == my_player_num()) {
-		if (total == 1)
-			randchat(chat_receive_one, 60);
-		else if (total >= 3)
-			randchat(chat_receive_many, 20);
-	} else if (total >= 3)
-		randchat(chat_other_receive_many, 30);
-}
-
-static void greedy_played_develop(gint player_num,
-				  G_GNUC_UNUSED gint card_idx,
-				  DevelType type)
-{
-	if (player_num != my_player_num() && type == DEVEL_MONOPOLY)
-		randchat(chat_monopoly_other, 20);
-}
-
-static void greedy_new_statistics(gint player_num, StatisticType type,
-				  gint num)
-{
-	if (num != 1)
-		return;
-	if (type == STAT_LONGEST_ROAD) {
-		if (player_num == my_player_num())
-			randchat(chat_longestroad_self, 10);
-		else
-			randchat(chat_longestroad_other, 10);
-	} else if (type == STAT_LARGEST_ARMY) {
-		if (player_num == my_player_num())
-			randchat(chat_largestarmy_self, 10);
-		else
-			randchat(chat_largestarmy_other, 10);
-	}
 }
 
 void greedy_init(void)
@@ -2100,16 +1877,5 @@ void greedy_init(void)
 	callbacks.discard_add = &greedy_discard_add;
 	callbacks.quote_start = &greedy_quote_start;
 	callbacks.quote = &greedy_consider_quote;
-	callbacks.game_over = &greedy_game_over;
-	callbacks.error = &greedy_error;
-
-	/* chatting */
-	callbacks.player_turn = &greedy_player_turn;
-	callbacks.robber_moved = &greedy_robber_moved;
-	callbacks.discard = &greedy_discard_start;
 	callbacks.gold_choose = &greedy_gold_choose;
-	callbacks.player_robbed = &greedy_player_robbed;
-	callbacks.get_rolled_resources = &greedy_get_rolled_resources;
-	callbacks.played_develop = &greedy_played_develop;
-	callbacks.new_statistics = &greedy_new_statistics;
 }
