@@ -37,8 +37,8 @@
 #include "avahi.h"
 #include "avahi-browser.h"
 #include "client.h"
-#include "gtkcompat.h"
 #include "common_gtk.h"
+#include "game-list.h"
 
 const int PRIVATE_GAME_HISTORY_SIZE = 10;
 
@@ -50,7 +50,7 @@ static gchar *connect_port;	/* Port of the server */
 static GtkWidget *connect_dlg;	/* Dialog for starting a new game */
 static GtkWidget *name_entry;	/* Name of the player */
 static GtkWidget *spectator_toggle;	/* Prefer to be a spectator */
-static GtkWidget *meta_server_entry;	/* Name of the metaserver */
+static GtkWidget *metaserver_entry;	/* Name of the metaserver */
 
 static GtkWidget *meta_dlg;	/* Dialog for joining a public game */
 static GtkWidget *server_status;	/* Description of the current metaserver */
@@ -76,8 +76,6 @@ enum {				/* The columns of the meta_games_model */
 	META_N_COLUMNS
 };
 
-static Session *ses;
-
 static GtkWidget *cserver_dlg;	/* Dialog for creating a public game */
 static GtkWidget *select_game;	/* select game type */
 static GtkWidget *game_settings;	/* game settings widget */
@@ -85,7 +83,9 @@ static GtkWidget *aiplayers_spin;	/* number of AI players */
 static GtkWidget *game_rules;	/* Adjust some rules */
 
 static gboolean cfg_terrain;	/* Random terrain */
-static guint cfg_num_players, cfg_victory_points, cfg_sevens_rule;
+static guint cfg_num_players;
+static guint cfg_victory_points;
+static guint cfg_sevens_rule;
 static guint cfg_ai_players;
 static const gchar *cfg_gametype;	/* Will be set be the widget */
 
@@ -100,19 +100,21 @@ static enum {
 
 static enum {
 	CREATE_MODE_SIGNON,
-	CREATE_MODE_WAIT_FOR_INFO
+	CREATE_MODE_WAIT_FOR_INFO,
+	CREATE_MODE_STARTING_OK,
+	CREATE_MODE_STARTING_ERROR
 } create_mode;
 
 static enum {
 	MODE_SIGNON,
 	MODE_REDIRECT,
 	MODE_LIST,
-	MODE_CAPABILITY
+	MODE_CAPABILITY,
+	MODE_REDIRECT_OVERFLOW,
+	MODE_DONE,
+	MODE_SERVER_INFO
 } meta_mode;
 
-static Session *gametype_ses;
-static Session *create_ses;
-
 /** Information about the metaserver */
 static struct {
 	/** Server name */
@@ -127,10 +129,14 @@ static struct {
 	gint num_redirects;
 	/** The metaserver can create remote games */
 	gboolean can_create_games;
-	/** The metaserver can send information about a game */
-	gboolean can_send_game_settings;
+	/** Active session */
+	Session *session;
+	/** Number of available game titles */
+	guint num_available_titles;
+	/** The settings of a game */
+	GameParams *params;
 } metaserver_info = {
-NULL, NULL, 0, 0, 0, FALSE, FALSE};
+NULL, NULL, 0, 0, 0, FALSE, NULL, 0u, NULL};
 
 #define STRARG_LEN 128
 #define INTARG_LEN 16
@@ -145,7 +151,7 @@ static gchar *server_sevenrule = NULL;
 static gchar *server_terrain = NULL;
 static gchar server_title[STRARG_LEN];
 
-static void query_meta_server(const gchar * server, const gchar * port);
+static void query_metaserver(const gchar * server, const gchar * port);
 static void show_waiting_box(const gchar * message, const gchar * server,
 			     const gchar * port);
 static void close_waiting_box(void);
@@ -153,16 +159,19 @@ static void close_waiting_box(void);
 static void connect_set_field(gchar ** field, const gchar * value);
 static void connect_close_all(gboolean user_pressed_ok,
 			      gboolean can_be_spectator);
-static void set_meta_serverinfo(void);
+static void set_metaserver_info(void);
 static void connect_private_dialog(G_GNUC_UNUSED GtkWidget * widget,
 				   GtkWindow * parent);
 
 static void connect_set_field(gchar ** field, const gchar * value)
 {
 	gchar *temp = g_strdup(value);
-	if (*field)
-		g_free(*field);
-	*field = g_strdup(g_strstrip(temp));
+	g_free(*field);
+	if (temp != NULL) {
+		*field = g_strdup(g_strstrip(temp));
+	} else {
+		*field = NULL;
+	}
 	g_free(temp);
 }
 
@@ -172,7 +181,7 @@ static void connect_dlg_destroyed(GtkWidget * widget,
 {
 	name_entry = NULL;
 	spectator_toggle = NULL;
-	meta_server_entry = NULL;
+	metaserver_entry = NULL;
 	gtk_widget_destroyed(widget, widget_pointer);
 }
 
@@ -210,18 +219,18 @@ void connect_set_server(const gchar * server)
 	connect_set_field(&connect_server, server);
 }
 
-static gchar *connect_get_meta_server(void)
+static gchar *connect_get_metaserver(void)
 {
-	if (meta_server_entry == NULL)
+	if (metaserver_entry == NULL)
 		return NULL;
-	return metaserver_get(METASERVER(meta_server_entry));
+	return metaserver_get(METASERVER(metaserver_entry));
 }
 
-void connect_set_meta_server(const gchar * meta_server)
+void connect_set_metaserver(const gchar * metaserver)
 {
-	connect_set_field(&metaserver_info.server, meta_server);
-	if (meta_server_entry != NULL)
-		metaserver_add(METASERVER(meta_server_entry),
+	connect_set_field(&metaserver_info.server, metaserver);
+	if (metaserver_entry != NULL)
+		metaserver_add(METASERVER(metaserver_entry),
 			       metaserver_info.server);
 }
 
@@ -240,7 +249,7 @@ static void connect_close_all(gboolean user_pressed_ok,
 {
 	connect_spectator_allowed = can_be_spectator;
 	if (user_pressed_ok) {
-		gchar *meta_server;
+		gchar *metaserver;
 
 		notifying_string_set(requested_name,
 				     gtk_entry_get_text(GTK_ENTRY
@@ -249,9 +258,9 @@ static void connect_close_all(gboolean user_pressed_ok,
 		    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
 						 (spectator_toggle));
 		/* Save connect dialogue entries */
-		meta_server = connect_get_meta_server();
-		config_set_string("connect/meta-server", meta_server);
-		g_free(meta_server);
+		metaserver = connect_get_metaserver();
+		config_set_string("connect/metaserver", metaserver);
+		g_free(metaserver);
 
 		frontend_gui_register_destroy(connect_dlg,
 					      GUI_CONNECT_TRY);
@@ -274,7 +283,7 @@ static void show_waiting_box(const gchar * message, const gchar * server,
 			     const gchar * port)
 {
 	if (meta_dlg) {
-		gchar *s = g_strdup_printf(_("Meta-server at %s, port %s"),
+		gchar *s = g_strdup_printf(_("Metaserver at %s, port %s"),
 					   server, port);
 		gtk_label_set_text(GTK_LABEL(server_status), s);
 		g_free(s);
@@ -289,80 +298,170 @@ static void close_waiting_box(void)
 
 /* -------------------- get game types -------------------- */
 
-static void meta_gametype_notify(NetEvent event,
-				 G_GNUC_UNUSED void *user_data, char *line)
+static void meta_gametype_notify(Session * ses, NetEvent event,
+				 const gchar * line,
+				 G_GNUC_UNUSED gpointer user_data)
 {
 	switch (event) {
 	case NET_CONNECT:
+		metaserver_info.num_available_titles = 0u;
 		break;
 	case NET_CONNECT_FAIL:
-		net_free(&gametype_ses);
+		log_message(MSG_ERROR,
+			    _("The metaserver is not available "
+			      "anymore.\n"));
+		net_free(&ses);
+		metaserver_info.session = NULL;
+		if (cserver_dlg != NULL) {
+			gtk_widget_destroy(GTK_WIDGET(cserver_dlg));
+		}
 		break;
 	case NET_CLOSE:
-		if (gametype_mode == GAMETYPE_MODE_SIGNON)
+		if (gametype_mode == GAMETYPE_MODE_SIGNON) {
 			log_message(MSG_ERROR,
-				    _("Meta-server kicked us off\n"));
-		net_free(&gametype_ses);
-		close_waiting_box();
+				    _("The metaserver closed "
+				      "the connection "
+				      "unexpectedly.\n"));
+			if (cserver_dlg != NULL) {
+				gtk_widget_destroy(GTK_WIDGET
+						   (cserver_dlg));
+			}
+		} else {
+			close_waiting_box();
+		}
+		if (cserver_dlg != NULL
+		    && metaserver_info.num_available_titles > 0u) {
+			gtk_dialog_set_response_sensitive(GTK_DIALOG
+							  (cserver_dlg),
+							  GTK_RESPONSE_OK,
+							  TRUE);
+		}
+		net_free(&ses);
+		metaserver_info.session = NULL;
 		break;
 	case NET_READ:
 		switch (gametype_mode) {
 		case GAMETYPE_MODE_SIGNON:
-			net_printf(gametype_ses, "listtypes\n");
+			net_printf(ses, "listtypes\n");
 			gametype_mode = GAMETYPE_MODE_LIST;
 			break;
 		case GAMETYPE_MODE_LIST:
 			/* A server description looks like this:
 			 * title=%s\n
 			 */
-			if (strncmp(line, "title=", 6) == 0)
-				select_game_add(SELECTGAME(select_game),
-						line + 6);
+			if (strncmp(line, "title=", 6) == 0) {
+				const GameParams *params;
+
+				line += 6;
+				if (game_list_is_empty()) {
+					game_list_prepare();
+				}
+				params = game_list_find_item(line);
+				if (params != NULL) {
+					select_game_add_details(SELECTGAME
+								(select_game),
+								params);
+				} else {
+					select_game_add(SELECTGAME
+							(select_game),
+							line);
+				}
+				metaserver_info.num_available_titles++;
+			}
 			break;
 		}
 		break;
 	}
 }
 
-static void get_meta_server_games_types(gchar * server, gchar * port)
+static void get_metaserver_games_types(const gchar * server,
+				       const gchar * port)
 {
-	show_waiting_box(_("Receiving game names from the meta server.\n"),
+	show_waiting_box(_("Receiving game names from the metaserver.\n"),
 			 server, port);
-	gametype_ses = net_new(meta_gametype_notify, NULL);
-	if (net_connect(gametype_ses, server, port))
+	g_assert(metaserver_info.session == NULL);
+	metaserver_info.session = net_new(meta_gametype_notify, NULL);
+	if (net_connect(metaserver_info.session, server, port))
 		gametype_mode = GAMETYPE_MODE_SIGNON;
 	else {
-		net_free(&gametype_ses);
-		close_waiting_box();
+		net_free(&metaserver_info.session);
+		log_message(MSG_ERROR,
+			    _("The metaserver is not available "
+			      "anymore.\n"));
+		if (cserver_dlg != NULL) {
+			gtk_widget_destroy(GTK_WIDGET(cserver_dlg));
+		}
 	}
 }
 
 /* -------------------- create game server -------------------- */
 
-static void meta_create_notify(NetEvent event,
-			       G_GNUC_UNUSED void *user_data, char *line)
+static void meta_create_notify(Session * ses, NetEvent event,
+			       const gchar * line,
+			       G_GNUC_UNUSED gpointer user_data)
 {
 	switch (event) {
 	case NET_CONNECT:
 		break;
 	case NET_CONNECT_FAIL:
-		net_free(&create_ses);
+		log_message(MSG_ERROR,
+			    _("The metaserver is not available "
+			      "anymore.\n"));
+		net_free(&ses);
+		metaserver_info.session = NULL;
+		if (cserver_dlg != NULL) {
+			gtk_widget_destroy(GTK_WIDGET(cserver_dlg));
+		}
 		break;
 	case NET_CLOSE:
-		if (create_mode == CREATE_MODE_SIGNON)
+		net_free(&ses);
+		metaserver_info.session = NULL;
+		switch (create_mode) {
+		case CREATE_MODE_STARTING_OK:
+			log_message(MSG_INFO,
+				    _("New game server requested "
+				      "on %s port %s.\n"),
+				    connect_server, connect_port);
+			/* The metaserver is now busy creating the new game.
+			 * UGLY FIX: Wait for some time */
+			g_usleep(500000);
+			connect_close_all(TRUE, FALSE);
+			break;
+		case CREATE_MODE_STARTING_ERROR:
 			log_message(MSG_ERROR,
-				    _("Meta-server kicked us off\n"));
-		net_free(&create_ses);
+				    _("Incomplete information about the "
+				      "new game server received.\n"));
+			if (cserver_dlg != NULL) {
+				gtk_dialog_set_response_sensitive
+				    (GTK_DIALOG(cserver_dlg),
+				     GTK_RESPONSE_OK, TRUE);
+			}
+			break;
+		case CREATE_MODE_SIGNON:
+		case CREATE_MODE_WAIT_FOR_INFO:
+			log_message(MSG_ERROR,
+				    _("The metaserver closed "
+				      "the connection "
+				      "unexpectedly.\n"));
+			if (cserver_dlg != NULL) {
+				gtk_dialog_set_response_sensitive
+				    (GTK_DIALOG(cserver_dlg),
+				     GTK_RESPONSE_OK, TRUE);
+			}
+			break;
+		}
 		break;
 	case NET_READ:
 		switch (create_mode) {
 		case CREATE_MODE_SIGNON:
-			net_printf(create_ses,
+			net_printf(ses,
 				   "create %d %d %d %d %d %s\n",
 				   cfg_terrain, cfg_num_players,
 				   cfg_victory_points, cfg_sevens_rule,
 				   cfg_ai_players, cfg_gametype);
 			create_mode = CREATE_MODE_WAIT_FOR_INFO;
+			connect_set_field(&connect_server, NULL);
+			connect_set_field(&connect_port, NULL);
 			break;
 
 		case CREATE_MODE_WAIT_FOR_INFO:
@@ -372,19 +471,26 @@ static void meta_create_notify(NetEvent event,
 			else if (strncmp(line, "port=", 5) == 0)
 				connect_set_field(&connect_port, line + 5);
 			else if (strcmp(line, "started") == 0) {
-				log_message(MSG_INFO,
-					    _(""
-					      "New game server requested on %s port %s\n"),
-					    connect_server, connect_port);
-				/* The meta server is now busy creating the new game.
-				 * UGLY FIX: Wait for some time */
-				g_usleep(500000);
-				connect_close_all(TRUE, FALSE);
+				if (connect_get_server() != NULL
+				    && connect_get_port() != NULL) {
+					create_mode =
+					    CREATE_MODE_STARTING_OK;
+				} else {
+					create_mode =
+					    CREATE_MODE_STARTING_ERROR;
+				}
+				net_close(ses);
+				metaserver_info.session = NULL;
 			} else
 				log_message(MSG_ERROR,
-					    _(""
-					      "Unknown message from the metaserver: %s\n"),
-					    line);
+					    _("Unknown message from the "
+					      "metaserver: %s\n"), line);
+			break;
+		case CREATE_MODE_STARTING_OK:
+		case CREATE_MODE_STARTING_ERROR:
+			log_message(MSG_ERROR,
+				    _("Unknown message from the "
+				      "metaserver: %s\n"), line);
 			break;
 		}
 	}
@@ -395,7 +501,7 @@ static void meta_create_notify(NetEvent event,
 static gboolean check_str_info(const gchar * line, const gchar * prefix,
 			       gchar * data)
 {
-	gint len = strlen(prefix);
+	size_t len = strlen(prefix);
 
 	if (strncmp(line, prefix, len) != 0)
 		return FALSE;
@@ -406,7 +512,7 @@ static gboolean check_str_info(const gchar * line, const gchar * prefix,
 static gboolean check_int_info(const gchar * line, const gchar * prefix,
 			       gchar * data)
 {
-	gint len = strlen(prefix);
+	size_t len = strlen(prefix);
 
 	if (strncmp(line, prefix, len) != 0)
 		return FALSE;
@@ -436,74 +542,77 @@ static void server_end(void)
 	}
 }
 
-static void meta_notify(NetEvent event, G_GNUC_UNUSED void *user_data,
-			char *line)
+static void meta_free_session(Session * ses)
+{
+	if (ses == metaserver_info.session) {
+		metaserver_info.session = NULL;
+	}
+	net_free(&ses);
+}
+
+/* Developer note: very similar code exists in server/meta.c
+ * Keep both routines up-to-date
+ */
+static void meta_notify(Session * ses,
+			NetEvent event, const gchar * line,
+			G_GNUC_UNUSED gpointer user_data)
 {
 	gchar argument[STRARG_LEN];
-
 	switch (event) {
-	case NET_CONNECT:
-		break;
-	case NET_CONNECT_FAIL:	/* Can't connect to the metaserver, don't show the GUI */
-		if (meta_dlg)
-			gtk_widget_destroy(GTK_WIDGET(meta_dlg));	/* Close the dialog */
-		break;
-	case NET_CLOSE:
-		if (meta_mode == MODE_SIGNON)
-			log_message(MSG_ERROR,
-				    _("Meta-server kicked us off\n"));
-		close_waiting_box();
-		net_free(&ses);
-		break;
 	case NET_READ:
+		if (ses != metaserver_info.session) {
+			log_message(MSG_ERROR,
+				    _("Receiving data from inactive "
+				      "session: %s\n"), line);
+			return;
+		}
 		switch (meta_mode) {
 		case MODE_SIGNON:
 		case MODE_REDIRECT:
 			if (strncmp(line, "goto ", 5) == 0) {
 				gchar **split_result;
 				const gchar *port;
-				meta_mode = MODE_REDIRECT;
-				net_close(ses);
-				if (metaserver_info.num_redirects++ == 10) {
-					log_message(MSG_INFO,
-						    _(""
-						      "Too many meta-server redirects\n"));
+				if (metaserver_info.num_redirects++ >= 10) {
+					log_message(MSG_ERROR,
+						    _("Too many "
+						      "metaserver "
+						      "redirects.\n"));
+					meta_mode = MODE_REDIRECT_OVERFLOW;
+					net_close(ses);
 					return;
 				}
+				meta_mode = MODE_REDIRECT;
+				meta_free_session(ses);
 				split_result = g_strsplit(line, " ", 0);
 				g_assert(split_result[0] != NULL);
 				g_assert(!strcmp(split_result[0], "goto"));
 				if (split_result[1]) {
-					if (metaserver_info.server)
-						g_free
-						    (metaserver_info.
-						     server);
+					g_free(metaserver_info.server);
+					g_free(metaserver_info.port);
+
 					metaserver_info.server =
 					    g_strdup(split_result[1]);
-					if (metaserver_info.port)
-						g_free
-						    (metaserver_info.port);
+
 					port = PIONEERS_DEFAULT_META_PORT;
 					if (split_result[2])
 						port = split_result[2];
 					metaserver_info.port =
 					    g_strdup(port);
-					query_meta_server
+					query_metaserver
 					    (metaserver_info.server,
 					     metaserver_info.port);
 				} else {
 					log_message(MSG_ERROR,
-						    _(""
-						      "Bad redirect line: %s\n"),
-						    line);
+						    _("Bad redirect line: "
+						      "%s\n"), line);
 				};
 				g_strfreev(split_result);
 				break;
 			}
-			metaserver_info.version_major =
-			    metaserver_info.version_minor = 0;
+
+			metaserver_info.version_major = 0;
+			metaserver_info.version_minor = 0;
 			metaserver_info.can_create_games = FALSE;
-			metaserver_info.can_send_game_settings = FALSE;
 			if (strncmp(line, "welcome ", 8) == 0) {
 				char *p = strstr(line, "version ");
 				if (p) {
@@ -519,9 +628,9 @@ static void meta_notify(NetEvent event, G_GNUC_UNUSED void *user_data,
 			}
 			if (metaserver_info.version_major < 1) {
 				log_message(MSG_INFO,
-					    _(""
-					      "Meta server too old to create "
-					      "servers (version %d.%d)\n"),
+					    _("The metaserver is too old "
+					      "to create servers "
+					      "(version %d.%d)\n"),
 					    metaserver_info.version_major,
 					    metaserver_info.version_minor);
 			} else {
@@ -541,22 +650,11 @@ static void meta_notify(NetEvent event, G_GNUC_UNUSED void *user_data,
 					   "client\n");
 				meta_mode = MODE_LIST;
 			}
-			gtk_dialog_set_response_sensitive(GTK_DIALOG
-							  (meta_dlg),
-							  META_RESPONSE_REFRESH,
-							  TRUE);
 			break;
 		case MODE_CAPABILITY:
 			if (!strcmp(line, "create games")) {
 				metaserver_info.can_create_games = TRUE;
-			} else if (!strcmp(line, "send game settings")) {
-				metaserver_info.can_send_game_settings =
-				    TRUE;
 			} else if (!strcmp(line, "end")) {
-				gtk_dialog_set_response_sensitive
-				    (GTK_DIALOG(meta_dlg),
-				     META_RESPONSE_NEW,
-				     metaserver_info.can_create_games);
 				net_printf(ses,
 					   metaserver_info.version_major >=
 					   1 ? "listservers\n" :
@@ -565,9 +663,18 @@ static void meta_notify(NetEvent event, G_GNUC_UNUSED void *user_data,
 			}
 			break;
 		case MODE_LIST:
-			if (strcmp(line, "server") == 0);	/* Information will come shortly */
-			else if (strcmp(line, "end") == 0) {
+			if (strcmp(line, "server") == 0) {
+				meta_mode = MODE_SERVER_INFO;
+			} else {
+				log_message(MSG_ERROR,
+					    _("Unexpected data from the "
+					      "metaserver: %s\n"), line);
+			}
+			break;
+		case MODE_SERVER_INFO:
+			if (strcmp(line, "end") == 0) {
 				server_end();
+				meta_mode = MODE_LIST;
 			} else
 			    if (check_str_info(line, "host=", server_host))
 				break;
@@ -637,28 +744,101 @@ static void meta_notify(NetEvent event, G_GNUC_UNUSED void *user_data,
 			else if (check_str_info
 				 (line, "comment=", server_title))
 				break;
+			else
+				log_message(MSG_ERROR,
+					    _("Unexpected data from the "
+					      "metaserver: %s\n"), line);
+			break;
+		case MODE_DONE:
+		case MODE_REDIRECT_OVERFLOW:
+			log_message(MSG_ERROR,
+				    _("Unexpected data from the "
+				      "metaserver: %s\n"), line);
 			break;
 		}
 		break;
+	case NET_CLOSE:
+		/* During a reconnect, different sessions might co-exist */
+		if (ses == metaserver_info.session) {
+			metaserver_info.session = NULL;
+			switch (meta_mode) {
+			case MODE_SIGNON:
+			case MODE_REDIRECT:
+			case MODE_SERVER_INFO:
+			case MODE_CAPABILITY:
+				log_message(MSG_ERROR,
+					    _("The metaserver closed "
+					      "the connection "
+					      "unexpectedly.\n"));
+				break;
+			case MODE_REDIRECT_OVERFLOW:
+				if (meta_dlg)
+					gtk_widget_destroy(GTK_WIDGET(meta_dlg));	/* Close the dialog */
+				if (ses == metaserver_info.session) {
+					metaserver_info.session = NULL;
+				}
+				break;
+			case MODE_LIST:
+			case MODE_DONE:
+				close_waiting_box();
+				break;
+			}
+			if (meta_dlg) {
+				gtk_dialog_set_response_sensitive
+				    (GTK_DIALOG(meta_dlg),
+				     META_RESPONSE_NEW,
+				     metaserver_info.can_create_games);
+				gtk_dialog_set_response_sensitive
+				    (GTK_DIALOG(meta_dlg),
+				     META_RESPONSE_REFRESH, TRUE);
+			}
+		}
+		net_free(&ses);
+		break;
+	case NET_CONNECT:
+		break;
+	case NET_CONNECT_FAIL:
+		/* Can't connect to the metaserver, don't show the GUI */
+		if (meta_dlg)
+			gtk_widget_destroy(GTK_WIDGET(meta_dlg));
+		if (ses == metaserver_info.session) {
+			metaserver_info.session = NULL;
+		}
+		net_free(&ses);
+		break;
 	}
 }
 
-static void query_meta_server(const gchar * server, const gchar * port)
+static void query_metaserver(const gchar * server, const gchar * port)
 {
-	if (metaserver_info.num_redirects > 0)
-		log_message(MSG_INFO,
-			    _(""
-			      "Redirected to meta-server at %s, port %s\n"),
-			    server, port);
-	show_waiting_box(_(""
-			   "Receiving a list of Pioneers servers from the meta server.\n"),
-			 server, port);
+	gchar *message;
 
-	ses = net_new(meta_notify, NULL);
-	if (net_connect(ses, server, port))
+	if (metaserver_info.num_redirects > 0) {
+		if (strcmp(port, PIONEERS_DEFAULT_META_PORT) == 0) {
+			message =
+			    g_strdup_printf(_("Redirected to the "
+					      "metaserver at %s.\n"),
+					    server);
+		} else {
+			message =
+			    g_strdup_printf(_("Redirected to the "
+					      "metaserver at %s, port %s.\n"),
+					    server, port);
+		}
+	} else
+		message =
+		    g_strdup(_
+			     ("Receiving a list of Pioneers servers "
+			      "from the metaserver.\n"));
+	show_waiting_box(message, server, port);
+	g_free(message);
+
+	g_assert(metaserver_info.session == NULL);
+	metaserver_info.session = net_new(meta_notify, NULL);
+	if (net_connect(metaserver_info.session, server, port))
 		meta_mode = MODE_SIGNON;
 	else {
-		net_free(&ses);
+		net_free(&metaserver_info.session);
 		close_waiting_box();
 	}
 }
@@ -674,43 +854,51 @@ static void player_change_cb(GameSettings * gs,
 
 	ai_spin = GTK_SPIN_BUTTON(aiplayers_spin);
 	players = game_settings_get_players(gs);
-	ai_players = gtk_spin_button_get_value_as_int(ai_spin);
+	ai_players = (guint) gtk_spin_button_get_value_as_int(ai_spin);
 	gtk_spin_button_set_range(ai_spin, 0, players - 1);
 	if (ai_players >= players)
 		gtk_spin_button_set_value(ai_spin, players - 1);
 }
 
+static void game_select_cb(SelectGame * sg,
+			   G_GNUC_UNUSED gpointer user_data)
+{
+	const GameParams *params;
+
+	params = select_game_get_active_game(sg);
+
+	if (params != NULL) {
+		game_settings_set_players(GAMESETTINGS(game_settings),
+					  params->num_players);
+		game_settings_set_victory_points(GAMESETTINGS
+						 (game_settings),
+						 params->victory_points);
+		game_rules_set_random_terrain(GAMERULES(game_rules),
+					      params->random_terrain);
+		game_rules_set_sevens_rule(GAMERULES(game_rules),
+					   params->sevens_rule);
+
+		player_change_cb(GAMESETTINGS(game_settings), NULL);
+	}
+}
 
 static GtkWidget *build_create_interface(void)
 {
 	GtkWidget *vbox;
 	GtkWidget *label;
-	GtkObject *adj;
+	GtkAdjustment *adj;
 	guint row;
 
-	vbox = gtk_vbox_new(FALSE, 0);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	gtk_widget_show(vbox);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 3);
 
 	select_game = select_game_new();
 	gtk_widget_show(select_game);
 	gtk_box_pack_start(GTK_BOX(vbox), select_game, FALSE, FALSE, 3);
-	/* The meta server does not send information about the game,
-	 * so don't adjust the game settings when another game is chosen.
-	 g_signal_connect(G_OBJECT(select_game),
-	 "activate",
-	 G_CALLBACK(game_select_cb), NULL);
-	 */
 
-	if (!metaserver_info.can_send_game_settings) {
-		label = gtk_label_new(NULL);
-		gtk_label_set_markup(GTK_LABEL(label),
-				     _("<b>Note</b>:\n"
-				       "\tThe metaserver does not send information about the games.\n"
-				       "\tPlease set appropriate values yourself."));
-		gtk_widget_show(label);
-		gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
-	}
+	g_signal_connect(G_OBJECT(select_game), "activate",
+			 G_CALLBACK(game_select_cb), NULL);
 
 	game_settings = game_settings_new(FALSE);
 	gtk_widget_show(game_settings);
@@ -729,12 +917,13 @@ static GtkWidget *build_create_interface(void)
 			 row + 1, GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 
-	adj = gtk_adjustment_new(0,
-				 0,
-				 game_settings_get_players(GAMESETTINGS
-							   (game_settings))
-				 - 1, 1, 4, 0);
-	aiplayers_spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1, 0);
+	adj = GTK_ADJUSTMENT(gtk_adjustment_new(0,
+						0,
+						game_settings_get_players
+						(GAMESETTINGS
+						 (game_settings))
+						- 1, 1, 4, 0));
+	aiplayers_spin = gtk_spin_button_new(adj, 1, 0);
 	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(aiplayers_spin), TRUE);
 	gtk_widget_show(aiplayers_spin);
 	gtk_entry_set_alignment(GTK_ENTRY(aiplayers_spin), 1.0);
@@ -749,9 +938,6 @@ static GtkWidget *build_create_interface(void)
 	gtk_widget_show(game_rules);
 	gtk_box_pack_start(GTK_BOX(vbox), game_rules, FALSE, FALSE, 3);
 
-	get_meta_server_games_types(metaserver_info.server,
-				    metaserver_info.port);
-
 	return vbox;
 }
 
@@ -764,28 +950,47 @@ static void create_server_dlg_cb(GtkDialog * dlg, gint arg1,
 
 	switch (arg1) {
 	case GTK_RESPONSE_OK:
-		log_message(MSG_INFO, _("Requesting new game server\n"));
+		gtk_dialog_set_response_sensitive(dlg, GTK_RESPONSE_OK,
+						  FALSE);
+
+		if (metaserver_info.session != NULL) {
+			log_message(MSG_INFO, _("Canceled.\n"));
+			net_close(metaserver_info.session);
+		}
+		log_message(MSG_INFO, _("Requesting new game server.\n"));
 
 		cfg_terrain = game_rules_get_random_terrain(gr);
 		cfg_num_players = game_settings_get_players(gs);
 		cfg_victory_points = game_settings_get_victory_points(gs);
 		cfg_sevens_rule = game_rules_get_sevens_rule(gr);
-		cfg_ai_players =
+		cfg_ai_players = (guint)
 		    gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON
 						     (aiplayers_spin));
-		cfg_gametype = select_game_get_active(sg);
+		cfg_gametype = select_game_get_active_title(sg);
 
-		create_ses = net_new(meta_create_notify, NULL);
+		metaserver_info.session =
+		    net_new(meta_create_notify, NULL);
 		if (net_connect
-		    (create_ses, metaserver_info.server,
-		     metaserver_info.port))
+		    (metaserver_info.session, metaserver_info.server,
+		     metaserver_info.port)) {
 			create_mode = CREATE_MODE_SIGNON;
-		else
-			net_free(&create_ses);
+		} else {
+			log_message(MSG_ERROR,
+				    _("The metaserver is not available "
+				      "anymore.\n"));
+			net_free(&metaserver_info.session);
+			gtk_widget_destroy(GTK_WIDGET(dlg));
+		}
 		break;
 	case GTK_RESPONSE_CANCEL:
 	default:		/* For the compiler */
 		gtk_widget_destroy(GTK_WIDGET(dlg));
+		if (metaserver_info.session != NULL) {
+			net_close(metaserver_info.session);
+			/* Canceled retrieving information 
+			 * from the metaserver */
+			log_message(MSG_INFO, _("Canceled.\n"));
+		}
 		break;
 	};
 }
@@ -796,19 +1001,19 @@ static void launch_server_gtk(G_GNUC_UNUSED GtkWidget * widget,
 {
 	gchar *child_argv[3];
 	GSpawnFlags flags = G_SPAWN_STDOUT_TO_DEV_NULL |
-	    G_SPAWN_STDERR_TO_DEV_NULL;
+	    G_SPAWN_STDERR_TO_DEV_NULL | G_SPAWN_SEARCH_PATH;
 	GError *error = NULL;
 	gint i;
 
-	child_argv[0] = g_strdup(PIONEERS_SERVER_GTK_PATH);
-	child_argv[1] = g_strdup(PIONEERS_SERVER_GTK_PATH);
+	child_argv[0] = g_strdup(PIONEERS_SERVER_GTK_PROGRAM_NAME);
+	child_argv[1] = g_strdup(PIONEERS_SERVER_GTK_PROGRAM_NAME);
 	child_argv[2] = NULL;
 	if (!g_spawn_async(NULL, child_argv, NULL, flags, NULL, NULL, NULL,
 			   &error)) {
 		/* Error message when program %1 is started, reason is %2 */
 		log_message(MSG_ERROR,
 			    _("Error starting %s: %s\n"),
-			    PIONEERS_SERVER_GTK_PATH, error->message);
+			    child_argv[0], error->message);
 		g_error_free(error);
 	}
 	for (i = 0; child_argv[i] != NULL; i++)
@@ -825,7 +1030,7 @@ static void create_server_dlg(G_GNUC_UNUSED GtkWidget * widget,
 		gtk_window_present(GTK_WINDOW(cserver_dlg));
 		return;
 	}
-	set_meta_serverinfo();
+	set_metaserver_info();
 
 	cserver_dlg =
 	    /* Dialog caption */
@@ -846,9 +1051,17 @@ static void create_server_dlg(G_GNUC_UNUSED GtkWidget * widget,
 	gtk_box_pack_start(GTK_BOX(dlg_vbox), vbox, TRUE, TRUE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
 
+	if (cserver_dlg != NULL) {
+		gtk_dialog_set_response_sensitive(GTK_DIALOG(cserver_dlg),
+						  GTK_RESPONSE_OK, FALSE);
+	}
 	g_signal_connect(G_OBJECT(cserver_dlg), "response",
 			 G_CALLBACK(create_server_dlg_cb), NULL);
 	gtk_widget_show(cserver_dlg);
+
+	get_metaserver_games_types(metaserver_info.server,
+				   metaserver_info.port);
+
 }
 
 /* -------------------- select server dialog -------------------- */
@@ -883,8 +1096,8 @@ static void meta_dlg_cb(GtkDialog * dlg, gint arg1,
 	case META_RESPONSE_REFRESH:	/* Refresh the list */
 		gtk_list_store_clear(meta_games_model);
 		metaserver_info.num_redirects = 0;
-		query_meta_server(metaserver_info.server,
-				  metaserver_info.port);
+		query_metaserver(metaserver_info.server,
+				 metaserver_info.port);
 		break;
 	case META_RESPONSE_NEW:	/* Add a server */
 		create_server_dlg(NULL, GTK_WINDOW(dlg));
@@ -911,19 +1124,26 @@ static void meta_dlg_cb(GtkDialog * dlg, gint arg1,
 	case GTK_RESPONSE_CANCEL:	/* Cancel */
 	default:
 		gtk_widget_destroy(GTK_WIDGET(dlg));
+		if (metaserver_info.session != NULL) {
+			net_close(metaserver_info.session);
+			/* Canceled retrieving information 
+			 * from the metaserver */
+			log_message(MSG_INFO, _("Canceled.\n"));
+		}
 		break;
 	}
 }
 
-static void set_meta_serverinfo(void)
+static void set_metaserver_info(void)
 {
 	gchar *meta_tmp;
 
-	meta_tmp = metaserver_get(METASERVER(meta_server_entry));
+	g_free(metaserver_info.server);
+	g_free(metaserver_info.port);
+
+	meta_tmp = metaserver_get(METASERVER(metaserver_entry));
 	metaserver_info.server = meta_tmp;	/* Take-over of the pointer */
-	if (!metaserver_info.port)
-		metaserver_info.port =
-		    g_strdup(PIONEERS_DEFAULT_META_PORT);
+	metaserver_info.port = g_strdup(PIONEERS_DEFAULT_META_PORT);
 }
 
 static void create_meta_dlg(G_GNUC_UNUSED GtkWidget * widget,
@@ -935,18 +1155,23 @@ static void create_meta_dlg(G_GNUC_UNUSED GtkWidget * widget,
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *renderer;
 
-	set_meta_serverinfo();
+	set_metaserver_info();
 
 	if (meta_dlg != NULL) {
-		if (ses == NULL) {
+		if (metaserver_info.session == NULL) {
+			gtk_list_store_clear(meta_games_model);
 			metaserver_info.num_redirects = 0;
-			query_meta_server(metaserver_info.server,
-					  metaserver_info.port);
-		}
-		return;
-	}
+			metaserver_info.version_major = 0;
+			metaserver_info.version_minor = 0;
+			metaserver_info.can_create_games = FALSE;
+			gtk_dialog_set_response_sensitive
+			    (GTK_DIALOG(meta_dlg),
+			     META_RESPONSE_NEW,
+			     metaserver_info.can_create_games);
 
-	if (meta_dlg) {
+			query_metaserver(metaserver_info.server,
+					 metaserver_info.port);
+		}
 		gtk_window_present(GTK_WINDOW(meta_dlg));
 		return;
 	}
@@ -972,25 +1197,25 @@ static void create_meta_dlg(G_GNUC_UNUSED GtkWidget * widget,
 	gtk_widget_set_tooltip_text(gui_get_dialog_button
 				    (GTK_DIALOG(meta_dlg), 0),
 				    /* Tooltip */
-				    _("Refresh the list of games"));
+				    _("Join the selected game"));
 	gtk_widget_set_tooltip_text(gui_get_dialog_button
 				    (GTK_DIALOG(meta_dlg), 1),
 				    /* Tooltip */
-				    _(""
-				      "Create a new public game at the meta server"));
+				    _("Don't join a public game"));
 	gtk_widget_set_tooltip_text(gui_get_dialog_button
 				    (GTK_DIALOG(meta_dlg), 2),
 				    /* Tooltip */
-				    _("Don't join a public game"));
+				    _(""
+				      "Create a new public game at the metaserver"));
 	gtk_widget_set_tooltip_text(gui_get_dialog_button
 				    (GTK_DIALOG(meta_dlg), 3),
 				    /* Tooltip */
-				    _("Join the selected game"));
+				    _("Refresh the list of games"));
 
 	dlg_vbox = gtk_dialog_get_content_area(GTK_DIALOG(meta_dlg));
 	gtk_widget_show(dlg_vbox);
 
-	vbox = gtk_vbox_new(FALSE, 2);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 2);
 	gtk_widget_show(vbox);
 	gtk_box_pack_start(GTK_BOX(dlg_vbox), vbox, TRUE, TRUE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
@@ -1164,7 +1389,7 @@ static void create_meta_dlg(G_GNUC_UNUSED GtkWidget * widget,
 	gtk_widget_show(meta_dlg);
 
 	metaserver_info.num_redirects = 0;
-	query_meta_server(metaserver_info.server, metaserver_info.port);
+	query_metaserver(metaserver_info.server, metaserver_info.port);
 
 	/* Workaround: Set the size of the widget as late as possible, to avoid a strange display */
 	gtk_widget_set_size_request(scroll_win, -1, 150);
@@ -1192,7 +1417,7 @@ static void connect_avahi_cb(G_GNUC_UNUSED GtkWidget * widget,
 	g_free(server);
 	g_free(port);
 
-	// connect
+	/* connect */
 	connect_close_all(TRUE, TRUE);
 }
 #endif
@@ -1207,10 +1432,10 @@ void connect_create_dlg(void)
 	GtkWidget *sep;
 #ifdef HAVE_AVAHI
 	GtkWidget *avahibrowser_entry;
-#endif				// HAVE_AVAHI
+#endif				/* HAVE_AVAHI */
 	gchar *fullname;
 	gchar *name;
-	gint row;
+	guint row;
 
 	if (connect_dlg) {
 		gtk_window_present(GTK_WINDOW(connect_dlg));
@@ -1280,7 +1505,7 @@ void connect_create_dlg(void)
 				    ("Check if you want to be a spectator"));
 	row++;
 
-	sep = gtk_hseparator_new();
+	sep = gtk_separator_new(GTK_ORIENTATION_HORIZONTAL);
 	gtk_widget_show(sep);
 	gtk_table_attach(GTK_TABLE(table), sep, 0, 3, row, row + 1,
 			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 6);
@@ -1314,34 +1539,34 @@ void connect_create_dlg(void)
 	g_signal_connect(G_OBJECT(btn), "clicked",
 			 G_CALLBACK(connect_avahi_cb), avahibrowser_entry);
 
-	// enable avahi
-	// storing the pointer to this widget for later use
+	/* enable avahi */
+	/* storing the pointer to this widget for later use */
 	avahi_register(AVAHIBROWSER(avahibrowser_entry));
 
-	sep = gtk_hseparator_new();
+	sep = gtk_separator_new(GTK_ORIENTATION_HORIZONTAL);
 	gtk_widget_show(sep);
 	gtk_table_attach(GTK_TABLE(table), sep, 0, 3, row, row + 1,
 			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 6);
 	row++;
-#endif				// HAVE_AVAHI
+#endif				/* HAVE_AVAHI */
 
 	/* Label */
-	lbl = gtk_label_new(_("Meta server"));
+	lbl = gtk_label_new(_("Metaserver"));
 	gtk_widget_show(lbl);
 	gtk_table_attach(GTK_TABLE(table), lbl, 0, 1, row, row + 1,
 			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 	gtk_misc_set_alignment(GTK_MISC(lbl), 0, 0.5);
 
-	meta_server_entry = metaserver_new();
-	gtk_widget_show(meta_server_entry);
-	gtk_table_attach(GTK_TABLE(table), meta_server_entry, 1, 3, row,
+	metaserver_entry = metaserver_new();
+	gtk_widget_show(metaserver_entry);
+	gtk_table_attach(GTK_TABLE(table), metaserver_entry, 1, 3, row,
 			 row + 1, GTK_EXPAND | GTK_FILL,
 			 GTK_EXPAND | GTK_FILL, 0, 0);
-	metaserver_add(METASERVER(meta_server_entry),
+	metaserver_add(METASERVER(metaserver_entry),
 		       metaserver_info.server);
 	row++;
 
-	hbox = gtk_hbox_new(FALSE, 3);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 3);
 	gtk_widget_show(hbox);
 	gtk_table_attach(GTK_TABLE(table), hbox, 0, 3, row, row + 1,
 			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 3);
@@ -1366,12 +1591,10 @@ void connect_create_dlg(void)
 	gtk_widget_set_tooltip_text(btn, _("Create a game"));
 	g_signal_connect(G_OBJECT(btn), "clicked",
 			 G_CALLBACK(launch_server_gtk), app_window);
-	fullname = g_find_program_in_path(PIONEERS_SERVER_GTK_PATH);
-	if (fullname) {
-		g_free(fullname);
-	} else {
-		gtk_widget_set_sensitive(GTK_WIDGET(btn), FALSE);
-	}
+	fullname =
+	    g_find_program_in_path(PIONEERS_SERVER_GTK_PROGRAM_NAME);
+	gtk_widget_set_sensitive(GTK_WIDGET(btn), fullname != NULL);
+	g_free(fullname);
 
 	/* Button */
 	btn = gtk_button_new_with_label(_("Join Private Game"));
@@ -1583,7 +1806,7 @@ static void connect_private_dialog(G_GNUC_UNUSED GtkWidget * widget,
 			 GTK_FILL, GTK_EXPAND, 0, 0);
 	gtk_misc_set_alignment(GTK_MISC(lbl), 0, 0.5);
 
-	hbox = gtk_hbox_new(FALSE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	gtk_widget_show(hbox);
 	gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, 1, 2,
 			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
