@@ -116,7 +116,7 @@ GtkWidget *develop_build_page(void)
 	GtkWidget *play_develop_btn;
 	GtkWidget *develop_list;
 
-	vbox = gtk_vbox_new(FALSE, 5);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5);
 	gtk_widget_show(vbox);
 
 	alignment = gtk_alignment_new(0.0, 0.0, 1.0, 1.0);
@@ -201,7 +201,7 @@ GtkWidget *develop_build_page(void)
 
 	gtk_widget_show(develop_list);
 
-	bbox = gtk_hbutton_box_new();
+	bbox = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
 	gtk_widget_show(bbox);
 	gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, TRUE, 0);
 
@@ -262,7 +262,7 @@ void frontend_bought_develop(DevelType type)
 	update_model(&iter, type);
 }
 
-void frontend_played_develop(gint player_num, G_GNUC_UNUSED gint card_idx,
+void frontend_played_develop(gint player_num, G_GNUC_UNUSED guint card_idx,
 			     DevelType type)
 {
 	GtkTreeIter iter;
@@ -274,10 +274,14 @@ void frontend_played_develop(gint player_num, G_GNUC_UNUSED gint card_idx,
 					 DEVELOP_COLUMN_ORDER,
 					 develtype_to_sortorder(type));
 		g_assert(found == FIND_MATCH_EXACT);
-		if (deck_card_amount(get_devel_deck(), type) == 0)
+		if (deck_card_amount(get_devel_deck(), type) == 0) {
 			gtk_list_store_remove(store, &iter);
-		else
+			selected_card_idx = -1;
+		} else {
 			update_model(&iter, type);
+			selected_card_idx =
+			    deck_card_oldest_card(get_devel_deck(), type);
+		}
 	};
 }
 
