@@ -83,6 +83,12 @@ void frontend_gui_update(void);
 /** program the state of a widget for when frontend_gui_update is called. */
 void frontend_gui_check(GuiEvent event, gboolean sensitive);
 
+/** are the widgets sensitive?
+ * @param event The GuiEvent.
+ * @return TRUE when the GuiEvent has widgets associated that are sensitive.
+ */
+gboolean frontend_gui_get_sensitive(GuiEvent event);
+
 /** initialise the frontend_gui_register_* functions */
 void frontend_gui_register_init(void);
 
@@ -102,6 +108,7 @@ void frontend_gui_route_event(GuiEvent event);
 /* callbacks */
 void frontend_init_gtk_et_al(int argc, char **argv);
 void frontend_init(void);
+void frontend_init_game(void);
 void frontend_new_statistics(gint player_num, StatisticType type,
 			     gint num);
 void frontend_new_points(gint player_num, Points * points, gboolean added);
@@ -121,7 +128,7 @@ void frontend_gold_add(gint player_num, gint gold_num);
 void frontend_gold_remove(gint player_num, gint * resources);
 void frontend_gold_choose(gint gold_num, const gint * bank);
 void frontend_gold_done(void);
-void frontend_setup(unsigned num_settlements, unsigned num_roads);
+void frontend_setup(gint num_settlements, gint num_roads);
 void frontend_quote(gint player_num, gint * they_supply,
 		    gint * they_receive);
 void frontend_roadbuilding(gint num_roads);
@@ -143,7 +150,7 @@ void frontend_quote_end(void);
 void frontend_quote_monitor(void);
 void frontend_rolled_dice(gint die1, gint die2, gint player_num);
 void frontend_bought_develop(DevelType type);
-void frontend_played_develop(gint player_num, gint card_idx,
+void frontend_played_develop(gint player_num, guint card_idx,
 			     DevelType type);
 void frontend_resource_change(Resource type, gint new_amount);
 void frontend_robber(void);
@@ -153,6 +160,7 @@ void frontend_robber_done(void);
 void frontend_game_over(gint player, gint points);
 Map *frontend_get_map(void);
 void frontend_set_map(Map * map);
+void frontend_quit(void);
 
 /* connect.c */
 const gchar *connect_get_server(void);
@@ -161,7 +169,7 @@ gboolean connect_get_spectator(void);
 void connect_set_server(const gchar * server);
 void connect_set_port(const gchar * port);
 void connect_set_spectator(gboolean spectator);
-void connect_set_meta_server(const gchar * meta_server);
+void connect_set_metaserver(const gchar * metaserver);
 void connect_create_dlg(void);
 
 /* trade.c */
@@ -257,12 +265,10 @@ void player_init(void);
 GdkColor *player_or_spectator_color(gint player_num);
 /** The colour of the player */
 GdkColor *player_color(gint player_num);
-/** Create an icon of the player, suitable for display on widget,
- *  for player_num, who is connected.
+/** Create an icon of the player, for player_num, who is connected.
  *  You should unref the pixbuf when it is no longer needed
  */
-GdkPixbuf *player_create_icon(GtkWidget * widget, gint player_num,
-			      gboolean connected);
+GdkPixbuf *player_create_icon(gint player_num, gboolean connected);
 void player_show_current(gint player_num);
 void set_num_players(gint num);
 
