@@ -37,10 +37,8 @@
 #include "histogram.h"
 #include "theme.h"
 #include "config-gnome.h"
-#include "gtkbugs.h"
 #include "audio.h"
 #include "notification.h"
-#include "gtkcompat.h"
 
 static GtkWidget *preferences_dlg;
 GtkWidget *app_window;		/* main application window */
@@ -523,19 +521,19 @@ void gui_cursor_set(CursorType type,
 
 void gui_draw_hex(const Hex * hex)
 {
-	if (gmap->pixmap != NULL)
+	if (gmap->surface != NULL)
 		guimap_draw_hex(gmap, hex);
 }
 
 void gui_draw_edge(const Edge * edge)
 {
-	if (gmap->pixmap != NULL)
+	if (gmap->surface != NULL)
 		guimap_draw_edge(gmap, edge);
 }
 
 void gui_draw_node(const Node * node)
 {
-	if (gmap->pixmap != NULL)
+	if (gmap->surface != NULL)
 		guimap_draw_node(gmap, node);
 }
 
@@ -551,7 +549,7 @@ static gint button_press_map_cb(GtkWidget * area, GdkEventButton * event,
 		return FALSE;
 
 	if (event->button == 1 && event->type == GDK_BUTTON_PRESS) {
-		guimap_cursor_select(gmap, event->x, event->y);
+		guimap_cursor_select(gmap);
 		return TRUE;
 	}
 	return FALSE;
@@ -574,7 +572,7 @@ static GtkWidget *build_messages_panel(void)
 	GtkWidget *label;
 	GtkWidget *scroll_win;
 
-	vbox = gtk_vbox_new(FALSE, 5);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5);
 	gtk_widget_show(vbox);
 
 	/* Label for messages log */
@@ -599,7 +597,7 @@ static GtkWidget *build_messages_panel(void)
 	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(messages_txt),
 				    GTK_WRAP_WORD);
 
-	message_window_set_text(messages_txt);
+	message_window_set_text(messages_txt, chat_panel);
 
 	return vbox;
 }
@@ -697,7 +695,6 @@ static GtkWidget *splash_build_page(void)
 static GtkWidget *build_map_panel(void)
 {
 	GtkWidget *lbl;
-	GtkWidget *legend_content;
 	GtkWidget *hbox;
 	GtkWidget *close_button;
 
@@ -739,20 +736,7 @@ static GtkWidget *build_map_panel(void)
 	/* Tab page name */
 	lbl = gtk_label_new(_("Legend"));
 	gtk_widget_show(lbl);
-	legend_content = legend_create_content();
-
-	legend_page = gtk_scrolled_window_new(NULL, NULL);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(legend_page),
-				       GTK_POLICY_AUTOMATIC,
-				       GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW
-					    (legend_page),
-					    GTK_SHADOW_NONE);
-	gtk_widget_show(legend_page);
-	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW
-					      (legend_page),
-					      legend_content);
-
+	legend_page = legend_create_content();
 	gtk_notebook_insert_page(GTK_NOTEBOOK(map_notebook),
 				 legend_page, lbl, LEGEND_PAGE);
 	if (!legend_page_enabled)
@@ -881,10 +865,10 @@ static GtkWidget *build_main_interface(void)
 	GtkWidget *vpaned;
 	GtkWidget *panel;
 
-	hpaned = gtk_hpaned_new();
+	hpaned = gtk_paned_new(GTK_ORIENTATION_HORIZONTAL);
 	gtk_widget_show(hpaned);
 
-	vbox = gtk_vbox_new(FALSE, 3);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 3);
 	gtk_widget_show(vbox);
 	gtk_paned_pack1(GTK_PANED(hpaned), vbox, FALSE, TRUE);
 
@@ -897,17 +881,17 @@ static GtkWidget *build_main_interface(void)
 	gtk_box_pack_start(GTK_BOX(vbox),
 			   player_build_summary(), TRUE, TRUE, 0);
 
-	main_paned = gtk_hpaned_new();
+	main_paned = gtk_paned_new(GTK_ORIENTATION_HORIZONTAL);
 	gtk_widget_show(main_paned);
 
-	vpaned = gtk_vpaned_new();
+	vpaned = gtk_paned_new(GTK_ORIENTATION_VERTICAL);
 	gtk_widget_show(vpaned);
 
 	gtk_paned_pack1(GTK_PANED(main_paned), vpaned, TRUE, TRUE);
 
 	gtk_paned_pack1(GTK_PANED(vpaned), build_map_panel(), TRUE, TRUE);
 
-	chat_panel = gtk_vbox_new(FALSE, 0);
+	chat_panel = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	gui_show_splash_page(TRUE, chat_panel);
 
 	panel = chat_build_panel();
@@ -927,7 +911,7 @@ static void quit_cb(G_GNUC_UNUSED GtkWidget * widget,
 		    G_GNUC_UNUSED void *data)
 {
 	guimap_delete(gmap);
-	gtk_main_quit();
+	frontend_quit();
 }
 
 static void theme_change_cb(GtkWidget * widget, G_GNUC_UNUSED void *data)
@@ -937,9 +921,9 @@ static void theme_change_cb(GtkWidget * widget, G_GNUC_UNUSED void *data)
 	if (theme != theme_get_current()) {
 		config_set_string("settings/theme", theme->name);
 		theme_set_current(theme);
-		if (gmap->pixmap != NULL) {
-			g_object_unref(gmap->pixmap);
-			gmap->pixmap = NULL;
+		if (gmap->surface != NULL) {
+			cairo_surface_destroy(gmap->surface);
+			gmap->surface = NULL;
 		}
 		theme_rescale(2 * gmap->x_point);
 	}
@@ -1389,7 +1373,7 @@ static void gui_toolbar_show_accelerators(gboolean show_accelerators)
 		GtkToolItem *ti;
 		GtkToolButton *tbtn;
 		gchar *text;
-		gint j;
+		guint j;
 
 		ti = gtk_toolbar_get_nth_item(tb, i);
 		tbtn = GTK_TOOL_BUTTON(ti);
@@ -1537,7 +1521,6 @@ static GtkWidget *build_status_bar(void)
 	GtkWidget *vsep;
 
 	app_bar = gtk_statusbar_new();
-	gtk_statusbar_set_has_resize_grip(GTK_STATUSBAR(app_bar), TRUE);
 	gtk_widget_show(app_bar);
 
 	vp_target_status = gtk_label_new("");
@@ -1545,7 +1528,7 @@ static GtkWidget *build_status_bar(void)
 	gtk_box_pack_start(GTK_BOX(app_bar), vp_target_status, FALSE, TRUE,
 			   0);
 
-	vsep = gtk_vseparator_new();
+	vsep = gtk_separator_new(GTK_ORIENTATION_VERTICAL);
 	gtk_widget_show(vsep);
 	gtk_box_pack_start(GTK_BOX(app_bar), vsep, FALSE, TRUE, 0);
 
@@ -1554,7 +1537,7 @@ static GtkWidget *build_status_bar(void)
 	gtk_widget_show(net_status);
 	gtk_box_pack_start(GTK_BOX(app_bar), net_status, FALSE, TRUE, 0);
 
-	vsep = gtk_vseparator_new();
+	vsep = gtk_separator_new(GTK_ORIENTATION_VERTICAL);
 	gtk_widget_show(vsep);
 	gtk_box_pack_start(GTK_BOX(app_bar), vsep, FALSE, TRUE, 0);
 
@@ -1569,7 +1552,7 @@ static GtkWidget *build_status_bar(void)
 
 static void register_pixmaps(void)
 {
-	gint idx;
+	guint idx;
 
 	GtkIconFactory *factory = gtk_icon_factory_new();
 
@@ -1628,7 +1611,7 @@ GtkWidget *gui_build_interface(void)
 
 	prepare_gtk_for_close_button_on_tab();
 
-	vbox = gtk_vbox_new(FALSE, 0);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	gtk_widget_show(vbox);
 	gtk_container_add(GTK_CONTAINER(app_window), vbox);
 
