@@ -23,6 +23,7 @@
 #include "config.h"
 #include "frontend.h"
 #include "theme.h"
+#include "map-icons.h"
 #include "cost.h"
 #include "resource-view.h"
 
@@ -49,19 +50,12 @@ static void legend_rules_changed(void);
 static void add_legend_terrain(GtkWidget * table, guint row, guint col,
 			       Terrain terrain, Resource resource)
 {
-	GtkWidget *area;
 	GtkWidget *label;
 
-	area = gtk_drawing_area_new();
-	gtk_widget_show(area);
-	gtk_table_attach(GTK_TABLE(table), area,
+	gtk_table_attach(GTK_TABLE(table), terrain_icon_new(terrain),
 			 col, col + 1, row, row + 1,
 			 (GtkAttachOptions) GTK_FILL,
 			 (GtkAttachOptions) GTK_FILL, 0, 0);
-	gtk_widget_set_size_request(area, 30, 34);
-	g_signal_connect(G_OBJECT(area), "expose_event",
-			 G_CALLBACK(expose_terrain_cb),
-			 GINT_TO_POINTER(terrain));
 
 	label = gtk_label_new(_(terrain_names[terrain]));
 	gtk_widget_show(label);
@@ -115,7 +109,8 @@ static void add_legend_cost(GtkWidget * table, guint row,
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 }
 
-GtkWidget *legend_create_content(void)
+static GtkWidget *legend_create_content_with_scrolling(gboolean
+						       enable_scrolling)
 {
 	GtkWidget *hbox;
 	GtkWidget *vbox;
@@ -124,10 +119,12 @@ GtkWidget *legend_create_content(void)
 	GtkWidget *vsep;
 	GtkWidget *alignment;
 	guint num_rows;
+	GtkWidget *viewport;
+	GtkWidget *scrolled_window;
 
-	hbox = gtk_hbox_new(FALSE, 6);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);
 
-	vbox = gtk_vbox_new(FALSE, 6);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 6);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
 
 	label = gtk_label_new(NULL);
@@ -153,7 +150,7 @@ GtkWidget *legend_create_content(void)
 	add_legend_terrain(table, 2, 0, MOUNTAIN_TERRAIN, ORE_RESOURCE);
 	add_legend_terrain(table, 3, 0, PASTURE_TERRAIN, WOOL_RESOURCE);
 
-	vsep = gtk_vseparator_new();
+	vsep = gtk_separator_new(GTK_ORIENTATION_VERTICAL);
 	gtk_widget_show(vsep);
 	gtk_table_attach(GTK_TABLE(table), vsep, 4, 5, 0, 4,
 			 GTK_FILL, GTK_FILL, 0, 0);
@@ -211,7 +208,31 @@ GtkWidget *legend_create_content(void)
 	gtk_widget_show(vbox);
 	gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
 	gtk_widget_show(hbox);
-	return hbox;
+
+	if (!enable_scrolling) {
+		return hbox;
+	}
+
+	viewport = gtk_viewport_new(NULL, NULL);
+	gtk_widget_show(viewport);
+	gtk_container_add(GTK_CONTAINER(viewport), hbox);
+
+	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
+				       (scrolled_window),
+				       GTK_POLICY_AUTOMATIC,
+				       GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW
+					    (scrolled_window),
+					    GTK_SHADOW_NONE);
+	gtk_widget_show(scrolled_window);
+	gtk_container_add(GTK_CONTAINER(scrolled_window), viewport);
+	return scrolled_window;
+}
+
+GtkWidget *legend_create_content(void)
+{
+	return legend_create_content_with_scrolling(TRUE);
 }
 
 GtkWidget *legend_create_dlg(void)
@@ -236,7 +257,7 @@ GtkWidget *legend_create_dlg(void)
 	dlg_vbox = gtk_dialog_get_content_area(GTK_DIALOG(legend_dlg));
 	gtk_widget_show(dlg_vbox);
 
-	vbox = legend_create_content();
+	vbox = legend_create_content_with_scrolling(FALSE);
 	gtk_box_pack_start(GTK_BOX(dlg_vbox), vbox, TRUE, TRUE, 0);
 
 	gtk_widget_show(legend_dlg);
