@@ -33,6 +33,7 @@
 #include "version.h"
 #include "notification.h"
 #include "client.h"
+#include "game-list.h"
 
 static gboolean have_dlg = FALSE;
 static gboolean connectable = FALSE;
@@ -41,9 +42,10 @@ static const gchar *name = NULL;
 static gboolean spectator = FALSE;
 static const gchar *server = NULL;
 static const gchar *port = NULL;
-static const gchar *meta_server = NULL;
+static const gchar *metaserver = NULL;
 static gboolean server_from_commandline = FALSE;
 static gboolean quit_when_offline = FALSE;
+static gboolean mainloop_started = FALSE;
 static gboolean enable_debug = FALSE;
 static gboolean show_version = FALSE;
 
@@ -60,9 +62,9 @@ static GOptionEntry commandline_entries[] = {
 	/* Commandline option of client: do we want to be a spectator */
 	{"spectator", 'v', 0, G_OPTION_ARG_NONE, &spectator,
 	 N_("Connect as a spectator"), NULL},
-	/* Commandline option of client: hostname of the meta-server */
-	{"meta-server", 'm', 0, G_OPTION_ARG_STRING, &meta_server,
-	 N_("Meta-server Host"), PIONEERS_DEFAULT_META_SERVER},
+	/* Commandline option of client: hostname of the metaserver */
+	{"metaserver", 'm', 0, G_OPTION_ARG_STRING, &metaserver,
+	 N_("Metaserver Host"), PIONEERS_DEFAULT_METASERVER},
 	{"debug", '\0', 0, G_OPTION_ARG_NONE, &enable_debug,
 	 /* Commandline option of client: enable debug logging */
 	 N_("Enable debug messages"), NULL},
@@ -150,8 +152,22 @@ void frontend_offline(void)
 
 static void frontend_main(void)
 {
+	mainloop_started = TRUE;
 	gtk_main();
+	notification_cleanup();
 	themes_cleanup();
+	game_list_cleanup();
+}
+
+void frontend_quit(void)
+{
+	if (mainloop_started) {
+		gtk_main_quit();
+	} else {
+		/* The main loop did not start yet.
+		 * Do not quit, so the user can read the log */
+		quit_when_offline = FALSE;
+	}
 }
 
 /* this function is called to let the frontend initialize itself. */
@@ -202,7 +218,7 @@ void frontend_init_gtk_et_al(int argc, char **argv)
 	g_set_application_name(_("Pioneers"));
 
 	callbacks.mainloop = &frontend_main;
-	callbacks.quit = &gtk_main_quit;
+	callbacks.quit = &frontend_quit;
 }
 
 static void frontend_change_name_cb(NotifyingString * name)
@@ -294,9 +310,9 @@ void frontend_init(void)
 	connect_set_server(server);
 	connect_set_port(port);
 
-	if (!meta_server)
-		meta_server = config_get_string("connect/meta-server="
-						PIONEERS_DEFAULT_META_SERVER,
-						&default_returned);
-	connect_set_meta_server(meta_server);
+	if (!metaserver)
+		metaserver = config_get_string("connect/metaserver="
+					       PIONEERS_DEFAULT_METASERVER,
+					       &default_returned);
+	connect_set_metaserver(metaserver);
 }
