@@ -66,7 +66,7 @@ void plenty_create_dlg(const gint * bank)
 							GTK_STOCK_OK,
 							GTK_RESPONSE_OK,
 							NULL);
-	g_signal_connect(GTK_OBJECT(plenty.dlg), "destroy",
+	g_signal_connect(G_OBJECT(plenty.dlg), "destroy",
 			 G_CALLBACK(gtk_widget_destroyed), &plenty.dlg);
 	gtk_widget_realize(plenty.dlg);
 	/* Disable close */
@@ -76,7 +76,7 @@ void plenty_create_dlg(const gint * bank)
 	dlg_vbox = gtk_dialog_get_content_area(GTK_DIALOG(plenty.dlg));
 	gtk_widget_show(dlg_vbox);
 
-	vbox = gtk_vbox_new(FALSE, 5);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5);
 	gtk_widget_show(vbox);
 	gtk_box_pack_start(GTK_BOX(dlg_vbox), vbox, FALSE, TRUE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
