@@ -31,7 +31,6 @@
 #include "frontend.h"
 #include "theme.h"
 #include "common_gtk.h"
-#include "gtkcompat.h"
 
 enum {
 	TRADE_COLUMN_PLAYER, /**< Player icon */
@@ -257,22 +256,23 @@ static void load_pixmaps(QuoteView * qv)
 {
 	static gboolean init = FALSE;
 	int width, height;
-	GdkPixmap *pixmap;
+	cairo_surface_t *surface;
 	cairo_t *cr;
+	cairo_rectangle_t extent;
 
 	if (init)
 		return;
 
 	gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
-	pixmap =
-	    gdk_pixmap_new(gtk_widget_get_window(qv->quotes), width,
-			   height,
-			   gdk_visual_get_depth(gtk_widget_get_visual
-						(qv->quotes)));
+	extent.width = width;
+	extent.height = height;
 
-	cr = gdk_cairo_create(pixmap);
-	gdk_cairo_set_source_pixmap(cr,
-				    theme_get_terrain_pixmap(SEA_TERRAIN),
+	surface =
+	    cairo_recording_surface_create(CAIRO_CONTENT_COLOR_ALPHA,
+					   &extent);
+	cr = cairo_create(surface);
+	gdk_cairo_set_source_pixbuf(cr,
+				    theme_get_terrain_pixbuf(SEA_TERRAIN),
 				    0.0, 0.0);
 	cairo_pattern_set_extend(cairo_get_source(cr),
 				 CAIRO_EXTEND_REPEAT);
@@ -283,10 +283,9 @@ static void load_pixmaps(QuoteView * qv)
 	cairo_rectangle(cr, 0.5, 0.5, width - 1, height - 1);
 	cairo_stroke(cr);
 	maritime_pixbuf =
-	    gdk_pixbuf_get_from_drawable(NULL, pixmap, NULL, 0, 0, 0, 0,
-					 -1, -1);
-	g_object_unref(pixmap);
+	    gdk_pixbuf_get_from_surface(surface, 0, 0, width, height);
 	cairo_destroy(cr);
+	cairo_surface_destroy(surface);
 
 	cross_pixbuf =
 	    gtk_widget_render_icon(qv->quotes, GTK_STOCK_CANCEL,
@@ -434,7 +433,7 @@ void quote_view_reject(QuoteView * qv, gint player_num)
 		gtk_list_store_insert_before(qv->store, &iter, &iter);
 	else
 		gtk_list_store_append(qv->store, &iter);
-	pixbuf = player_create_icon(GTK_WIDGET(qv), player_num, TRUE);
+	pixbuf = player_create_icon(player_num, TRUE);
 	gtk_list_store_set(qv->store, &iter, TRADE_COLUMN_PLAYER, pixbuf,
 			   TRADE_COLUMN_POSSIBLE, cross_pixbuf,
 			   /* Trade: a player has rejected trade */
@@ -582,7 +581,7 @@ void quote_view_add_quote(QuoteView * qv, gint player_num,
 		gtk_list_store_insert_before(qv->store, &iter, &iter);
 	else
 		gtk_list_store_append(qv->store, &iter);
-	pixbuf = player_create_icon(GTK_WIDGET(qv), player_num, TRUE);
+	pixbuf = player_create_icon(player_num, TRUE);
 	gtk_list_store_set(qv->store, &iter, TRADE_COLUMN_PLAYER, pixbuf,
 			   TRADE_COLUMN_POSSIBLE,
 			   qv->check_quote_func(quote) ? qv->
@@ -696,24 +695,24 @@ void quote_view_set_maritime_filters(QuoteView * qv,
 void quote_view_theme_changed(QuoteView * qv)
 {
 	int width, height;
-	GdkPixmap *pixmap;
+	cairo_surface_t *surface;
 	cairo_t *cr;
+	cairo_rectangle_t extent;
 	QuoteInfo *quote;
 
 	if (!qv->with_maritime)
 		return;
 
 	gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
+	extent.width = width;
+	extent.height = height;
 
-	pixmap =
-	    gdk_pixmap_new(gtk_widget_get_window(qv->quotes), width,
-			   height,
-			   gdk_visual_get_depth(gtk_widget_get_visual
-						(qv->quotes)));
-
-	cr = gdk_cairo_create(pixmap);
-	gdk_cairo_set_source_pixmap(cr,
-				    theme_get_terrain_pixmap(SEA_TERRAIN),
+	surface =
+	    cairo_recording_surface_create(CAIRO_CONTENT_COLOR_ALPHA,
+					   &extent);
+	cr = cairo_create(surface);
+	gdk_cairo_set_source_pixbuf(cr,
+				    theme_get_terrain_pixbuf(SEA_TERRAIN),
 				    0.0, 0.0);
 	cairo_pattern_set_extend(cairo_get_source(cr),
 				 CAIRO_EXTEND_REPEAT);
@@ -726,10 +725,9 @@ void quote_view_theme_changed(QuoteView * qv)
 	if (maritime_pixbuf)
 		g_object_unref(maritime_pixbuf);
 	maritime_pixbuf =
-	    gdk_pixbuf_get_from_drawable(NULL, pixmap, NULL, 0, 0, 0, 0,
-					 -1, -1);
+	    gdk_pixbuf_get_from_surface(surface, 0, 0, width, height);
 	cairo_destroy(cr);
-	g_object_unref(pixmap);
+	cairo_surface_destroy(surface);
 
 	/* Remove all maritime quotes */
 	quote = quotelist_first(qv->quote_list);
