@@ -31,7 +31,7 @@ enum { TYPE_NUM,
 	TYPE_STRING
 };
 
-static void add_setting_desc(GtkWidget * table, gint row, gint col,
+static void add_setting_desc(GtkWidget * table, guint row, guint col,
 			     const gchar * desc)
 {
 	GtkWidget *label;
@@ -40,11 +40,11 @@ static void add_setting_desc(GtkWidget * table, gint row, gint col,
 	gtk_widget_show(label);
 	gtk_table_attach(GTK_TABLE(table), label, col, col + 1, row,
 			 row + 1, GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5f);
 }
 
-static void add_setting_desc_with_image(GtkWidget * table, gint row,
-					gint col, const gchar * desc,
+static void add_setting_desc_with_image(GtkWidget * table, guint row,
+					guint col, const gchar * desc,
 					const gchar * iconname)
 {
 	GtkWidget *icon;
@@ -57,7 +57,7 @@ static void add_setting_desc_with_image(GtkWidget * table, gint row,
 	add_setting_desc(table, row, col + 1, desc);
 }
 
-static void add_setting_val(GtkWidget * table, gint row, gint col,
+static void add_setting_val(GtkWidget * table, guint row, guint col,
 			    gint type, gint int_val,
 			    const gchar * char_val, gboolean right_aligned)
 {
@@ -95,7 +95,7 @@ static void add_setting_val(GtkWidget * table, gint row, gint col,
 			 col, col + 1, row, row + 1,
 			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 	gtk_misc_set_alignment(GTK_MISC(label),
-			       (right_aligned ? 1.0 : 0.0), 0.5);
+			       (right_aligned ? 1.0 : 0.0), 0.5f);
 }
 
 static GtkWidget *settings_create_content(void)
@@ -112,7 +112,7 @@ static GtkWidget *settings_create_content(void)
 	guint row;
 
 	/* Create some space inside the dialog */
-	dlg_vbox = gtk_vbox_new(FALSE, 6);
+	dlg_vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 6);
 	gtk_container_set_border_width(GTK_CONTAINER(dlg_vbox), 6);
 	gtk_widget_show(dlg_vbox);
 
@@ -139,7 +139,7 @@ static GtkWidget *settings_create_content(void)
 	gtk_widget_show(alignment);
 	gtk_box_pack_start(GTK_BOX(dlg_vbox), alignment, FALSE, FALSE, 0);
 
-	table = gtk_table_new(10, 2, FALSE);
+	table = gtk_table_new(13, 2, FALSE);
 	gtk_widget_show(table);
 	gtk_container_add(GTK_CONTAINER(alignment), table);
 	gtk_table_set_row_spacings(GTK_TABLE(table), 3);
@@ -191,6 +191,24 @@ static GtkWidget *settings_create_content(void)
 	add_setting_val(table, row, 1, TYPE_STRING, 0, sevens_desc, FALSE);
 	g_free(sevens_desc);
 
+	row++;
+	add_setting_desc(table, row, 0,
+			 _("Use dice deck instead of dice:"));
+	add_setting_val(table, row, 1, TYPE_BOOL,
+			game_params->use_dice_deck, NULL, FALSE);
+
+	row++;
+	add_setting_desc(table, row, 0, _("Number of dice decks:"));
+	add_setting_val(table, row, 1, TYPE_NUM,
+			game_params->num_dice_decks, NULL, FALSE);
+
+	row++;
+	add_setting_desc(table, row, 0,
+			 _(""
+			   "Number of dice cards removed after shuffling:"));
+	add_setting_val(table, row, 1, TYPE_NUM,
+			game_params->num_removed_dice_cards, NULL, FALSE);
+
 	row++;
 	add_setting_desc(table, row, 0,
 			 _("Use the pirate to block ships:"));
@@ -198,7 +216,7 @@ static GtkWidget *settings_create_content(void)
 			NULL, FALSE);
 
 	if (game_params->island_discovery_bonus) {
-		gint idx;
+		guint idx;
 		island_bonus =
 		    g_strdup_printf("%d",
 				    g_array_index
@@ -228,11 +246,11 @@ static GtkWidget *settings_create_content(void)
 	g_free(island_bonus);
 
 	/* Double space, otherwise the columns are too close */
-	hbox = gtk_hbox_new(FALSE, 24);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 24);
 	gtk_widget_show(hbox);
 	gtk_box_pack_start(GTK_BOX(dlg_vbox), hbox, TRUE, FALSE, 0);
 
-	vbox = gtk_vbox_new(FALSE, 6);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 6);
 	gtk_widget_show(vbox);
 	gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
 
@@ -292,7 +310,7 @@ static GtkWidget *settings_create_content(void)
 			game_params->num_build_type[BUILD_BRIDGE], NULL,
 			TRUE);
 
-	vbox = gtk_vbox_new(FALSE, 6);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 6);
 	gtk_widget_show(vbox);
 	gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, TRUE, 0);
 
@@ -394,7 +412,7 @@ GtkWidget *settings_create_dlg(void)
 							  GTK_STOCK_CLOSE,
 							  GTK_RESPONSE_CLOSE,
 							  NULL);
-	g_signal_connect(GTK_OBJECT(settings_dlg), "destroy",
+	g_signal_connect(G_OBJECT(settings_dlg), "destroy",
 			 G_CALLBACK(gtk_widget_destroyed), &settings_dlg);
 
 	dlg_vbox = gtk_dialog_get_content_area(GTK_DIALOG(settings_dlg));
@@ -403,7 +421,7 @@ GtkWidget *settings_create_dlg(void)
 	vbox = settings_create_content();
 	gtk_box_pack_start(GTK_BOX(dlg_vbox), vbox, FALSE, FALSE, 0);
 
-	g_signal_connect(settings_dlg, "response",
+	g_signal_connect(G_OBJECT(settings_dlg), "response",
 			 G_CALLBACK(gtk_widget_destroy), NULL);
 
 	gtk_widget_show(settings_dlg);
