@@ -25,37 +25,6 @@
 #include "game.h"
 #include "cards.h"
 
-DevelDeck *deck_new(GameParams * params)
-{
-	DevelDeck *deck;
-	gint num;
-	gint idx;
-
-	deck = g_malloc0(sizeof(*deck));
-	for (num = idx = 0; idx < G_N_ELEMENTS(params->num_develop_type);
-	     idx++)
-		num += params->num_develop_type[idx];
-	deck->max_cards = num;
-	deck->cards = g_malloc0(deck->max_cards * sizeof(*deck->cards));
-	return deck;
-}
-
-void deck_free(DevelDeck * deck)
-{
-	if (deck->cards != NULL)
-		g_free(deck->cards);
-	g_free(deck);
-}
-
-void deck_card_add(DevelDeck * deck, DevelType type, gint turn_bought)
-{
-	if (deck->num_cards >= deck->max_cards)
-		return;
-	deck->cards[deck->num_cards].type = type;
-	deck->cards[deck->num_cards].turn_bought = turn_bought;
-	deck->num_cards++;
-}
-
 gboolean is_victory_card(DevelType type)
 {
 	return type == DEVEL_CHAPEL
@@ -64,52 +33,44 @@ gboolean is_victory_card(DevelType type)
 	    || type == DEVEL_LIBRARY || type == DEVEL_MARKET;
 }
 
-DevelType deck_card_type(const DevelDeck * deck, gint idx)
+gboolean deck_card_playable(const Deck * deck, guint num_playable_cards,
+			    guint idx)
 {
-	return deck->cards[idx].type;
-}
-
-gboolean deck_card_playable(const DevelDeck * deck,
-			    gboolean played_develop, gint idx, gint turn)
-{
-	if (idx >= deck->num_cards)
+	if (idx >= deck_count(deck))
 		return FALSE;
 
-	if (is_victory_card(deck->cards[idx].type))
+	if (is_victory_card(deck_get_guint(deck, idx)))
 		return TRUE;
 
-	return !played_develop && deck->cards[idx].turn_bought < turn;
+	return idx < num_playable_cards;
 }
 
-gboolean deck_card_play(DevelDeck * deck,
-			gboolean played_develop, gint idx, gint turn)
+gboolean deck_card_play(Deck * deck, guint num_playable_cards, guint idx)
 {
-	if (!deck_card_playable(deck, played_develop, idx, turn))
+	if (!deck_card_playable(deck, num_playable_cards, idx)) {
 		return FALSE;
-
-	deck->num_cards--;
-	memmove(deck->cards + idx, deck->cards + idx + 1,
-		(deck->num_cards - idx) * sizeof(*deck->cards));
+	}
+	deck_remove(deck, idx);
 
 	return TRUE;
 }
 
-gint deck_card_amount(const DevelDeck * deck, DevelType type)
+gint deck_card_amount(const Deck * deck, DevelType type)
 {
-	gint idx;
+	guint idx;
 	gint amount = 0;
 
-	for (idx = 0; idx < deck->num_cards; ++idx)
-		if (deck->cards[idx].type == type)
+	for (idx = 0; idx < deck_count(deck); ++idx)
+		if (deck_get_guint(deck, idx) == type)
 			++amount;
 	return amount;
 }
 
-gint deck_card_oldest_card(const DevelDeck * deck, DevelType type)
+gint deck_card_oldest_card(const Deck * deck, DevelType type)
 {
-	gint idx;
-	for (idx = 0; idx < deck->num_cards; ++idx)
-		if (deck->cards[idx].type == type)
+	guint idx;
+	for (idx = 0; idx < deck_count(deck); ++idx)
+		if (deck_get_guint(deck, idx) == type)
 			return idx;
 	return -1;
 }
