@@ -21,35 +21,18 @@
 #ifndef __cards_h
 #define __cards_h
 
+#include "deck.h"
 #include "game.h"
 
-/* It is important to know which turn a development card was bought
- * in.  You cannot play a card in the same turn that it was bought.
- */
-typedef struct {
-	DevelType type;
-	gint turn_bought;
-} DevelCard;
-
-typedef struct {
-	DevelCard *cards;
-	gint num_cards;
-	gint max_cards;
-} DevelDeck;
-
 gboolean is_victory_card(DevelType type);
 const gchar *get_devel_name(DevelType type);
 const gchar *get_devel_description(DevelType description);
 
-DevelDeck *deck_new(GameParams * params);
-void deck_free(DevelDeck * deck);
-void deck_card_add(DevelDeck * deck, DevelType type, gint turn_bought);
-gboolean deck_card_playable(const DevelDeck * deck,
-			    gboolean played_develop, gint idx, gint turn);
-gboolean deck_card_play(DevelDeck * deck,
-			gboolean played_develop, gint idx, gint turn);
-DevelType deck_card_type(const DevelDeck * deck, gint idx);
+gboolean deck_card_playable(const Deck * deck, guint num_playable_cards,
+			    guint idx);
+gboolean deck_card_play(Deck * deck, guint num_playable_cards, guint idx);
+
+gint deck_card_amount(const Deck * deck, DevelType type);
+gint deck_card_oldest_card(const Deck * deck, DevelType type);
 
-gint deck_card_amount(const DevelDeck * deck, DevelType type);
-gint deck_card_oldest_card(const DevelDeck * deck, DevelType type);
 #endif
