@@ -22,9 +22,6 @@
 #define __driver_h
 
 #include "log.h"
-#include "state.h"
-
-typedef void (*InputFunc) (gpointer);
 
 typedef struct {
 	/* function for clearing the event queue */
@@ -33,11 +30,6 @@ typedef struct {
 	/* Function to write logs and data to the system display */
 	LogFunc log_write;	/* ==> void log_write( gint msg_type, gchar *text ); */
 
-	/* event-loop related functions */
-	 guint(*input_add_read) (gint fd, InputFunc func, gpointer param);
-	 guint(*input_add_write) (gint fd, InputFunc func, gpointer param);
-	void (*input_remove) (guint tag);
-
 	/* callbacks for the server */
 	void (*player_added) (void *player);	/* these really should be ... */
 	void (*player_renamed) (void *player);	/* ... `Player *player', but */
