@@ -27,7 +27,9 @@
 #include "common_gtk.h"
 #include "cards.h"
 
+static GtkCssProvider *small_button_style_provider;
 static GtkWidget *message_txt;
+static GtkWidget *message_container;
 static gboolean msg_colors = TRUE;
 
 /* Local function prototypes */
@@ -141,6 +143,10 @@ void message_window_log_message_string(gint msg_type, const gchar * text)
 		break;
 	}
 
+	if (message_container != NULL) {
+		gtk_widget_show(message_container);
+	}
+
 	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(message_txt));
 
 	/* insert text at the end */
@@ -159,12 +165,13 @@ void message_window_log_message_string(gint msg_type, const gchar * text)
 }
 
 /* set the text widget. */
-void message_window_set_text(GtkWidget * textWidget)
+void message_window_set_text(GtkWidget * textWidget, GtkWidget * container)
 {
 	GtkTextBuffer *buffer;
 	GtkTextIter iter;
 
 	message_txt = textWidget;
+	message_container = container;
 
 	/* Prepare all tags */
 	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(message_txt));
@@ -230,10 +237,6 @@ UIDriver GTK_Driver = {
 	 *   the message window is created. */
 	log_message_string_console,
 
-	NULL,			/* add read input */
-	NULL,			/* add write input */
-	NULL,			/* remove input */
-
 	/* callbacks for the server; NULL for now -- let the server set them */
 	NULL,			/* player added */
 	NULL,			/* player renamed */
@@ -331,14 +334,19 @@ void check_victory_points(GameParams * params, GtkWindow * main_window)
 
 void prepare_gtk_for_close_button_on_tab(void)
 {
-	gtk_rc_parse_string("style \"tab-with-close-button-style\"\n"
-			    "{\n"
-			    "  GtkWidget::focus-padding = 0\n"
-			    "  GtkWidget::focus-line-width = 0\n"
-			    "  xthickness = 0\n"
-			    "  ythickness = 0\n"
-			    "}\n"
-			    "widget \"*.tab-with-close-button\" style \"tab-with-close-button-style\"");
+	/* The code for adding the button is based on the code on
+	 * http://www.micahcarrick.com/gtk-notebook-tabs-with-close-button.html
+	 * by Micah Carrick
+	 */
+	const gchar *data = ".button {\n"
+	    "-GtkButton-default-border : 0px;\n"
+	    "-GtkButton-default-outside-border : 0px;\n"
+	    "-GtkButton-inner-border: 0px;\n"
+	    "-GtkWidget-focus-line-width : 0px;\n"
+	    "-GtkWidget-focus-padding : 0px;\n" "padding: 0px;\n" "}";
+	small_button_style_provider = gtk_css_provider_new();
+	gtk_css_provider_load_from_data(small_button_style_provider, data,
+					-1, NULL);
 }
 
 GtkWidget *create_label_with_close_button(const gchar * label_text,
@@ -349,7 +357,7 @@ GtkWidget *create_label_with_close_button(const gchar * label_text,
 	GtkWidget *lbl;
 	GtkWidget *close_image;
 
-	hbox = gtk_hbox_new(FALSE, 4);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
 	lbl = gtk_label_new(label_text);
 	gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
 
@@ -362,8 +370,12 @@ GtkWidget *create_label_with_close_button(const gchar * label_text,
 	gtk_widget_set_tooltip_text(*button, tooltip_text);
 	gtk_box_pack_start(GTK_BOX(hbox), *button, FALSE, FALSE, 0);
 
-	/* Set buttons name so the style will affect it */
-	gtk_widget_set_name(*button, "tab-with-close-button");
+	/* Apply the style, to make the button smaller */
+	gtk_style_context_add_provider(gtk_widget_get_style_context
+				       (*button),
+				       GTK_STYLE_PROVIDER
+				       (small_button_style_provider),
+				       GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 
 	gtk_widget_show_all(hbox);
 	return hbox;
@@ -384,7 +396,7 @@ void build_frame(GtkWidget * parent, const gchar * title,
 	GtkWidget *fix;
 	gchar *title_with_markup;
 
-	vbox = gtk_vbox_new(FALSE, 3);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 3);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
 	gtk_box_pack_start(GTK_BOX(parent), vbox, extend, TRUE, 0);
 
@@ -396,7 +408,7 @@ void build_frame(GtkWidget * parent, const gchar * title,
 	gtk_widget_show(label);
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 0);
 
-	hbox = gtk_hbox_new(FALSE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
 
 	fix = gtk_fixed_new();
@@ -415,3 +427,31 @@ void set_tooltip_on_column(GtkTreeViewColumn * column,
 	gtk_widget_show(label);
 	gtk_tree_view_column_set_widget(column, label);
 }
+
+/* Code based on gtk_widget_get_pointer in gtkwidget.c */
+void get_mouse_position(GtkWidget * widget, gdouble * x, gdouble * y)
+{
+	gint xi;
+	gint yi;
+
+	g_return_if_fail(GTK_IS_WIDGET(widget));
+
+	xi = -1;
+	yi = -1;
+
+	if (gtk_widget_get_realized(widget)) {
+		gdk_window_get_device_position(gtk_widget_get_window
+					       (widget),
+					       gdk_device_manager_get_client_pointer
+					       (gdk_display_get_device_manager
+						(gtk_widget_get_display
+						 (widget))), &xi, &yi,
+					       NULL);
+
+	}
+
+	if (x)
+		*x = xi;
+	if (y)
+		*y = yi;
+}
