@@ -4,7 +4,7 @@
  * Copyright (C) 2000 Dave Cole
  * Copyright (C) 2003 Bas Wijnen <shevek@fmf.nl>
  * Copyright (C) 2005 Roland Clobus <rclobus@bigfoot.com>
- * Copyright (C) 2005 Ferenc Bnhidi <banhidi@inf.elte.hu>
+ * Copyright (C) 2005 Ferenc Bánhidi <banhidi@inf.elte.hu>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -93,17 +93,17 @@ static void config_sync(void)
 	data = g_key_file_to_data(keyfile, &length, &error);
 
 	if (!error) {
-		int f = open(filename, O_WRONLY | O_CREAT | O_TRUNC,
-			     S_IRUSR | S_IWUSR);
+		int f = g_open(filename, O_WRONLY | O_CREAT | O_TRUNC,
+			       S_IRUSR | S_IWUSR);
 		if (f == -1) {
 			/* Create the config dir, if it is missing */
 			/* Access mode: 0700 (drwx------) */
 			g_mkdir(g_get_user_config_dir(), S_IRWXU);
-			f = open(filename, O_WRONLY | O_CREAT | O_TRUNC,
-				 S_IRUSR | S_IWUSR);
+			f = g_open(filename, O_WRONLY | O_CREAT | O_TRUNC,
+				   S_IRUSR | S_IWUSR);
 		};
 		if (f != -1) {
-			if (write(f, data, length) < length) {
+			if (write(f, data, length) < (ssize_t) length) {
 				g_warning
 				    ("Incomplete settings file written");
 			}
