@@ -3,7 +3,8 @@
  *
  * Copyright (C) 1999 Dave Cole
  * Copyright (C) 2003 Bas Wijnen <shevek@fmf.nl>
- * Copyright (C) 2004-2011 Roland Clobus <rclobus@bigfoot.com>
+ * Copyright (C) 2004-2014 Roland Clobus <rclobus@rclobus.nl>
+ * Copyright (C) 2013 Micah Bunting <Amnykon@gmail.com>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +21,9 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+/** @file guimap.c
+ */
+
 #include "config.h"
 #include <math.h>
 #include <stdio.h>
@@ -50,8 +54,7 @@ static void calc_node_poly(const GuiMap * gmap, const Node * node,
 static void calc_hex_poly(const GuiMap * gmap, const Hex * hex,
 			  const Polygon * shape, Polygon * poly,
 			  double scale_factor, gint x_shift);
-static void guimap_cursor_move(GuiMap * gmap, gint x, gint y,
-			       MapElement * element);
+static void guimap_cursor_move(GuiMap * gmap, MapElement * element);
 
 /* Square */
 static gint sqr(gint a)
@@ -59,6 +62,9 @@ static gint sqr(gint a)
 	return a * a;
 }
 
+/** Creates a new GuiMap.
+ * @return A newly allocated GuiMap.
+ */
 GuiMap *guimap_new(void)
 {
 	GuiMap *gmap;
@@ -70,15 +76,18 @@ GuiMap *guimap_new(void)
 	return gmap;
 }
 
+/** Frees a GuiMap.
+ * @param gmap The GuiMap to free.
+ */
 void guimap_delete(GuiMap * gmap)
 {
 	if (gmap->area != NULL) {
 		g_object_unref(gmap->area);
 		gmap->area = NULL;
 	}
-	if (gmap->pixmap != NULL) {
-		g_object_unref(gmap->pixmap);
-		gmap->pixmap = NULL;
+	if (gmap->surface != NULL) {
+		cairo_surface_destroy(gmap->surface);
+		gmap->surface = NULL;
 	}
 	if (gmap->cr != NULL) {
 		cairo_destroy(gmap->cr);
@@ -108,35 +117,38 @@ void guimap_delete(GuiMap * gmap)
 	g_free(gmap);
 }
 
+
 void guimap_reset(GuiMap * gmap)
 {
 	gmap->highlight_chit = -1;
 	gmap->player_num = -1;
 }
 
-static gint expose_map_cb(GtkWidget * area, GdkEventExpose * event,
-			  gpointer user_data)
+static gboolean draw_map_cb(GtkWidget * area, cairo_t * cr,
+			    gpointer user_data)
 {
 	GuiMap *gmap = user_data;
-	cairo_t *cr;
+	GtkAllocation allocation;
 
-	if (gtk_widget_get_window(area) == NULL || gmap->map == NULL)
+	if (gmap->map == NULL) {
 		return FALSE;
+	}
 
-	if (gmap->pixmap == NULL) {
-		GtkAllocation allocation;
-		gtk_widget_get_allocation(area, &allocation);
-		gmap->pixmap = gdk_pixmap_new(gtk_widget_get_window(area),
-					      allocation.width,
-					      allocation.height, -1);
+	gtk_widget_get_allocation(area, &allocation);
+	if (gmap->surface == NULL) {
+		cairo_rectangle_t extent =
+		    { 0.0, 0.0, allocation.width, allocation.height };
+		gmap->surface =
+		    cairo_recording_surface_create
+		    (CAIRO_CONTENT_COLOR_ALPHA, &extent);
 		guimap_display(gmap);
 	}
 
-	cr = gdk_cairo_create(gtk_widget_get_window(area));
-	gdk_cairo_set_source_pixmap(cr, gmap->pixmap, 0, 0);
-	gdk_cairo_rectangle(cr, &event->area);
+
+	cairo_set_source_surface(cr, gmap->surface, 0.0, 0.0);
+	cairo_rectangle(cr, 0.0, 0.0, allocation.width, allocation.height);
 	cairo_fill(cr);
-	cairo_destroy(cr);
+
 	return FALSE;
 }
 
@@ -150,9 +162,9 @@ static gint configure_map_cb(GtkWidget * area,
 	if (gtk_widget_get_window(area) == NULL || gmap->map == NULL)
 		return FALSE;
 
-	if (gmap->pixmap) {
-		g_object_unref(gmap->pixmap);
-		gmap->pixmap = NULL;
+	if (gmap->surface != NULL) {
+		cairo_surface_destroy(gmap->surface);
+		gmap->surface = NULL;
 	}
 	gtk_widget_get_allocation(area, &allocation);
 	guimap_scale_to_size(gmap, allocation.width, allocation.height);
@@ -166,10 +178,8 @@ static gboolean motion_notify_map_cb(GtkWidget * area,
 				     gpointer user_data)
 {
 	GuiMap *gmap = user_data;
-	gint x;
-	gint y;
-	static gint last_x;
-	static gint last_y;
+	gdouble x;
+	gdouble y;
 	GdkModifierType state;
 	MapElement dummyElement;
 	g_assert(area != NULL);
@@ -177,27 +187,23 @@ static gboolean motion_notify_map_cb(GtkWidget * area,
 	if (gtk_widget_get_window(area) == NULL || gmap->map == NULL)
 		return FALSE;
 
-	if (event->is_hint)
-		gdk_window_get_pointer(event->window, &x, &y, &state);
-	else {
-		x = event->x;
-		y = event->y;
-		state = event->state;
-	}
+	x = event->x;
+	y = event->y;
+	state = event->state;
 
 	if (state & GDK_BUTTON2_MASK) {
 		gmap->is_custom_view = TRUE;
-		gmap->x_margin += x - last_x;
-		gmap->y_margin += y - last_y;
+		gmap->x_margin += x - gmap->last_x;
+		gmap->y_margin += y - gmap->last_y;
 
 		guimap_display(gmap);
 		gtk_widget_queue_draw(gmap->area);
 	}
-	last_x = x;
-	last_y = y;
+	gmap->last_x = x;
+	gmap->last_y = y;
 
 	dummyElement.pointer = NULL;
-	guimap_cursor_move(gmap, x, y, &dummyElement);
+	guimap_cursor_move(gmap, &dummyElement);
 
 	return FALSE;
 }
@@ -207,11 +213,12 @@ static gboolean zoom_map_cb(GtkWidget * area, GdkEventScroll * event,
 {
 	GuiMap *gmap = user_data;
 	GtkAllocation allocation;
+	gint radius;
 
 	if (gtk_widget_get_window(area) == NULL || gmap->map == NULL)
 		return FALSE;
 
-	gint radius = gmap->hex_radius;
+	radius = gmap->hex_radius;
 	gmap->is_custom_view = TRUE;
 
 	if (event->direction == GDK_SCROLL_UP) {
@@ -243,6 +250,12 @@ static gboolean zoom_map_cb(GtkWidget * area, GdkEventScroll * event,
 	return FALSE;
 }
 
+/** Builds the drawing area for gmap.
+ * @param gmap The GuiMap to build the drawing area for.
+ * @param width The requested width of the drawing area.
+ * @param Height The requested height of the drawing area.
+ * @return The newly created drawing area for gmap.
+ */
 GtkWidget *guimap_build_drawingarea(GuiMap * gmap, gint width, gint height)
 {
 	gmap->area = gtk_drawing_area_new();
@@ -253,8 +266,8 @@ GtkWidget *guimap_build_drawingarea(GuiMap * gmap, gint width, gint height)
 			      | GDK_POINTER_MOTION_HINT_MASK);
 
 	gtk_widget_set_size_request(gmap->area, width, height);
-	g_signal_connect(G_OBJECT(gmap->area), "expose_event",
-			 G_CALLBACK(expose_map_cb), gmap);
+	g_signal_connect(G_OBJECT(gmap->area), "draw",
+			 G_CALLBACK(draw_map_cb), gmap);
 	g_signal_connect(G_OBJECT(gmap->area), "configure_event",
 			 G_CALLBACK(configure_map_cb), gmap);
 
@@ -409,6 +422,9 @@ static void reverse_calc_hex_pos(const GuiMap * gmap,
 				 gint x_coor, gint y_coor, gint * hex_x,
 				 gint * hex_y)
 {
+	gint relx;
+	gint rely;
+
 	y_coor -= gmap->y_margin + gmap->hex_radius;
 	y_coor += gmap->hex_radius;
 	*hex_y = y_coor / (gmap->hex_radius + gmap->y_point);
@@ -423,8 +439,6 @@ static void reverse_calc_hex_pos(const GuiMap * gmap,
 
 	/* The (0,0) will be on the upper left corner of the hex,
 	 * outside the hex */
-	gint relx;
-	gint rely;
 	relx = x_coor - *hex_x * (gmap->x_point * 2);
 	rely = y_coor - *hex_y * (gmap->hex_radius + gmap->y_point);
 
@@ -666,6 +680,110 @@ static void guimap_pirate_polygon(const GuiMap * gmap, const Hex * hex,
 	calc_hex_poly(gmap, hex, &pirate_poly, poly, 80.0, 0);
 }
 
+void draw_port_indicator(PangoLayout * layout, cairo_t * cr,
+			 gdouble x_offset, gdouble y_offset,
+			 gdouble chit_radius, gint resource)
+{
+	MapTheme *theme = theme_get_current();
+
+	if (resource != NO_RESOURCE && chit_radius > 0) {
+		const gchar *str = "";
+		gint width, height;
+		gint tileno = resource == ANY_RESOURCE ?
+		    ANY_PORT_TILE : resource;
+		gboolean drawit;
+		gboolean typeind;
+
+		/* Fill/tile port indicator */
+		if (theme->port_tiles[tileno]) {
+			gdk_cairo_set_source_pixbuf(cr,
+						    theme->port_tiles
+						    [tileno],
+						    x_offset -
+						    theme->port_tiles_width
+						    [tileno] / 2,
+						    y_offset -
+						    theme->port_tiles_height
+						    [tileno] / 2);
+			cairo_pattern_set_extend(cairo_get_source(cr),
+						 CAIRO_EXTEND_REPEAT);
+			typeind = TRUE;
+			drawit = TRUE;
+		} else if (!theme->colors[TC_PORT_BG].transparent) {
+			gdk_cairo_set_source_color(cr,
+						   &theme->colors
+						   [TC_PORT_BG].color);
+			typeind = FALSE;
+			drawit = TRUE;
+		} else {
+			typeind = FALSE;
+			drawit = FALSE;
+		}
+		if (drawit) {
+			cairo_arc(cr, x_offset, y_offset,
+				  chit_radius, 0.0, 2 * M_PI);
+			cairo_fill(cr);
+		}
+
+		/* Outline port indicator */
+		if (!theme->colors[TC_PORT_BD].transparent) {
+			gdk_cairo_set_source_color(cr,
+						   &theme->colors
+						   [TC_PORT_BD].color);
+			cairo_arc(cr, x_offset, y_offset,
+				  chit_radius, 0.0, 2 * M_PI);
+			cairo_stroke(cr);
+		}
+		/* Print trading ratio */
+		if (!theme->colors[TC_PORT_FG].transparent) {
+			if (typeind) {
+				if (resource < NO_RESOURCE)
+					/* Port indicator for a resource: trade 2 for 1 */
+					str = _("2:1");
+				else
+					/* Port indicator: trade 3 for 1 */
+					str = _("3:1");
+			} else {
+				switch (resource) {
+				case BRICK_RESOURCE:
+					/* Port indicator for brick */
+					str = Q_("Brick port|B");
+					break;
+				case GRAIN_RESOURCE:
+					/* Port indicator for grain */
+					str = Q_("Grain port|G");
+					break;
+				case ORE_RESOURCE:
+					/* Port indicator for ore */
+					str = Q_("Ore port|O");
+					break;
+				case WOOL_RESOURCE:
+					/* Port indicator for wool */
+					str = Q_("Wool port|W");
+					break;
+				case LUMBER_RESOURCE:
+					/* Port indicator for lumber */
+					str = Q_("Lumber port|L");
+					break;
+				default:
+					/* General port indicator */
+					str = _("3:1");
+					break;
+				}
+			}
+			pango_layout_set_markup(layout, str, -1);
+			pango_layout_get_pixel_size(layout, &width,
+						    &height);
+			gdk_cairo_set_source_color(cr,
+						   &theme->colors
+						   [TC_PORT_FG].color);
+			cairo_move_to(cr, x_offset - width / 2,
+				      y_offset - height / 2);
+			pango_cairo_show_layout(cr, layout);
+		}
+	}
+}
+
 void draw_dice_roll(PangoLayout * layout, cairo_t * cr,
 		    gdouble x_offset, gdouble y_offset, gdouble radius,
 		    gint n, gint terrain, gboolean highlight)
@@ -733,7 +851,7 @@ static gboolean display_hex(const Hex * hex, gpointer closure)
 	gint x_offset, y_offset;
 	GdkPoint points[MAX_POINTS];
 	Polygon poly;
-	int idx;
+	guint idx;
 	const MapTheme *theme = theme_get_current();
 	const GuiMap *gmap = closure;
 
@@ -747,8 +865,8 @@ static gboolean display_hex(const Hex * hex, gpointer closure)
 	poly_offset(&poly, x_offset, y_offset);
 
 	/* Draw the hex */
-	gdk_cairo_set_source_pixmap(gmap->cr,
-				    theme->terrain_tiles[hex->terrain],
+	gdk_cairo_set_source_pixbuf(gmap->cr,
+				    theme_get_terrain_pixbuf(hex->terrain),
 				    x_offset - gmap->x_point,
 				    y_offset - gmap->hex_radius);
 	cairo_pattern_set_extend(cairo_get_source(gmap->cr),
@@ -775,16 +893,10 @@ static gboolean display_hex(const Hex * hex, gpointer closure)
 
 	/* Draw ports */
 	if (hex->resource != NO_RESOURCE && gmap->chit_radius > 0) {
-		const gchar *str = "";
-		gint width, height;
-		int tileno = hex->resource == ANY_RESOURCE ?
-		    ANY_PORT_TILE : hex->resource;
-		gboolean drawit;
-		gboolean typeind;
+		const double dashes[] = { 4.0 };
 
 		/* Draw lines from port to shore */
 		gdk_cairo_set_source_color(gmap->cr, &white);
-		const double dashes[] = { 4.0 };
 		cairo_set_dash(gmap->cr, dashes, G_N_ELEMENTS(dashes),
 			       0.0);
 		cairo_move_to(gmap->cr, x_offset, y_offset);
@@ -796,94 +908,9 @@ static gboolean display_hex(const Hex * hex, gpointer closure)
 		cairo_stroke(gmap->cr);
 		cairo_set_dash(gmap->cr, NULL, 0, 0.0);
 
-		/* Fill/tile port indicator */
-		if (theme->port_tiles[tileno]) {
-			gdk_cairo_set_source_pixmap(gmap->cr,
-						    theme->port_tiles
-						    [tileno],
-						    x_offset -
-						    theme->port_tiles_width
-						    [tileno] / 2,
-						    y_offset -
-						    theme->port_tiles_height
-						    [tileno] / 2);
-			cairo_pattern_set_extend(cairo_get_source
-						 (gmap->cr),
-						 CAIRO_EXTEND_REPEAT);
-			typeind = TRUE;
-			drawit = TRUE;
-		} else if (!theme->colors[TC_PORT_BG].transparent) {
-			gdk_cairo_set_source_color(gmap->cr,
-						   &theme->colors
-						   [TC_PORT_BG].color);
-			typeind = FALSE;
-			drawit = TRUE;
-		} else {
-			typeind = FALSE;
-			drawit = FALSE;
-		}
-		if (drawit) {
-			cairo_arc(gmap->cr, x_offset, y_offset,
-				  gmap->chit_radius, 0.0, 2 * M_PI);
-			cairo_fill(gmap->cr);
-		}
-
-		/* Outline port indicator */
-		if (!theme->colors[TC_PORT_BD].transparent) {
-			gdk_cairo_set_source_color(gmap->cr,
-						   &theme->colors
-						   [TC_PORT_BD].color);
-			cairo_arc(gmap->cr, x_offset, y_offset,
-				  gmap->chit_radius, 0.0, 2 * M_PI);
-			cairo_stroke(gmap->cr);
-		}
-		/* Print trading ratio */
-		if (!theme->colors[TC_PORT_FG].transparent) {
-			if (typeind) {
-				if (hex->resource < NO_RESOURCE)
-					/* Port indicator for a resource: trade 2 for 1 */
-					str = _("2:1");
-				else
-					/* Port indicator: trade 3 for 1 */
-					str = _("3:1");
-			} else {
-				switch (hex->resource) {
-				case BRICK_RESOURCE:
-					/* Port indicator for brick */
-					str = Q_("Brick port|B");
-					break;
-				case GRAIN_RESOURCE:
-					/* Port indicator for grain */
-					str = Q_("Grain port|G");
-					break;
-				case ORE_RESOURCE:
-					/* Port indicator for ore */
-					str = Q_("Ore port|O");
-					break;
-				case WOOL_RESOURCE:
-					/* Port indicator for wool */
-					str = Q_("Wool port|W");
-					break;
-				case LUMBER_RESOURCE:
-					/* Port indicator for lumber */
-					str = Q_("Lumber port|L");
-					break;
-				default:
-					/* General port indicator */
-					str = _("3:1");
-					break;
-				}
-			}
-			pango_layout_set_markup(gmap->layout, str, -1);
-			pango_layout_get_pixel_size(gmap->layout, &width,
-						    &height);
-			gdk_cairo_set_source_color(gmap->cr,
-						   &theme->colors
-						   [TC_PORT_FG].color);
-			cairo_move_to(gmap->cr, x_offset - width / 2,
-				      y_offset - height / 2);
-			pango_cairo_show_layout(gmap->cr, gmap->layout);
-		}
+		draw_port_indicator(gmap->layout, gmap->cr,
+				    x_offset, y_offset, gmap->chit_radius,
+				    hex->resource);
 	}
 
 	cairo_set_line_width(gmap->cr, 1.0);
@@ -1031,6 +1058,11 @@ void guimap_scale_with_radius(GuiMap * gmap, gint radius)
 
 void guimap_scale_to_size(GuiMap * gmap, gint width, gint height)
 {
+	const gint reserved_width = 0;
+	const gint reserved_height = 0;
+	gint width_radius;
+	gint height_radius;
+
 	if (gmap->is_custom_view) {
 		gint x_margin = gmap->x_margin;
 		gint y_margin = gmap->y_margin;
@@ -1041,10 +1073,6 @@ void guimap_scale_to_size(GuiMap * gmap, gint width, gint height)
 		gmap->height = height;
 		return;
 	}
-	const gint reserved_width = 0;
-	const gint reserved_height = 0;
-	gint width_radius;
-	gint height_radius;
 
 	width_radius = (width - reserved_width)
 	    / ((gmap->map->x_size * 2 + 1
@@ -1101,16 +1129,16 @@ void guimap_display(GuiMap * gmap)
 	PangoFontDescription *pfd;
 	gint font_size;
 
-	if (gmap->pixmap == NULL)
+	if (gmap->surface == NULL)
 		return;
 
 	if (gmap->cr != NULL) {
 		cairo_destroy(gmap->cr);
 	}
 
-	gmap->cr = gdk_cairo_create(gmap->pixmap);
+	gmap->cr = cairo_create(gmap->surface);
 
-	gdk_cairo_set_source_pixmap(gmap->cr,
+	gdk_cairo_set_source_pixbuf(gmap->cr,
 				    theme_get_current()->terrain_tiles
 				    [BOARD_TILE], 0, 0);
 	cairo_pattern_set_extend(cairo_get_source(gmap->cr),
@@ -1177,10 +1205,12 @@ void guimap_zoom_normal(GuiMap * gmap)
 void guimap_zoom_center_map(GuiMap * gmap)
 {
 	GtkAllocation allocation;
+	gint width;
+	gint height;
 
 	if (!gmap->is_custom_view)
 		return;
-	gint width = gmap->map->x_size * 2 * gmap->x_point + gmap->x_point;
+	width = gmap->map->x_size * 2 * gmap->x_point + gmap->x_point;
 	if (gmap->map->shrink_left)
 		width -= gmap->x_point;
 	if (gmap->map->shrink_right)
@@ -1189,7 +1219,7 @@ void guimap_zoom_center_map(GuiMap * gmap)
 	gtk_widget_get_allocation(gmap->area, &allocation);
 	gmap->x_margin = allocation.width / 2 - width / 2;
 
-	gint height = (gmap->map->y_size - 1) *
+	height = (gmap->map->y_size - 1) *
 	    (gmap->hex_radius + gmap->y_point)
 	    + 2 * gmap->hex_radius;
 
@@ -1199,73 +1229,113 @@ void guimap_zoom_center_map(GuiMap * gmap)
 	gtk_widget_queue_draw(gmap->area);
 }
 
-static void find_edge(GuiMap * gmap, gint x, gint y, MapElement * element)
+/** Finds the closest edge to the last known cursor position.
+ * @param gmap The GuiMap containing the edge.
+ * @return The closest edge.
+ */
+Edge *guimap_get_current_edge(GuiMap * gmap)
 {
-	Hex *hex = guimap_find_hex(gmap, x, y);
+	Hex *hex = guimap_get_current_hex(gmap);
 	if (hex) {
 		gint center_x;
 		gint center_y;
+		gdouble angle;
+		gint idx;
+
 		calc_hex_pos(gmap, hex->x, hex->y, &center_x, &center_y);
 
-		gdouble angle = atan2(y - center_y, x - center_x);
-		gint idx =
+		angle =
+		    atan2(gmap->last_y - center_y,
+			  gmap->last_x - center_x);
+		idx =
 		    (gint) (floor(-angle / 2.0 / M_PI * 6 + 0.5) + 6) % 6;
-		element->edge = hex->edges[idx];
+		return hex->edges[idx];
 	} else {
-		element->edge = NULL;
+		return NULL;
 	}
 }
 
-Node *guimap_find_node(GuiMap * gmap, gint x, gint y)
+/** Finds the closest edge to the last known cursor position.
+ * @param gmap The GuiMap containing the edge.
+ * @param[out] element The MapElement to return the edge in.
+ */
+static void find_edge(GuiMap * gmap, MapElement * element)
 {
-	Hex *hex = guimap_find_hex(gmap, x, y);
+	element->edge = guimap_get_current_edge(gmap);
+}
+
+/** Finds the closest node to the last known cursor position.
+ * @param gmap The GuiMap containing the node.
+ * @return The closest node.
+ */
+Node *guimap_get_current_node(GuiMap * gmap)
+{
+	Hex *hex = guimap_get_current_hex(gmap);
 	if (hex) {
 		gint center_x;
 		gint center_y;
+		gdouble angle;
+		gint idx;
+
 		calc_hex_pos(gmap, hex->x, hex->y, &center_x, &center_y);
 
-		gdouble angle = atan2(y - center_y, x - center_x);
-		gint idx = (gint) (floor(-angle / 2.0 / M_PI * 6) + 6) % 6;
+		angle =
+		    atan2(gmap->last_y - center_y,
+			  gmap->last_x - center_x);
+		idx = (gint) (floor(-angle / 2.0 / M_PI * 6) + 6) % 6;
 		return hex->nodes[idx];
 	}
 	return NULL;
 }
 
-static void find_node(GuiMap * gmap, gint x, gint y, MapElement * element)
+/** Finds the closest node to the last known cursor position.
+ * @param gmap The GuiMap containing the node.
+ * @param[out] element The MapElement to return the node in.
+ */
+static void find_node(GuiMap * gmap, MapElement * element)
 {
-	element->node = guimap_find_node(gmap, x, y);
+	element->node = guimap_get_current_node(gmap);
 }
 
-Hex *guimap_find_hex(GuiMap * gmap, gint x, gint y)
+/** Finds the closest hex to the last known cursor position.
+ * @param gmap The GuiMap containing the hex.
+ * @return The closest hex.
+ */
+Hex *guimap_get_current_hex(GuiMap * gmap)
 {
 	gint x_hex;
 	gint y_hex;
 
-	reverse_calc_hex_pos(gmap, x, y, &x_hex, &y_hex);
+	reverse_calc_hex_pos(gmap, gmap->last_x, gmap->last_y, &x_hex,
+			     &y_hex);
 	return map_hex(gmap->map, x_hex, y_hex);
 }
 
-static void find_hex(GuiMap * gmap, gint x, gint y, MapElement * element)
+/** Finds the closest hex to the last known cursor position.
+ * @param gmap The GuiMap containing the hex.
+ * @param[out] element The MapElement to return the hex in.
+ */
+static void find_hex(GuiMap * gmap, MapElement * element)
 {
-	element->hex = guimap_find_hex(gmap, x, y);
+	element->hex = guimap_get_current_hex(gmap);
 }
 
 void guimap_draw_edge(GuiMap * gmap, const Edge * edge)
 {
 	GdkRectangle rect;
-	gint idx;
+	guint idx;
 	Polygon poly;
 	GdkPoint points[MAX_POINTS];
 
 	g_return_if_fail(edge != NULL);
-	g_return_if_fail(gmap->pixmap != NULL);
+	g_return_if_fail(gmap->surface != NULL);
 
 	poly.num_points = G_N_ELEMENTS(points);
 	poly.points = points;
 	calc_edge_poly(gmap, edge, &largest_edge_poly, &poly);
 	poly_bound_rect(&poly, 1, &rect);
 
-	gdk_cairo_set_source_pixmap(gmap->cr,
+	gdk_cairo_set_source_pixbuf(gmap->cr,
 				    theme_get_current()->terrain_tiles
 				    [BOARD_TILE], 0, 0);
 	cairo_pattern_set_extend(cairo_get_source(gmap->cr),
@@ -1344,19 +1414,19 @@ static void draw_bridge_cursor(GuiMap * gmap)
 void guimap_draw_node(GuiMap * gmap, const Node * node)
 {
 	GdkRectangle rect;
-	int idx;
+	guint idx;
 	Polygon poly;
 	GdkPoint points[MAX_POINTS];
 
 	g_return_if_fail(node != NULL);
-	g_return_if_fail(gmap->pixmap != NULL);
+	g_return_if_fail(gmap->surface != NULL);
 
 	poly.num_points = G_N_ELEMENTS(points);
 	poly.points = points;
 	calc_node_poly(gmap, node, &largest_node_poly, &poly);
 	poly_bound_rect(&poly, 1, &rect);
 
-	gdk_cairo_set_source_pixmap(gmap->cr,
+	gdk_cairo_set_source_pixbuf(gmap->cr,
 				    theme_get_current()->terrain_tiles
 				    [BOARD_TILE], 0, 0);
 	cairo_pattern_set_extend(cairo_get_source(gmap->cr),
@@ -1556,7 +1626,7 @@ void guimap_highlight_chits(GuiMap * gmap, gint roll)
 	closure.gmap = gmap;
 	closure.old_highlight = gmap->highlight_chit;
 	gmap->highlight_chit = roll;
-	if (gmap->pixmap != NULL)
+	if (gmap->surface != NULL)
 		map_traverse_const(gmap->map, highlight_chits, &closure);
 }
 
@@ -1584,7 +1654,7 @@ void guimap_draw_hex(GuiMap * gmap, const Hex * hex)
 }
 
 typedef struct {
-	void (*find) (GuiMap * gmap, gint x, gint y, MapElement * element);
+	void (*find) (GuiMap * gmap, MapElement * element);
 	void (*erase_cursor) (GuiMap * gmap);
 	void (*draw_cursor) (GuiMap * gmap);
 } ModeCursor;
@@ -1608,18 +1678,16 @@ CheckFunc roadF, shipF, bridgeF, settlementF, cityF, cityWallF, shipMoveF;
 SelectFunc roadS, shipS, bridgeS, settlementS, cityS, cityWallS, shipMoveS;
 CancelFunc shipMoveC;
 
-/** Calculate the distance between the element and the cursor.
+/** Calculate the distance between the element and the last known position
+ *  of the cursor.
  *  @param gmap The GuiMap
- *  @param element An Edge or a Node
- *  @param isEdge TRUE if element is an Edge
- *  @param cursor_x X position of the cursor
- *  @param cursor_y Y position of the cursor
+ *  @param element A MapElement
+ *  @param type The type of the MapElement
  *  @return The square of the distance
  */
 gint guimap_distance_cursor(const GuiMap * gmap,
 			    const MapElement * element,
-			    MapElementType type, gint cursor_x,
-			    gint cursor_y)
+			    MapElementType type)
 {
 	static GdkPoint single_point = { 0, 0 };
 	static const Polygon simple_poly = { &single_point, 1 };
@@ -1639,12 +1707,12 @@ gint guimap_distance_cursor(const GuiMap * gmap,
 			      120.0, 0);
 		break;
 	}
-	return sqr(cursor_x - poly.points[0].x) + sqr(cursor_y -
-						      poly.points[0].y);
+	return sqr(gmap->last_x - poly.points[0].x) + sqr(gmap->last_y -
+							  poly.
+							  points[0].y);
 }
 
-void guimap_cursor_move(GuiMap * gmap, gint x, gint y,
-			MapElement * element)
+void guimap_cursor_move(GuiMap * gmap, MapElement * element)
 {
 	ModeCursor *mode;
 
@@ -1663,7 +1731,7 @@ void guimap_cursor_move(GuiMap * gmap, gint x, gint y,
 		gint distance_node = 0;
 
 		dummyElement.pointer = NULL;
-		find_edge(gmap, x, y, element);
+		find_edge(gmap, element);
 		if (element->pointer) {
 			can_build_road = (roadM
 					  && roadF(*element,
@@ -1693,10 +1761,10 @@ void guimap_cursor_move(GuiMap * gmap, gint x, gint y,
 				hex2.hex = element->edge->hexes[1];
 				distance1 =
 				    guimap_distance_cursor(gmap, &hex1,
-							   MAP_HEX, x, y);
+							   MAP_HEX);
 				distance2 =
 				    guimap_distance_cursor(gmap, &hex2,
-							   MAP_HEX, x, y);
+							   MAP_HEX);
 				if (distance1 == distance2)
 					can_build_ship = FALSE;
 				else {
@@ -1724,14 +1792,13 @@ void guimap_cursor_move(GuiMap * gmap, gint x, gint y,
 				node2.node = element->edge->nodes[1];
 				distanceNode =
 				    MIN(guimap_distance_cursor
-					(gmap, &node1, MAP_NODE, x, y),
+					(gmap, &node1, MAP_NODE),
 					guimap_distance_cursor(gmap,
 							       &node2,
-							       MAP_NODE, x,
-							       y));
+							       MAP_NODE));
 				distanceEdge =
 				    guimap_distance_cursor(gmap, element,
-							   MAP_EDGE, x, y);
+							   MAP_EDGE);
 				if (distanceNode < distanceEdge)
 					can_build_ship = FALSE;
 				else
@@ -1743,10 +1810,10 @@ void guimap_cursor_move(GuiMap * gmap, gint x, gint y,
 			if (can_build_edge)
 				distance_edge =
 				    guimap_distance_cursor(gmap, element,
-							   MAP_EDGE, x, y);
+							   MAP_EDGE);
 		}
 
-		find_node(gmap, x, y, element);
+		find_node(gmap, element);
 		if (element->pointer) {
 			can_build_settlement = (settlementM
 						&& settlementF(*element,
@@ -1767,7 +1834,7 @@ void guimap_cursor_move(GuiMap * gmap, gint x, gint y,
 			if (can_build_node)
 				distance_node =
 				    guimap_distance_cursor(gmap, element,
-							   MAP_NODE, x, y);
+							   MAP_NODE);
 		}
 
 		/* When both edge and node can be built,
@@ -1826,7 +1893,7 @@ void guimap_cursor_move(GuiMap * gmap, gint x, gint y,
 	}
 
 	mode = cursors + gmap->cursor_type;
-	mode->find(gmap, x, y, element);
+	mode->find(gmap, element);
 	if (element->pointer != gmap->cursor.pointer) {
 		if (gmap->cursor.pointer != NULL)
 			mode->erase_cursor(gmap);
@@ -1842,12 +1909,13 @@ void guimap_cursor_move(GuiMap * gmap, gint x, gint y,
 	}
 }
 
-void guimap_cursor_select(GuiMap * gmap, gint x, gint y)
+/** Select the active cursor */
+void guimap_cursor_select(GuiMap * gmap)
 {
 	MapElement cursor;
 	SelectFunc check_select;
 	MapElement user_data;
-	guimap_cursor_move(gmap, x, y, &cursor);
+	guimap_cursor_move(gmap, &cursor);
 
 	if (cursor.pointer == NULL)
 		return;
