@@ -29,7 +29,6 @@
 #include "game.h"
 #include "theme.h"
 #include "config-gnome.h"
-#include "gtkcompat.h"
 
 /*
 
@@ -81,9 +80,9 @@
   
 */
 
-#define TCOL_INIT(r,g,b)	{ TRUE, FALSE, FALSE, { 0, r, g, b } }
-#define TCOL_TRANSP()		{ TRUE, TRUE, FALSE, { 0, 0, 0, 0 } }
-#define TCOL_UNSET()		{ FALSE, FALSE, FALSE, { 0, 0, 0, 0 } }
+#define TCOL_INIT(r,g,b)	{ TRUE, FALSE, { 0, r, g, b } }
+#define TCOL_TRANSP()		{ TRUE, TRUE, { 0, 0, 0, 0 } }
+#define TCOL_UNSET()		{ FALSE, FALSE, { 0, 0, 0, 0 } }
 #define TSCALE				{ NULL, NULL, 0, 0.0 }
 
 static TColor default_colors[] = {
@@ -175,11 +174,9 @@ static gboolean parsecolor(char *p, TColor * tc, const gchar * filename,
 			   int lno);
 static MapTheme *theme_config_parse(const gchar * themename,
 				    const gchar * subdir);
-static gboolean theme_load_pixmap(const gchar * file,
+static gboolean theme_load_pixbuf(const gchar * file,
 				  const gchar * themename,
-				  GdkPixbuf ** pixbuf,
-				  GdkPixmap ** pixmap_return,
-				  GdkBitmap ** mask_return);
+				  GdkPixbuf ** pixbuf);
 
 /** Find a theme with the given name */
 static gint theme_list_locate(gconstpointer item, gconstpointer data)
@@ -302,22 +299,18 @@ GList *theme_get_list(void)
 	return theme_list;
 }
 
-/** Load a pixbuf, its pixmap and its mask.
+/** Load a pixbuf.
  *  If loading fails, no objects need to be freed.
- *  @return TRUE if succesful
+ *  @return TRUE if successful
  */
-gboolean theme_load_pixmap(const gchar * file, const gchar * themename,
-			   GdkPixbuf ** pixbuf,
-			   GdkPixmap ** pixmap_return,
-			   GdkBitmap ** mask_return)
+gboolean theme_load_pixbuf(const gchar * file, const gchar * themename,
+			   GdkPixbuf ** pixbuf)
 {
 	g_return_val_if_fail(themename != NULL, FALSE);
 	g_return_val_if_fail(pixbuf != NULL, FALSE);
 	g_return_val_if_fail(file != NULL, FALSE);
-	g_return_val_if_fail(pixmap_return != NULL, FALSE);
 
 	*pixbuf = NULL;
-	*pixmap_return = NULL;
 
 	/* check that file exists */
 	if (!g_file_test(file, G_FILE_TEST_EXISTS)) {
@@ -329,17 +322,9 @@ gboolean theme_load_pixmap(const gchar * file, const gchar * themename,
 
 	/* load pixmap/mask */
 	*pixbuf = gdk_pixbuf_new_from_file(file, NULL);
-	*pixmap_return = NULL;
-	if (*pixbuf != NULL) {
-		gdk_pixbuf_render_pixmap_and_mask(*pixbuf,
-						  pixmap_return,
-						  mask_return, 1);
-	}
 
 	/* check result */
-	if (*pixmap_return == NULL) {
-		if (*pixbuf)
-			g_object_unref(*pixbuf);
+	if (*pixbuf == NULL) {
 		g_warning
 		    ("Could not load \'%s\' pixmap file in theme \'%s\'.",
 		     file, themename);
@@ -349,27 +334,26 @@ gboolean theme_load_pixmap(const gchar * file, const gchar * themename,
 }
 
 /** Initialize the theme.
- *  @return TRUE if succesful
+ *  @return TRUE if successful
  */
 static gboolean theme_initialize(MapTheme * t)
 {
-	int i, j;
-	GdkColormap *cmap;
+	guint i;
 
 	/* load terrain tiles */
 	for (i = 0; i < G_N_ELEMENTS(t->terrain_tiles); ++i) {
-		GdkPixbuf *pixbuf, *pixbuf_copy;
-		if (!theme_load_pixmap
-		    (t->terrain_tile_names[i], t->name, &pixbuf,
-		     &(t->terrain_tiles[i]), NULL)) {
+		GdkPixbuf *pixbuf;
+		GdkPixbuf *pixbuf_copy;
+		if (!theme_load_pixbuf
+		    (t->terrain_tile_names[i], t->name, &pixbuf)) {
 			g_error("Could not find pixmap file: %s",
 				t->terrain_tile_names[i]);
 		};
+		t->terrain_tiles[i] = pixbuf;
 		pixbuf_copy = gdk_pixbuf_copy(pixbuf);
 		if (pixbuf_copy == NULL) {
 			return FALSE;
 		}
-		t->scaledata[i].image = pixbuf;
 		t->scaledata[i].native_image = pixbuf_copy;
 		t->scaledata[i].native_width =
 		    gdk_pixbuf_get_width(pixbuf);
@@ -383,57 +367,35 @@ static gboolean theme_initialize(MapTheme * t)
 		/* if a theme doesn't define a port tile, it will be drawn with
 		 * its resource letter instead */
 		if (t->port_tile_names[i]) {
-			GdkPixbuf *pixbuf;
-			if (theme_load_pixmap
-			    (t->port_tile_names[i], t->name, &pixbuf,
-			     &(t->port_tiles[i]), NULL)) {
-				gdk_pixmap_get_size(t->port_tiles[i],
-						    &t->port_tiles_width
-						    [i],
-						    &t->port_tiles_height
-						    [i]);
-				g_object_unref(pixbuf);
+			if (theme_load_pixbuf
+			    (t->port_tile_names[i], t->name,
+			     &(t->port_tiles[i]))) {
+				t->port_tiles_width[i] =
+				    gdk_pixbuf_get_width(t->port_tiles[i]);
+				t->port_tiles_height[i] =
+				    gdk_pixbuf_get_height(t->port_tiles
+							  [i]);
 			}
 		} else
 			t->port_tiles[i] = NULL;
 	}
 
-	/* allocate defined colors */
-	cmap = gdk_colormap_get_system();
-
 	for (i = 0; i < G_N_ELEMENTS(t->colors); ++i) {
 		TColor *tc = &(t->colors[i]);
 		if (!tc->set)
 			*tc = default_colors[i];
-		else if (!tc->transparent && !tc->allocated) {
-			gdk_colormap_alloc_color(cmap, &(tc->color), FALSE,
-						 TRUE);
-			tc->allocated = TRUE;
-		}
 	}
 
-	for (i = 0; i < TC_MAX_OVRTILE; ++i) {
-		for (j = 0; j < TC_MAX_OVERRIDE; ++j) {
-			TColor *tc = &(t->ovr_colors[i][j]);
-			if (tc->set && !tc->transparent && !tc->allocated) {
-				gdk_colormap_alloc_color(cmap,
-							 &(tc->color),
-							 FALSE, TRUE);
-				tc->allocated = TRUE;
-			}
-		}
-	}
 	return TRUE;
 }
 
 static void theme_cleanup(MapTheme * t)
 {
-	int i;
+	guint i;
 
 	/* terrain tiles */
 	for (i = 0; i < G_N_ELEMENTS(t->terrain_tiles); ++i) {
 		g_object_unref(t->terrain_tiles[i]);
-		g_object_unref(t->scaledata[i].image);
 		g_object_unref(t->scaledata[i].native_image);
 	}
 	/* port tiles */
@@ -458,7 +420,7 @@ static void theme_cleanup(MapTheme * t)
 
 void theme_rescale(int new_width)
 {
-	int i;
+	guint i;
 
 	switch (current_theme->scaling) {
 	case NEVER:
@@ -500,21 +462,12 @@ void theme_rescale(int new_width)
 		if (new_height <= 0)
 			new_height = 1;
 		/* rescale the pixbuf */
-		g_object_unref(current_theme->scaledata[i].image);
-		current_theme->scaledata[i].image =
+		g_object_unref(current_theme->terrain_tiles[i]);
+		current_theme->terrain_tiles[i] =
 		    gdk_pixbuf_scale_simple(current_theme->scaledata[i].
 					    native_image, new_width,
 					    new_height,
 					    GDK_INTERP_BILINEAR);
-
-		/* render a new pixmap */
-		g_object_unref(current_theme->terrain_tiles[i]);
-		gdk_pixbuf_render_pixmap_and_mask(current_theme->scaledata
-						  [i].image,
-						  &
-						  (current_theme->
-						   terrain_tiles[i]), NULL,
-						  1);
 	}
 }
 
@@ -535,7 +488,7 @@ static gint getvar(gchar ** p, const gchar * filename, int lno)
 {
 	char *q, qsave;
 	struct tvars *tv;
-	gint idx;
+	guint idx;
 	gboolean found;
 
 	*p += strspn(*p, " \t");
@@ -629,7 +582,8 @@ static MapTheme *theme_config_parse(const gchar * themename,
 	struct tvars *tv;
 	gboolean ok = TRUE;
 	gboolean *used;
-	gint idx;
+	guint idx;
+	gint var;
 	gchar *filename;
 
 	filename = g_build_filename(subdir, "theme.cfg", NULL);
@@ -654,12 +608,13 @@ static MapTheme *theme_config_parse(const gchar * themename,
 			continue;
 		}
 		p = line;
-		idx = getvar(&p, filename, lno);
-		if ((idx == -1) || !(q = getval(&p, filename, lno))) {
+		var = getvar(&p, filename, lno);
+		if ((var == -1) || !(q = getval(&p, filename, lno))) {
 			ok = FALSE;
 			g_free(line);
 			continue;
 		}
+		idx = var;
 		tv = &theme_vars[idx];
 
 		switch (tv->type) {
@@ -750,38 +705,7 @@ void theme_register_callback(GCallback callback)
 	callback_list = g_list_append(callback_list, callback);
 }
 
-GdkPixmap *theme_get_terrain_pixmap(Terrain terrain)
+GdkPixbuf *theme_get_terrain_pixbuf(Terrain terrain)
 {
 	return theme_get_current()->terrain_tiles[terrain];
 }
-
-gint expose_terrain_cb(GtkWidget * area,
-		       G_GNUC_UNUSED GdkEventExpose * event,
-		       gpointer terraindata)
-{
-	MapTheme *theme = theme_get_current();
-	cairo_t *cr;
-	GdkPixbuf *p;
-	gint height;
-	GtkAllocation allocation;
-	Terrain terrain = GPOINTER_TO_INT(terraindata);
-
-	if (gtk_widget_get_window(area) == NULL)
-		return FALSE;
-
-	cr = gdk_cairo_create(gtk_widget_get_window(area));
-
-	gtk_widget_get_allocation(area, &allocation);
-	height = allocation.width / theme->scaledata[terrain].aspect;
-	p = gdk_pixbuf_scale_simple(theme->scaledata[terrain].native_image,
-				    allocation.width, height,
-				    GDK_INTERP_BILINEAR);
-
-	gdk_cairo_set_source_pixbuf(cr, p, 0, 0);
-	cairo_rectangle(cr, 0, 0, allocation.width, height);
-	cairo_fill(cr);
-
-	g_object_unref(p);
-	cairo_destroy(cr);
-	return FALSE;
-}
