@@ -27,6 +27,7 @@
 #include <glib.h>
 
 #include "game.h"
+#include "random.h"
 #include "map.h"
 
 /* The numbering of the hexes, nodes and edges:
@@ -241,7 +242,53 @@ static void set_edge_hex(Hex * hex, int dir, Hex * new_hex)
 	get_edge(hex, dir)->hexes[(dir + 3) / 3 % 2] = new_hex;
 }
 
-GRand *g_rand_ctx = NULL;
+Resource terrain_to_resource(Terrain terrain)
+{
+	switch (terrain) {
+	case HILL_TERRAIN:
+		return BRICK_RESOURCE;
+	case FIELD_TERRAIN:
+		return GRAIN_RESOURCE;
+	case MOUNTAIN_TERRAIN:
+		return ORE_RESOURCE;
+	case PASTURE_TERRAIN:
+		return WOOL_RESOURCE;
+	case FOREST_TERRAIN:
+		return LUMBER_RESOURCE;
+	case DESERT_TERRAIN:
+		return NO_RESOURCE;
+	case SEA_TERRAIN:
+		return NO_RESOURCE;
+	case GOLD_TERRAIN:
+		return GOLD_RESOURCE;
+	case LAST_TERRAIN:
+		return NO_RESOURCE;
+	}
+	g_assert_not_reached();
+}
+
+Terrain resource_to_terrain(Resource resource)
+{
+	switch (resource) {
+	case BRICK_RESOURCE:
+		return HILL_TERRAIN;
+	case GRAIN_RESOURCE:
+		return FIELD_TERRAIN;
+	case ORE_RESOURCE:
+		return MOUNTAIN_TERRAIN;
+	case WOOL_RESOURCE:
+		return PASTURE_TERRAIN;
+	case LUMBER_RESOURCE:
+		return FOREST_TERRAIN;
+	case NO_RESOURCE:
+		return SEA_TERRAIN;
+	case ANY_RESOURCE:
+		return SEA_TERRAIN;
+	case GOLD_RESOURCE:
+		return GOLD_TERRAIN;
+	}
+	g_assert_not_reached();
+}
 
 Hex *map_hex(Map * map, gint x, gint y)
 {
@@ -508,11 +555,11 @@ static gboolean connect_network(Hex * hex, G_GNUC_UNUSED gpointer closure)
 static gboolean layout_chits(Map * map)
 {
 	Hex **hexes;
-	gint num_chits;
+	guint num_chits;
 	gint x, y;
-	gint idx;
-	gint chit_idx;
-	gint num_deserts;
+	guint idx;
+	guint chit_idx;
+	guint num_deserts;
 
 	g_return_val_if_fail(map != NULL, FALSE);
 	g_return_val_if_fail(map->chits != NULL, FALSE);
@@ -525,8 +572,9 @@ static gboolean layout_chits(Map * map)
 	for (x = 0; x < map->x_size; x++)
 		for (y = 0; y < map->y_size; y++) {
 			Hex *hex = map->grid[y][x];
-			if (hex != NULL && hex->chit_pos >= num_chits)
-				num_chits = hex->chit_pos + 1;
+			if (hex != NULL
+			    && hex->chit_pos >= (gint) num_chits)
+				num_chits = (guint) (hex->chit_pos + 1);
 			if (hex != NULL && hex->terrain == DESERT_TERRAIN)
 				num_deserts++;
 		}
@@ -550,7 +598,7 @@ static gboolean layout_chits(Map * map)
 
 	/* Check the number of chits */
 	if (num_chits < map->chits->len + num_deserts) {
-		g_warning("More chits (%d + %d) than available tiles (%d)",
+		g_warning("More chits (%u + %u) than available tiles (%u)",
 			  map->chits->len, num_deserts, num_chits);
 		return FALSE;
 	}
@@ -636,16 +684,14 @@ void map_shuffle_terrain(Map * map)
 		for (y = 0; y < map->y_size; y++) {
 			Hex *hex = map->grid[y][x];
 			gint num;
-			gint idx;
+			guint idx;
 
 			if (hex == NULL || hex->shuffle == FALSE)
 				continue;
 			if (hex->terrain == SEA_TERRAIN) {
 				if (hex->resource == NO_RESOURCE)
 					continue;
-				num =
-				    g_rand_int_range(g_rand_ctx, 0,
-						     num_port);
+				num = random_guint(num_port);
 				for (idx = 0;
 				     idx < G_N_ELEMENTS(port_count);
 				     idx++) {
@@ -657,8 +703,7 @@ void map_shuffle_terrain(Map * map)
 				num_port--;
 				hex->resource = idx;
 			} else {
-				num = g_rand_int_range(g_rand_ctx, 0,
-						       num_terrain);
+				num = random_guint(num_terrain);
 				for (idx = 0;
 				     idx < G_N_ELEMENTS(terrain_count);
 				     idx++) {
@@ -760,7 +805,7 @@ static Hex *copy_hex(Map * map, const Hex * hex)
 
 static gboolean set_nosetup_nodes(const Hex * hex, gpointer closure)
 {
-	gint idx;
+	guint idx;
 	Map *copy = closure;
 	for (idx = 0; idx < G_N_ELEMENTS(hex->nodes); ++idx) {
 		const Node *node = hex->nodes[idx];
@@ -779,7 +824,7 @@ static gboolean set_nosetup_nodes(const Hex * hex, gpointer closure)
 static GArray *copy_int_list(GArray * array)
 {
 	GArray *copy;
-	int idx;
+	guint idx;
 
 	if (array == NULL) {
 		return NULL;
@@ -900,7 +945,7 @@ gchar *map_format_line(Map * map, gboolean write_secrets, gint y)
 				case GOLD_RESOURCE:
 					g_assert_not_reached();
 				}
-				*bufferpos++ = hex->facing + '0';
+				*bufferpos++ = (gchar) (hex->facing + '0');
 				break;
 			case LAST_TERRAIN:
 				*bufferpos++ = '-';
@@ -1100,8 +1145,9 @@ gboolean map_parse_finish(Map * map)
  */
 static void hex_free(Hex * hex)
 {
-	g_assert(hex != NULL);
 	gint idx;
+
+	g_assert(hex != NULL);
 	/* Transfer ownership of edges to adjacent hexes. */
 	for (idx = 0; idx < 6; idx++) {
 		Edge *edge = get_edge(hex, idx);
@@ -1433,6 +1479,9 @@ void map_modify_column_count(Map * map, MapModify type,
  */
 static Hex *move_hex(Hex * hex, HexDirection direction)
 {
+	Hex *ret_hex;
+	int idx;
+
 	if (hex->map->grid[hex->y][hex->x] == hex) {
 		hex->map->grid[hex->y][hex->x] = NULL;
 	};
@@ -1468,17 +1517,16 @@ static Hex *move_hex(Hex * hex, HexDirection direction)
 		hex->y++;
 		break;
 	}
-	Hex *ret_hex = map_hex(hex->map, hex->x, hex->y);
+	ret_hex = map_hex(hex->map, hex->x, hex->y);
 
 	hex->map->grid[hex->y][hex->x] = hex;
-	int idx;
 	for (idx = 0; idx < 6; idx++) {
 		Edge *edge = hex->edges[idx];
+		Node *node = hex->nodes[idx];
 		if (edge != NULL && edge->pos == idx) {
 			edge->x = hex->x;
 			edge->y = hex->y;
 		};
-		Node *node = hex->nodes[idx];
 		if (node != NULL && node->pos == idx) {
 			node->x = hex->x;
 			node->y = hex->y;
