@@ -57,6 +57,19 @@ typedef enum {
 	GOLD_RESOURCE		/* Gold */
 } Resource;
 
+/** Maps a Terrain to a Resource.
+ * This does not take ports into account.
+ * @param terrain The terrain
+ * @return The resource the terrain produces
+ */
+Resource terrain_to_resource(Terrain terrain);
+
+/** Maps a Resource to a Terrain.
+ * @param resource The resource
+ * @return The terrain that produces this resource
+ */
+Terrain resource_to_terrain(Resource resource);
+
 /* Types of structure that can be built
  */
 typedef enum {
@@ -282,11 +295,9 @@ gboolean map_bridge_vacant(Map * map, gint x, gint y, gint pos);
 gboolean map_bridge_connect_ok(const Map * map, gint owner, gint x, gint y,
 			       gint pos);
 /* information gathering */
-void map_longest_road(Map * map, gint * lengths, gint num_players);
+void map_longest_road(Map * map, guint * lengths, guint num_players);
 gboolean map_is_island_discovered(Map * map, Node * node, gint owner);
 void map_maritime_info(const Map * map, MaritimeInfo * info, gint owner);
 guint map_count_islands(const Map * map);
 
-extern GRand *g_rand_ctx;
-
 #endif
