@@ -31,13 +31,11 @@ typedef enum {
 	NET_READ
 } NetEvent;
 
-typedef void (*NetNotifyFunc) (NetEvent event, void *user_data,
-			       gchar * line);
-
+typedef struct _Service Service;
 typedef struct _Session Session;
 
-void set_enable_debug(gboolean enabled);
-void debug(const gchar * fmt, ...);
+typedef void (*NetNotifyFunc) (Session * ses, NetEvent event,
+			       const gchar * line, gpointer user_data);
 
 /** Initialize the network drivers */
 void net_init(void);
@@ -45,50 +43,59 @@ void net_init(void);
 /* Finish the network drivers */
 void net_finish(void);
 
-Session *net_new(NetNotifyFunc notify_func, void *user_data);
+Session *net_new(NetNotifyFunc notify_func, gpointer user_data);
 void net_free(Session ** ses);
 
-void net_use_fd(Session * ses, int fd, gboolean do_ping);
+void net_set_user_data(Session * ses, gpointer user_data);
+void net_set_notify_func(Session * ses, NetNotifyFunc notify_func,
+			 gpointer user_data);
+
 gboolean net_connect(Session * ses, const gchar * host,
 		     const gchar * port);
 gboolean net_connected(Session * ses);
 
-/** Open a socket for listening.
+/** Check whether the connection is alive by sending messages.
+ * @param ses The session
+ * @param period Interval in seconds, use 0 to stop checking
+ */
+void net_set_check_connection_alive(Session * ses, guint period);
+
+/** Check whether the connection is has timed out.
+ * @param ses The session
+ * @return TRUE when the connection timed out
+ */
+gboolean net_get_connection_timed_out(Session * ses);
+
+/** Create a service that listens on the mentioned port.
  *  @param port The port
+ *  @param notify_func The notification function for new Sessions
+ *  @param user_data The user data in new Sessions
  *  @retval error_message If opening fails, a description of the error
  *          You should g_free the error_message
- *  @return A file descriptor if succesfull, or -1 if it fails
+ *  @return The service, or NULL on error
  */
-int net_open_listening_socket(const gchar * port, gchar ** error_message);
+Service *net_service_new(guint16 port, NetNotifyFunc notify_func,
+			 gpointer user_data, gchar ** error_message);
 
-/** Close a socket
+/** Free the service that was created by net_service_new.
+ *  @param service The service to free
  */
-void net_closesocket(int fd);
+void net_service_free(Service * service);
 
 /** Get peer name
- *  @param fd File descriptor to resolve
- *  @retval hostname The resolved hostname
- *  @retval servname The resolved port name/service name
- *  @retval error_message The error message when it fails
+ *  @param ses The session
+ *  @retval hostname The resolved hostname (free with g_free)
+ *  @retval servname The resolved port name/service name (free with g_free)
+ *  @retval error The error when it fails, or NULL to ignore
  *  @return TRUE is successful
  */
-gboolean net_get_peer_name(gint fd, gchar ** hostname, gchar ** servname,
-			   gchar ** error_message);
+gboolean net_get_peer_name(Session * ses, gchar ** hostname,
+			   gchar ** servname, GError ** error);
 
-/** Accept incoming connections
- * @param accept_fd The file descriptor
- * @retval error_message The message if it fails
- * @return The file descriptor of the connection
- */
-gint net_accept(gint accept_fd, gchar ** error_message);
-
-/** Close a session
+/** Close a session after the pending data was sent.
  * @param ses The session to close
- * @return TRUE if the session can be removed
  */
-gboolean net_close(Session * ses);
-void net_close_when_flushed(Session * ses);
-void net_wait_for_close(Session * ses);
+void net_close(Session * ses);
 void net_printf(Session * ses, const gchar * fmt, ...);
 
 /** Write data.
@@ -97,20 +104,15 @@ void net_printf(Session * ses, const gchar * fmt, ...);
  */
 void net_write(Session * ses, const gchar * data);
 
-/** Get the hostname of this computer.
- * @return "localhost" if the hostname could not be determined.
- */
-gchar *get_my_hostname(void);
-
-/** Get the name of the meta server.
- *  First the environment variable PIONEERS_META_SERVER is queried
+/** Get the name of the metaserver.
+ *  First the environment variable PIONEERS_METASERVER is queried
  *  If it is not set, the use_default flag is used.
- * @param use_default If true, return the default meta server if the
+ * @param use_default If true, return the default metaserver if the
  *                    environment variable is not set.
  *                    If false, return the hostname of this computer.
- * @return The hostname of the meta server
+ * @return The hostname of the metaserver
  */
-gchar *get_meta_server_name(gboolean use_default);
+gchar *get_metaserver_name(gboolean use_default);
 
 /** Get the directory of the game related files.
  *  First the environment variable PIONEERS_DIR is queried
