@@ -31,6 +31,33 @@
 #include "game.h"
 #include "log.h"
 #include "state.h"
+#include "network.h"
+
+struct StateMachine {
+	gpointer user_data;	/* parameter for mode functions */
+	/* FIXME RC 2004-11-13 in practice: 
+	 * it is NULL or a Player*
+	 * the value is set by sm_new.
+	 * Why? Can the player not be bound to a 
+	 * StateMachine otherwise? */
+
+	StateFunc global;	/* global state - test after current state */
+	StateFunc unhandled;	/* global state - process unhandled states */
+	StateFunc stack[16];	/* handle sm_push() to save context */
+	const gchar *stack_name[16];	/* state names used for a stack dump */
+	gint stack_ptr;		/* stack index */
+	const gchar *current_state;	/* name of current state */
+
+	gchar *line;		/* line passed in from network event */
+	size_t line_offset;	/* line prefix handling */
+
+	Session *ses;		/* network session feeding state machine */
+	gint use_count;		/* # functions is in use by */
+	gboolean is_dead;	/* is this machine waiting to be killed? */
+
+	gboolean use_cache;	/* cache the data that is sent */
+	GList *cache;		/* cache for the delayed data */
+};
 
 static void route_event(StateMachine * sm, gint event);
 
@@ -126,8 +153,13 @@ void sm_cancel_prefix(StateMachine * sm)
 	sm->line_offset = 0;
 }
 
-static void net_event(NetEvent event, StateMachine * sm, gchar * line)
+static void net_event(Session * ses, NetEvent event, const gchar * line,
+		      gpointer user_data)
 {
+	StateMachine *sm = (StateMachine *) user_data;
+
+	g_assert(ses == sm->ses);
+
 	sm_inc_use_count(sm);
 
 	switch (event) {
@@ -141,7 +173,9 @@ static void net_event(NetEvent event, StateMachine * sm, gchar * line)
 		route_event(sm, SM_NET_CLOSE);
 		break;
 	case NET_READ:
-		sm->line = line;
+		if (sm->line)
+			g_free(sm->line);
+		sm->line = g_strdup(line);
 		/* Only handle data if there is a context.  Fixes bug that
 		 * clients starting to send data immediately crash the
 		 * server */
@@ -164,7 +198,7 @@ gboolean sm_connect(StateMachine * sm, const gchar * host,
 	if (sm->ses != NULL)
 		net_free(&(sm->ses));
 
-	sm->ses = net_new((NetNotifyFunc) net_event, sm);
+	sm->ses = net_new(net_event, sm);
 	log_message(MSG_INFO, _("Connecting to %s, port %s\n"), host,
 		    port);
 	if (net_connect(sm->ses, host, port))
@@ -174,31 +208,33 @@ gboolean sm_connect(StateMachine * sm, const gchar * host,
 	return FALSE;
 }
 
-void sm_use_fd(StateMachine * sm, gint fd, gboolean do_ping)
+void sm_set_session(StateMachine * sm, Session * ses)
 {
+	sm_inc_use_count(sm);
 	if (sm->ses != NULL)
 		net_free(&(sm->ses));
-
-	sm->ses = net_new((NetNotifyFunc) net_event, sm);
-	net_use_fd(sm->ses, fd, do_ping);
-}
+	sm->ses = ses;
+	net_set_notify_func(sm->ses, net_event, sm);
+	sm_dec_use_count(sm);
+};
 
 gboolean sm_recv(StateMachine * sm, const gchar * fmt, ...)
 {
 	va_list ap;
-	gint offset;
+	ssize_t offset;
 
 	va_start(ap, fmt);
 	offset = game_vscanf(sm->line + sm->line_offset, fmt, ap);
 	va_end(ap);
 
-	return offset > 0 && sm->line[sm->line_offset + offset] == '\0';
+	return offset > 0
+	    && sm->line[sm->line_offset + (size_t) offset] == '\0';
 }
 
 gboolean sm_recv_prefix(StateMachine * sm, const gchar * fmt, ...)
 {
 	va_list ap;
-	gint offset;
+	ssize_t offset;
 
 	va_start(ap, fmt);
 	offset = game_vscanf(sm->line + sm->line_offset, fmt, ap);
@@ -206,7 +242,7 @@ gboolean sm_recv_prefix(StateMachine * sm, const gchar * fmt, ...)
 
 	if (offset < 0)
 		return FALSE;
-	sm->line_offset += offset;
+	sm->line_offset += (size_t) offset;
 	return TRUE;
 }
 
@@ -216,7 +252,7 @@ void sm_write(StateMachine * sm, const gchar * str)
 		/* Protect against strange/slow connects */
 		if (g_list_length(sm->cache) > 1000) {
 			net_write(sm->ses, "ERR connection too slow\n");
-			net_close_when_flushed(sm->ses);
+			net_close(sm->ses);
 		} else {
 			sm->cache =
 			    g_list_append(sm->cache, g_strdup(str));
@@ -271,6 +307,11 @@ void sm_set_use_cache(StateMachine * sm, gboolean use_cache)
 	sm->use_cache = use_cache;
 }
 
+gboolean sm_get_use_cache(const StateMachine * sm)
+{
+	return sm->use_cache;
+}
+
 void sm_global_set(StateMachine * sm, StateFunc state)
 {
 	sm->global = state;
@@ -285,7 +326,7 @@ static void push_new_state(StateMachine * sm)
 {
 	++sm->stack_ptr;
 	/* check for stack overflows */
-	if (sm->stack_ptr >= G_N_ELEMENTS(sm->stack)) {
+	if (sm->stack_ptr >= (gint) G_N_ELEMENTS(sm->stack)) {
 		log_message(MSG_ERROR,
 			    /* Error message */
 			    _(""
@@ -416,8 +457,8 @@ void sm_pop_all_and_goto(StateMachine * sm, StateFunc new_state)
  */
 StateFunc sm_stack_inspect(const StateMachine * sm, guint offset)
 {
-	if (sm->stack_ptr >= offset)
-		return sm->stack[sm->stack_ptr - offset];
+	if (sm->stack_ptr >= (gint) offset)
+		return sm->stack[(guint) sm->stack_ptr - offset];
 	else
 		return NULL;
 }
@@ -445,6 +486,8 @@ StateMachine *sm_new(gpointer user_data)
  */
 void sm_free(StateMachine * sm)
 {
+	g_free(sm->line);
+	sm->line = NULL;
 	if (sm->ses != NULL) {
 		net_free(&(sm->ses));
 		return;
@@ -473,10 +516,19 @@ void sm_close(StateMachine * sm)
 	}
 }
 
-void sm_stack_dump(StateMachine * sm)
+void sm_copy_stack(StateMachine * dest, const StateMachine * src)
+{
+	memcpy(dest->stack, src->stack, sizeof(dest->stack));
+	memcpy(dest->stack_name, src->stack_name,
+	       sizeof(dest->stack_name));
+	dest->stack_ptr = src->stack_ptr;
+	dest->current_state = src->current_state;
+}
+
+void sm_stack_dump(const StateMachine * sm)
 {
 	gint sp;
-	fprintf(stderr, "Stack dump for %p\n", sm);
+	fprintf(stderr, "Stack dump for %p\n", (const void *) sm);
 	for (sp = 0; sp <= sm->stack_ptr; ++sp) {
 		fprintf(stderr, "Stack %2d: %s\n", sp, sm->stack_name[sp]);
 	}
