@@ -26,11 +26,11 @@
 void check_longest_road(Game * game)
 {
 	Map *map = game->params->map;
-	gint road_length[MAX_PLAYERS];
+	guint road_length[MAX_PLAYERS];
 	gint num_have_longest;
-	gint longest_length;
+	guint longest_length;
 	gboolean tie;
-	gint i;
+	guint i;
 
 	map_longest_road(map, road_length, game->params->num_players);
 
@@ -40,15 +40,16 @@ void check_longest_road(Game * game)
 	for (i = 0; i < game->params->num_players; i++) {
 		if (road_length[i] >= 5) {
 			if (road_length[i] > longest_length) {
-				num_have_longest = i;
+				num_have_longest = (gint) i;
 				longest_length = road_length[i];
 				tie = FALSE;
 			} else if (road_length[i] == longest_length) {
 				tie = TRUE;
 				if (game->longest_road != NULL
-				    && i == game->longest_road->num) {
+				    && (gint) i ==
+				    game->longest_road->num) {
 					/* Current owner in the tie */
-					num_have_longest = i;
+					num_have_longest = (gint) i;
 				}
 			}
 		}
