@@ -33,6 +33,7 @@
 #include "authors.h"
 #include "aboutbox.h"
 #include "game.h"
+#include "game-list.h"
 #include "common_gtk.h"
 
 #include "config-gnome.h"
@@ -44,8 +45,6 @@
 #include "theme.h"
 #include "metaserver.h"		/* Custom widget */
 
-#include "avahi.h"
-
 #define MAINICON_FILE	"pioneers-server.png"
 
 static GtkWidget *settings_notebook;	/* relevant settings */
@@ -54,17 +53,16 @@ static GtkWidget *select_game;	/* select game type */
 static GtkWidget *game_settings;	/* the settings of the game */
 static GtkWidget *game_rules;	/* the rules of the game */
 
-static GtkWidget *meta_entry;	/* name of meta server */
+static GtkWidget *meta_entry;	/* name of metaserver */
 static GtkWidget *overridden_hostname_entry;	/* name of server (allows masquerading) */
 
 static GtkWidget *start_btn;	/* start/stop the server */
 
 static GtkListStore *store;	/* shows player connection status */
-static Game *game = NULL;	/* the current game */
 
 static gchar *overridden_hostname;	/* override reported hostname */
 static gchar *server_port = NULL;	/* port of the game */
-static gboolean register_server = TRUE;	/* Register at the meta server */
+static gboolean register_server = TRUE;	/* Register at the metaserver */
 static gboolean want_ai_chat = TRUE;
 static gboolean random_order = TRUE;	/* random seating order */
 static gboolean enable_debug = FALSE;
@@ -176,20 +174,28 @@ static void game_settings_change_cb(GameSettings *gs, G_GNUC_UNUSED gpointer use
 
 static void update_game_settings(const GameParams * params)
 {
-	g_return_if_fail(params != NULL);
+	if (params == NULL) {
+		return;
+	}
 
 	/* Update the UI */
 	game_settings_set_players(GAMESETTINGS(game_settings),
-				  params->num_players);
+				  (guint) params->num_players);
 	game_settings_set_victory_points(GAMESETTINGS(game_settings),
-					 params->victory_points);
+					 (guint) params->victory_points);
 	game_rules_set_victory_at_end_of_turn(GAMERULES(game_rules),
 					      params->
 					      check_victory_at_end_of_turn);
 	game_rules_set_random_terrain(GAMERULES(game_rules),
 				      params->random_terrain);
 	game_rules_set_sevens_rule(GAMERULES(game_rules),
-				   params->sevens_rule);
+				   (guint) params->sevens_rule);
+	game_rules_set_use_dice_deck(GAMERULES(game_rules),
+				     params->use_dice_deck);
+	game_rules_set_num_dice_decks(GAMERULES(game_rules),
+				      params->num_dice_decks);
+	game_rules_set_num_removed_dice_cards(GAMERULES(game_rules),
+					      params->num_removed_dice_cards);
 	game_rules_set_domestic_trade(GAMERULES(game_rules),
 				      params->domestic_trade);
 	game_rules_set_strict_trade(GAMERULES(game_rules),
@@ -204,11 +210,9 @@ static void update_game_settings(const GameParams * params)
 static void game_activate(GtkWidget * widget,
 			  G_GNUC_UNUSED gpointer user_data)
 {
-	const gchar *title;
 	const GameParams *params;
 
-	title = select_game_get_active(SELECTGAME(widget));
-	params = game_list_find_item(title);
+	params = select_game_get_active_game(SELECTGAME(widget));
 	update_game_settings(params);
 }
 
@@ -234,31 +238,41 @@ static void gui_set_server_state(gboolean running)
 }
 
 static void start_clicked_cb(G_GNUC_UNUSED GtkButton * widget,
-			     G_GNUC_UNUSED gpointer user_data)
+			     gpointer user_data)
 {
-	if (server_is_running(game)) {
-		server_stop(game);
-		gui_set_server_state(server_is_running(game));
-		avahi_unregister_game();
-	} else {		/* not running */
-		const gchar *title;
-		GameParams *params;
-		gchar *meta_server_name;
+	Game **game = user_data;
 
-		title = select_game_get_active(SELECTGAME(select_game));
-		params = params_copy(game_list_find_item(title));
-		cfg_set_num_players(params,
+	if (server_is_running(*game)) {
+		server_stop(*game);
+		gui_set_server_state(server_is_running(*game));
+	} else {		/* not running */
+		GameParams *params;
+		gchar *metaserver_name;
+
+		params =
+		    params_copy(select_game_get_active_game
+				(SELECTGAME(select_game)));
+		cfg_set_num_players(params, (gint)
 				    game_settings_get_players(GAMESETTINGS
 							      (game_settings)));
-		cfg_set_victory_points(params,
+		cfg_set_victory_points(params, (gint)
 				       game_settings_get_victory_points
 				       (GAMESETTINGS(game_settings)));
 		params->check_victory_at_end_of_turn =
 		    game_rules_get_victory_at_end_of_turn(GAMERULES
 							  (game_rules));
-		cfg_set_sevens_rule(params,
+		cfg_set_sevens_rule(params, (gint)
 				    game_rules_get_sevens_rule(GAMERULES
 							       (game_rules)));
+		cfg_set_use_dice_deck(params,
+				      game_rules_get_use_dice_deck
+				      (GAMERULES(game_rules)));
+		cfg_set_num_dice_decks(params,
+				       game_rules_get_num_dice_decks
+				       (GAMERULES(game_rules)));
+		cfg_set_num_removed_dice_cards(params,
+					       game_rules_get_num_removed_dice_cards
+					       (GAMERULES(game_rules)));
 		cfg_set_terrain_type(params,
 				     game_rules_get_random_terrain
 				     (GAMERULES(game_rules)));
@@ -276,21 +290,20 @@ static void start_clicked_cb(G_GNUC_UNUSED GtkButton * widget,
 							  (game_rules));
 		update_game_settings(params);
 
-		meta_server_name = metaserver_get(METASERVER(meta_entry));
+		metaserver_name = metaserver_get(METASERVER(meta_entry));
 
 		g_assert(server_port != NULL);
 
-		if (game != NULL)
-			game_free(game);
-		game =
+		if (*game != NULL)
+			game_free(*game);
+		*game =
 		    server_start(params, overridden_hostname, server_port,
-				 register_server, meta_server_name,
+				 register_server, metaserver_name,
 				 random_order);
-		if (server_is_running(game)) {
-			avahi_register_game(game);
+		if (server_is_running(*game)) {
 			gui_set_server_state(TRUE);
-			config_set_string("server/meta-server",
-					  meta_server_name);
+			config_set_string("server/metaserver",
+					  metaserver_name);
 			config_set_string("server/port", server_port);
 			config_set_int("server/register", register_server);
 			config_set_string("server/overridden-hostname",
@@ -304,12 +317,18 @@ static void start_clicked_cb(G_GNUC_UNUSED GtkButton * widget,
 			config_set_int("game/num-players",
 				       params->num_players);
 			config_set_int("game/victory-points",
-				       params->victory_points);
+				       (gint) params->victory_points);
 			config_set_int("game/check-victory-at-end-of-turn",
 				       params->
 				       check_victory_at_end_of_turn);
 			config_set_int("game/sevens-rule",
 				       params->sevens_rule);
+			config_set_int("game/use_dice-deck",
+				       params->use_dice_deck);
+			config_set_int("game/num-dice-decks",
+				       params->num_dice_decks);
+			config_set_int("game/num-removed_cards",
+				       params->num_removed_dice_cards);
 			config_set_int("game/use-pirate",
 				       params->use_pirate);
 			config_set_int("game/strict-trade",
@@ -318,7 +337,7 @@ static void start_clicked_cb(G_GNUC_UNUSED GtkButton * widget,
 				       params->domestic_trade);
 		}
 		params_free(params);
-		g_free(meta_server_name);
+		g_free(metaserver_name);
 	}
 }
 
@@ -327,15 +346,15 @@ static void launchclient_clicked_cb(G_GNUC_UNUSED GtkButton * widget,
 {
 	gchar *child_argv[7];
 	GSpawnFlags child_flags = G_SPAWN_STDOUT_TO_DEV_NULL |
-	    G_SPAWN_STDERR_TO_DEV_NULL;
+	    G_SPAWN_STDERR_TO_DEV_NULL | G_SPAWN_SEARCH_PATH;
 	GError *error = NULL;
 	gint i;
 
 	g_assert(server_port != NULL);
 
 	/* Populate argv. */
-	child_argv[0] = g_strdup(PIONEERS_CLIENT_GTK_PATH);
-	child_argv[1] = g_strdup(PIONEERS_CLIENT_GTK_PATH);
+	child_argv[0] = g_strdup(PIONEERS_CLIENT_GTK_PROGRAM_NAME);
+	child_argv[1] = g_strdup(PIONEERS_CLIENT_GTK_PROGRAM_NAME);
 	child_argv[2] = g_strdup("-s");
 	child_argv[3] = g_strdup("localhost");
 	child_argv[4] = g_strdup("-p");
@@ -348,7 +367,7 @@ static void launchclient_clicked_cb(G_GNUC_UNUSED GtkButton * widget,
 		/* Error message when program %1 is started, reason is %2 */
 		log_message(MSG_ERROR,
 			    _("Error starting %s: %s\n"),
-			    PIONEERS_CLIENT_GTK_PATH, error->message);
+			    child_argv[0], error->message);
 		g_error_free(error);
 	}
 
@@ -359,10 +378,12 @@ static void launchclient_clicked_cb(G_GNUC_UNUSED GtkButton * widget,
 }
 
 static void addcomputer_clicked_cb(G_GNUC_UNUSED GtkButton * widget,
-				   G_GNUC_UNUSED gpointer user_data)
+				   gpointer user_data)
 {
+	Game **game = user_data;
+
 	g_assert(server_port != NULL);
-	add_computer_player(game, want_ai_chat);
+	add_computer_player(*game, want_ai_chat);
 	config_set_int("ai/enable-chat", want_ai_chat);
 }
 
@@ -388,8 +409,10 @@ static void gui_player_rename(void *data)
 		    player->name);
 }
 
-static gboolean everybody_left(G_GNUC_UNUSED gpointer data)
+static gboolean everybody_left(gpointer data)
 {
+	Game *game = data;
+
 	server_stop(game);
 	gui_set_server_state(server_is_running(game));
 	return FALSE;
@@ -425,7 +448,7 @@ static void gui_player_change(void *data)
 	}
 	playerlist_dec_use_count(game);
 	if (number_of_players == 0 && game->is_game_over) {
-		g_timeout_add(100, everybody_left, NULL);
+		g_timeout_add(100, everybody_left, game);
 	}
 }
 
@@ -433,8 +456,7 @@ static void add_game_to_list(gpointer name,
 			     G_GNUC_UNUSED gpointer user_data)
 {
 	GameParams *a = (GameParams *) name;
-	select_game_add_with_map(SELECTGAME(select_game), a->title,
-				 a->map);
+	select_game_add_details(SELECTGAME(select_game), a);
 }
 
 static void overridden_hostname_changed_cb(GtkEntry * widget,
@@ -459,7 +481,7 @@ static GtkWidget *build_game_settings(GtkWindow * main_window)
 {
 	GtkWidget *vbox;
 
-	vbox = gtk_vbox_new(FALSE, 3);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 3);
 	gtk_widget_show(vbox);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 3);
 
@@ -487,7 +509,7 @@ static GtkWidget *build_game_rules(void)
 	GtkWidget *vbox;
 
 
-	vbox = gtk_vbox_new(FALSE, 3);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 3);
 	gtk_widget_show(vbox);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 3);
 
@@ -507,7 +529,7 @@ static GtkWidget *build_server_frame(void)
 	/*       server port label */
 	/*        - port entry */
 	/*       register toggle */
-	/*       meta server label */
+	/*       metaserver label */
 	/*        - meta entry */
 	/*       hostname label */
 	/*        - hostname entry */
@@ -519,6 +541,7 @@ static GtkWidget *build_server_frame(void)
 	GtkWidget *port_entry;
 
 	gint novar;
+	gchar *metaserver_name;
 
 	/* table */
 	table = gtk_table_new(6, 2, FALSE);
@@ -561,12 +584,12 @@ static GtkWidget *build_server_frame(void)
 			 G_CALLBACK(register_toggle_cb), NULL);
 	gtk_widget_set_tooltip_text(toggle,
 				    _(""
-				      "Register this game at the meta server"));
+				      "Register this game at the metaserver"));
 	register_server =
 	    config_get_int_with_default("server/register", TRUE);
 
-	/* meta server label */
-	label = gtk_label_new(_("Meta server"));
+	/* metaserver label */
+	label = gtk_label_new(_("Metaserver"));
 	gtk_widget_show(label);
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
 			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
@@ -582,14 +605,13 @@ static GtkWidget *build_server_frame(void)
 
 	/* initialize meta entry */
 	novar = 0;
-	gchar *meta_server_name;
-	meta_server_name = config_get_string("server/meta-server", &novar);
-	if (novar || !strlen(meta_server_name)
-	    || !strncmp(meta_server_name, "gnocatan.debian.net",
-			strlen(meta_server_name) + 1))
-		meta_server_name = get_meta_server_name(TRUE);
-	metaserver_add(METASERVER(meta_entry), meta_server_name);
-	g_free(meta_server_name);
+	metaserver_name = config_get_string("server/metaserver", &novar);
+	if (novar || !strlen(metaserver_name)
+	    || !strncmp(metaserver_name, "gnocatan.debian.net",
+			strlen(metaserver_name) + 1))
+		metaserver_name = get_metaserver_name(TRUE);
+	metaserver_add(METASERVER(meta_entry), metaserver_name);
+	g_free(metaserver_name);
 
 	/* hostname label */
 	label = gtk_label_new(_("Reported hostname"));
@@ -786,9 +808,10 @@ static GtkWidget *build_player_connected_frame(void)
 
 	GtkWidget *vbox;
 	GtkWidget *button;
+	gchar *fullname;
 
 	/* vbox */
-	vbox = gtk_vbox_new(FALSE, 5);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5);
 	gtk_widget_show(vbox);
 
 	/* connected tree_view */
@@ -801,6 +824,12 @@ static GtkWidget *build_player_connected_frame(void)
 						  _(""
 						    "Launch Pioneers Client"));
 	gtk_widget_show(button);
+
+	fullname =
+	    g_find_program_in_path(PIONEERS_CLIENT_GTK_PROGRAM_NAME);
+	gtk_widget_set_sensitive(button, fullname != NULL);
+	g_free(fullname);
+
 	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(button), "clicked",
 			 G_CALLBACK(launchclient_clicked_cb), NULL);
@@ -814,7 +843,7 @@ static GtkWidget *build_player_connected_frame(void)
 /** Builds the composite ai frame widget.
  *  @return returns the composite widget.
  */
-static GtkWidget *build_ai_frame(void)
+static GtkWidget *build_ai_frame(Game ** game)
 {
 	/* ai vbox */
 	/*       ai chat toggle */
@@ -825,19 +854,14 @@ static GtkWidget *build_ai_frame(void)
 	GtkWidget *button;
 
 	gchar *fullname;
-	gboolean ai_settings_enabled = TRUE;
 
 	/* ai vbox */
-	vbox = gtk_vbox_new(FALSE, 5);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5);
 	gtk_widget_show(vbox);
 
-	fullname = g_find_program_in_path(PIONEERS_AI_PATH);
-	if (fullname) {
-		g_free(fullname);
-	} else {
-		ai_settings_enabled = FALSE;
-	}
-	gtk_widget_set_sensitive(vbox, ai_settings_enabled);
+	fullname = g_find_program_in_path(PIONEERS_AI_PROGRAM_NAME);
+	gtk_widget_set_sensitive(vbox, fullname != NULL);
+	g_free(fullname);
 
 	/* ai chat toggle */
 	toggle = gtk_check_button_new_with_label(_("Enable chat"));
@@ -861,7 +885,7 @@ static GtkWidget *build_ai_frame(void)
 	gtk_widget_show(button);
 	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(button), "clicked",
-			 G_CALLBACK(addcomputer_clicked_cb), NULL);
+			 G_CALLBACK(addcomputer_clicked_cb), game);
 	gtk_widget_set_tooltip_text(button,
 				    /* Tooltip */
 				    _(""
@@ -898,7 +922,7 @@ static GtkWidget *build_message_frame(void)
 	gtk_widget_set_tooltip_text(message_text,
 				    /* Tooltip */
 				    _("Messages from the server"));
-	message_window_set_text(message_text);
+	message_window_set_text(message_text, NULL);
 
 	return scroll_win;
 }
@@ -907,7 +931,7 @@ static GtkWidget *build_message_frame(void)
  *  @param main_window The top-level window.
  *  @return returns the composite widget.
  */
-static GtkWidget *build_interface(GtkWindow * main_window)
+static GtkWidget *build_interface(GtkWindow * main_window, Game ** game)
 {
 	GtkWidget *vbox;
 	GtkWidget *hbox_settings;
@@ -916,7 +940,7 @@ static GtkWidget *build_interface(GtkWindow * main_window)
 	GtkWidget *label_with_close_button;
 
 	/* vbox */
-	vbox = gtk_vbox_new(FALSE, 5);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5);
 	gtk_widget_show(vbox);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
 
@@ -929,14 +953,14 @@ static GtkWidget *build_interface(GtkWindow * main_window)
 			   0);
 
 	/* settings tab hbox */
-	hbox_settings = gtk_hbox_new(FALSE, 5);
+	hbox_settings = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5);
 	gtk_widget_show(hbox_settings);
 	gtk_notebook_append_page(GTK_NOTEBOOK(settings_notebook),
 				 hbox_settings,
 				 gtk_label_new(_("Game settings")));
 
 	/* left part in settings tab */
-	vbox_settings = gtk_vbox_new(FALSE, 5);
+	vbox_settings = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5);
 	gtk_widget_show(vbox_settings);
 
 	/* Game settings frame */
@@ -951,7 +975,7 @@ static GtkWidget *build_interface(GtkWindow * main_window)
 			   FALSE, 0);
 
 	/* right part in settings tab */
-	vbox_settings = gtk_vbox_new(FALSE, 5);
+	vbox_settings = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5);
 	gtk_widget_show(vbox_settings);
 
 	/* Rules frame */
@@ -972,10 +996,10 @@ static GtkWidget *build_interface(GtkWindow * main_window)
 									 "Stop the server"),
 									&stop_game_button_on_tab);
 	g_signal_connect(G_OBJECT(stop_game_button_on_tab), "clicked",
-			 G_CALLBACK(start_clicked_cb), NULL);
+			 G_CALLBACK(start_clicked_cb), game);
 
 	/* game tab vbox */
-	vbox_settings = gtk_vbox_new(FALSE, 5);
+	vbox_settings = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5);
 	gtk_notebook_append_page(GTK_NOTEBOOK(settings_notebook),
 				 vbox_settings, label_with_close_button);
 
@@ -985,7 +1009,7 @@ static GtkWidget *build_interface(GtkWindow * main_window)
 
 	/* ai frame */
 	build_frame(vbox_settings, _("Computer players"),
-		    build_ai_frame(), FALSE);
+		    build_ai_frame(game), FALSE);
 
 	/* start button */
 	start_btn = gtk_button_new();
@@ -993,7 +1017,7 @@ static GtkWidget *build_interface(GtkWindow * main_window)
 
 	gtk_box_pack_start(GTK_BOX(vbox), start_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(start_btn), "clicked",
-			 G_CALLBACK(start_clicked_cb), NULL);
+			 G_CALLBACK(start_clicked_cb), game);
 
 	/* message frame */
 	build_frame(vbox, _("Messages"), build_message_frame(), TRUE);
@@ -1044,6 +1068,15 @@ static void load_last_game_params(void)
 	temp = config_get_int("game/sevens-rule", &default_returned);
 	if (!default_returned)
 		cfg_set_sevens_rule(params, temp);
+	temp = config_get_int("game/use-dice-deck", &default_returned);
+	if (!default_returned)
+		cfg_set_use_dice_deck(params, temp);
+	temp = config_get_int("game/num-dice-decks", &default_returned);
+	if (!default_returned)
+		cfg_set_num_dice_decks(params, temp);
+	temp = config_get_int("game/num-removed_cards", &default_returned);
+	if (!default_returned)
+		cfg_set_num_removed_dice_cards(params, temp);
 	temp = config_get_int("game/use-pirate", &default_returned);
 	if (!default_returned)
 		params->use_pirate = temp;
@@ -1060,23 +1093,32 @@ static void load_last_game_params(void)
 static void check_vp_cb(G_GNUC_UNUSED GObject * caller,
 			gpointer main_window)
 {
-	const gchar *title;
 	GameParams *params;
 
-	title = select_game_get_active(SELECTGAME(select_game));
-	params = params_copy(game_list_find_item(title));
-	cfg_set_num_players(params,
+	params =
+	    params_copy(select_game_get_active_game
+			(SELECTGAME(select_game)));
+	cfg_set_num_players(params, (gint)
 			    game_settings_get_players(GAMESETTINGS
 						      (game_settings)));
-	cfg_set_victory_points(params,
+	cfg_set_victory_points(params, (gint)
 			       game_settings_get_victory_points
 			       (GAMESETTINGS(game_settings)));
 	params->check_victory_at_end_of_turn =
 	    game_rules_get_victory_at_end_of_turn(GAMERULES(game_rules));
-	cfg_set_sevens_rule(params,
+	cfg_set_sevens_rule(params, (gint)
 			    game_rules_get_sevens_rule(GAMERULES
 						       (game_rules)));
-	cfg_set_terrain_type(params,
+	cfg_set_use_dice_deck(params,
+			      game_rules_get_use_dice_deck(GAMERULES
+							   (game_rules)));
+	cfg_set_num_dice_decks(params,
+			       game_rules_get_num_dice_decks(GAMERULES
+							     (game_rules)));
+	cfg_set_num_removed_dice_cards(params,
+				       game_rules_get_num_removed_dice_cards
+				       (GAMERULES(game_rules)));
+	cfg_set_terrain_type(params, (gint)
 			     game_rules_get_random_terrain(GAMERULES
 							   (game_rules)));
 	params->strict_trade =
@@ -1098,7 +1140,6 @@ static void check_vp_cb(G_GNUC_UNUSED GObject * caller,
 static void quit_cb(void)
 {
 	gtk_main_quit();
-	avahi_unregister_game();
 }
 
 static void help_about_cb(void)
@@ -1145,6 +1186,7 @@ int main(int argc, char *argv[])
 	GtkAccelGroup *accel_group;
 	GError *error = NULL;
 	GOptionContext *context;
+	Game *game;
 
 	net_init();
 
@@ -1195,7 +1237,7 @@ int main(int argc, char *argv[])
 	/* Name in the titlebar of the server */
 	gtk_window_set_title(GTK_WINDOW(window), _("Pioneers Server"));
 
-	vbox = gtk_vbox_new(FALSE, 0);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	gtk_container_add(GTK_CONTAINER(window), vbox);
 
 	action_group = gtk_action_group_new("MenuActions");
@@ -1221,8 +1263,6 @@ int main(int argc, char *argv[])
 	config_init("pioneers-server");
 
 	themes_init();
-	game_list_prepare();
-
 	icon_file =
 	    g_build_filename(DATADIR, "pixmaps", MAINICON_FILE, NULL);
 	if (g_file_test(icon_file, G_FILE_TEST_EXISTS)) {
@@ -1235,9 +1275,17 @@ int main(int argc, char *argv[])
 	menubar = gtk_ui_manager_get_widget(ui_manager, "/MainMenu");
 	gtk_box_pack_start(GTK_BOX(vbox), menubar, FALSE, FALSE, 0);
 
+	game = NULL;
 	gtk_box_pack_start(GTK_BOX(vbox),
-			   build_interface(GTK_WINDOW(window)), TRUE, TRUE,
-			   0);
+			   build_interface(GTK_WINDOW(window), &game),
+			   TRUE, TRUE, 0);
+
+	/* in theory, all windows are created now...
+	 *   set logging to message window */
+	log_set_func_message_window();
+
+	game_list_prepare();
+
 	load_last_game_params();
 
 	gtk_widget_show_all(window);
@@ -1245,12 +1293,10 @@ int main(int argc, char *argv[])
 	g_signal_connect(G_OBJECT(window), "delete_event",
 			 G_CALLBACK(quit_cb), NULL);
 
-	/* in theory, all windows are created now...
-	 *   set logging to message window */
-	log_set_func_message_window();
-
 	gtk_main();
 
+	log_set_func_default();
+	game_free(game);
 	config_finish();
 	net_finish();
 	g_option_context_free(context);
