@@ -33,6 +33,7 @@
 #include <unistd.h>
 #include <limits.h>
 #include <glib.h>
+#include <glib-object.h>
 
 #include "driver.h"
 #include "game.h"
@@ -43,18 +44,21 @@
 #include "buildrec.h"
 #include "server.h"
 
+#include "common_glib.h"
 #include "glib-driver.h"
 
 #include "admin.h"
-#include "avahi.h"
 
 static GMainLoop *event_loop;
 
 static gint num_players = 0;
 static gint num_points = 0;
 static gint sevens_rule = -1;
+static gint use_dice_deck = -1;
+static gint num_dice_decks = -1;
+static gint num_removed_dice_cards = -1;
 static gint terrain = -1;
-static gint timeout = 0;
+static guint timeout = 0;
 static gint num_ai_players = 0;
 static gchar *server_port = NULL;
 static gchar *admin_port = NULL;
@@ -65,7 +69,7 @@ static gint tournament_time = -1;
 static gboolean quit_when_done = FALSE;
 static gchar *hostname = NULL;
 static gboolean register_server = FALSE;
-static gchar *meta_server_name = NULL;
+static gchar *metaserver_name = NULL;
 static gboolean fixed_seating_order = FALSE;
 static gboolean enable_debug = FALSE;
 static gboolean show_version = FALSE;
@@ -89,6 +93,16 @@ static GOptionEntry commandline_game_entries[] = {
 	{"seven-rule", 'R', 0, G_OPTION_ARG_INT, &sevens_rule,
 	 /* Commandline server-console: seven-rule */
 	 N_("Override seven-rule handling"), "0|1|2"},
+	{"use-dice-deck", 'd', 0, G_OPTION_ARG_INT, &use_dice_deck,
+	 /* Commandline server-console: dice-deck */
+	 N_("Override dice-deck handling"), "0|1"},
+	{"dice-deck", 'D', 0, G_OPTION_ARG_INT, &num_dice_decks,
+	 /* Commandline server-console: num-dice-decks */
+	 N_("Override num-dice-decks handling"), "0|1"},
+	{"num-removed-dice-cards", 'C', 0, G_OPTION_ARG_INT,
+	 &num_removed_dice_cards,
+	 /* Commandline server-console: num-removed-dice-cards */
+	 N_("Override num-removed-dice-cards handling"), NULL},
 	{"terrain", 'T', 0, G_OPTION_ARG_INT, &terrain,
 	 /* Commandline server-console: terrain */
 	 N_("Override terrain type, 0=default 1=random"), "0|1"},
@@ -104,11 +118,11 @@ static GOptionEntry commandline_game_entries[] = {
 static GOptionEntry commandline_meta_entries[] = {
 	{"register", 'r', 0, G_OPTION_ARG_NONE, &register_server,
 	 /* Commandline server-console: register */
-	 N_("Register server with meta-server"), NULL},
-	{"meta-server", 'm', 0, G_OPTION_ARG_STRING, &meta_server_name,
-	 /* Commandline server-console: meta-server */
-	 N_("Register at meta-server name (implies -r)"),
-	 PIONEERS_DEFAULT_META_SERVER},
+	 N_("Register server with metaserver"), NULL},
+	{"metaserver", 'm', 0, G_OPTION_ARG_STRING, &metaserver_name,
+	 /* Commandline server-console: metaserver */
+	 N_("Register at metaserver name (implies -r)"),
+	 PIONEERS_DEFAULT_METASERVER},
 	{"hostname", 'n', 0, G_OPTION_ARG_STRING, &hostname,
 	 /* Commandline server-console: hostname */
 	 N_("Use this hostname when registering"), NULL},
@@ -163,6 +177,8 @@ int main(int argc, char *argv[])
 
 	driver->player_change = srv_player_change;
 
+	g_type_init();
+
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
@@ -179,10 +195,10 @@ int main(int argc, char *argv[])
 					  PACKAGE);
 	context_group = g_option_group_new("meta",
 					   /* Commandline server-console: Short description of meta group */
-					   _("Meta-server Options"),
+					   _("Metaserver Options"),
 					   /* Commandline server-console: Long description of meta group */
 					   _(""
-					     "Options for the meta-server"),
+					     "Options for the metaserver"),
 					   NULL, NULL);
 	g_option_group_set_translation_domain(context_group, PACKAGE);
 	g_option_group_add_entries(context_group,
@@ -216,10 +232,6 @@ int main(int argc, char *argv[])
 
 	if (server_port == NULL)
 		server_port = g_strdup(PIONEERS_DEFAULT_GAME_PORT);
-	if (disable_game_start)
-		if (admin_port == NULL)
-			admin_port = g_strdup(PIONEERS_DEFAULT_ADMIN_PORT);
-
 	if (game_title && game_file) {
 		/* server-console commandline error */
 		g_print(_(""
@@ -243,10 +255,10 @@ int main(int argc, char *argv[])
 		return 3;
 	}
 
-	if (meta_server_name != NULL)
+	if (metaserver_name != NULL)
 		register_server = TRUE;
-	if (register_server && meta_server_name == NULL)
-		meta_server_name = get_meta_server_name(TRUE);
+	if (register_server && metaserver_name == NULL)
+		metaserver_name = get_metaserver_name(TRUE);
 
 	g_assert(server_port != NULL);
 
@@ -256,7 +268,17 @@ int main(int argc, char *argv[])
 	if (sevens_rule != -1)
 		cfg_set_sevens_rule(params, sevens_rule);
 
-	if (num_points)
+	if (use_dice_deck != -1)
+		cfg_set_use_dice_deck(params, use_dice_deck);
+
+	if (num_dice_decks != -1)
+		cfg_set_num_dice_decks(params, num_dice_decks);
+
+	if (num_removed_dice_cards != -1)
+		cfg_set_num_removed_dice_cards(params,
+					       num_removed_dice_cards);
+
+	if (num_points > 0)
 		cfg_set_victory_points(params, num_points);
 
 	if (tournament_time != -1)
@@ -269,36 +291,38 @@ int main(int argc, char *argv[])
 
 	net_init();
 
-	if (admin_port != NULL) {
-		if (!admin_listen(admin_port)) {
-			/* Error message */
-			g_print(_("Admin port not available.\n"));
-			return 5;
-		}
-	}
-
-	if (disable_game_start && admin_port == NULL) {
-		/* server-console commandline error */
-		g_print(_(""
-			  "Admin port is not set, "
-			  "cannot disable game start too\n"));
-		return 4;
-
-	}
-
 	if (!disable_game_start) {
 		game =
 		    server_start(params, hostname, server_port,
-				 register_server, meta_server_name,
+				 register_server, metaserver_name,
 				 !fixed_seating_order);
 		if (game != NULL) {
+			if (admin_port != NULL) {
+				if (!admin_init(admin_port, &game)) {
+					/* Error message */
+					g_print(_("The network port (%s) "
+						  "for the admin "
+						  "interface is not "
+						  "available.\n"),
+						admin_port);
+				}
+			}
 			game->no_player_timeout = timeout;
 			num_ai_players =
 			    CLAMP(num_ai_players, 0,
-				  game->params->num_players);
+				  (gint) game->params->num_players);
 			for (i = 0; i < num_ai_players; ++i)
 				add_computer_player(game, TRUE);
-			avahi_register_game(game);
+		}
+	} else {
+		if (admin_port == NULL)
+			admin_port = g_strdup(PIONEERS_DEFAULT_ADMIN_PORT);
+		if (!admin_init(admin_port, &game)) {
+			/* Error message */
+			g_print(_("The network port (%s) for the admin "
+				  "interface is not available.\n"),
+				admin_port);
+			return 5;
 		}
 	}
 	if (disable_game_start || game != NULL) {
@@ -307,8 +331,6 @@ int main(int argc, char *argv[])
 		g_main_loop_unref(event_loop);
 		game_free(game);
 		game = NULL;
-
-		avahi_unregister_game();
 	}
 
 	net_finish();
