@@ -25,6 +25,8 @@
 #include <string.h>
 #include <unistd.h>
 #include "server.h"
+#include "network.h"
+#include "random.h"
 
 /* Local function prototypes */
 static gboolean mode_check_version(Player * player, gint event);
@@ -32,7 +34,7 @@ static gboolean mode_check_status(Player * player, gint event);
 static gboolean mode_bad_version(Player * player, gint event);
 static gboolean mode_global(Player * player, gint event);
 static gboolean mode_unhandled(Player * player, gint event);
-static void player_setup(Player * player, int playernum,
+static void player_setup(Player * player, gint playernum,
 			 const gchar * name, gboolean force_spectator);
 static Player *player_by_name(Game * game, char *name);
 
@@ -60,7 +62,7 @@ static gint next_free_player_num(Game * game, gboolean force_spectator)
 	if (!force_spectator) {
 		GList *list;
 		gboolean player_taken[MAX_PLAYERS];
-		gint available = game->params->num_players;
+		guint available = game->params->num_players;
 
 		memset(player_taken, 0, sizeof(player_taken));
 		playerlist_inc_use_count(game);
@@ -75,9 +77,9 @@ static gint next_free_player_num(Game * game, gboolean force_spectator)
 		}
 		playerlist_dec_use_count(game);
 		if (available > 0) {
-			gint skip;
+			guint skip;
 			if (game->random_order) {
-				skip = get_rand(available);
+				skip = random_guint(available);
 			} else {
 				skip = 0;
 			}
@@ -89,7 +91,7 @@ static gint next_free_player_num(Game * game, gboolean force_spectator)
 	}
 
 	/* No players available/wanted, look for a spectator number */
-	idx = game->params->num_players;
+	idx = (gint) game->params->num_players;
 	while (player_by_num(game, idx) != NULL)
 		++idx;
 	return idx;
@@ -110,7 +112,7 @@ static gboolean mode_global(Player * player, gint event)
 		if (player->location != NULL)
 			g_free(player->location);
 		if (player->devel != NULL)
-			deck_free(player->devel);
+			deck_free(player->devel, NULL);
 		if (player->num >= 0
 		    && !player_is_spectator(game, player->num)
 		    && !player->disconnected) {
@@ -206,7 +208,7 @@ static gboolean mode_unhandled(Player * player, gint event)
  */
 static gboolean tournament_start_cb(gpointer data)
 {
-	int i;
+	guint i;
 	Game *game = (Game *) data;
 	GList *player;
 	gboolean human_player_present;
@@ -234,7 +236,7 @@ static gboolean tournament_start_cb(gpointer data)
 	for (player = game->player_list; player != NULL;
 	     player = g_list_next(player)) {
 		Player *p = player->data;
-		if (p->disconnected && !p->sm->use_cache) {
+		if (p->disconnected && !sm_get_use_cache(p->sm)) {
 			player_free(p);
 		}
 	}
@@ -411,7 +413,7 @@ Player *player_new(Game * game, const gchar * name)
 
 	player->game = game;
 	player->location = g_strdup("not connected");
-	player->devel = deck_new(game->params);
+	player->devel = deck_new();
 	game->player_list = g_list_append(game->player_list, player);
 	player->num = -1;
 	player->chapel_played = 0;
@@ -431,12 +433,25 @@ Player *player_new(Game * game, const gchar * name)
 	return player;
 }
 
-Player *player_new_connection(Game * game, int fd, const gchar * location)
+Player *player_new_connection(Game * game, Session * ses)
 {
 	gchar name[100];
-	gint i;
+	size_t i;
 	Player *player;
 	StateMachine *sm;
+	GError *error;
+	gchar *location;
+	gchar *port;
+
+	error = NULL;
+	if (!net_get_peer_name(ses, &location, &port, &error)) {
+		/* %s = error message */
+		log_message(MSG_ERROR,
+			    _("Unable to determine the "
+			      "hostname of the player: %s"),
+			    error->message);
+		g_error_free(error);
+	}
 
 	/* give player a name, some functions need it */
 	strcpy(name, "connecting");
@@ -448,46 +463,37 @@ Player *player_new_connection(Game * game, int fd, const gchar * location)
 	}
 	if (i == G_N_ELEMENTS(name) - 1) {
 		/* there are too many pending connections */
-		ssize_t bytes_written;
-		bytes_written =
-		    write(fd, "ERR Too many connections\n", 25);
-		/* This ugly construction is to suppress a compiler warning */
-		bytes_written = bytes_written;
-		net_closesocket(fd);
+		net_write(ses, "ERR Too many connections\n");
+		g_free(location);
+		g_free(port);
 		return NULL;
 	}
 
 	if (game->is_game_over) {
 		/* The game is over, don't accept new players */
-		Session *ses = net_new(NULL, NULL);
-		gchar *message;
-		net_use_fd(ses, fd, FALSE);
 		/* Message to send to the client when the game is already over
 		 * when a connection is made. */
-		message =
-		    g_strdup_printf("NOTE %s\n",
-				    N_("Sorry, game is over."));
-		net_write(ses, message);
+		net_printf(ses, "NOTE %s\n", N_("Sorry, game is over."));
 		log_message(MSG_INFO,
 			    _("Player from %s is refused: game is over\n"),
 			    location);
-		net_close_when_flushed(ses);
-		g_free(message);
+		g_free(location);
+		g_free(port);
 		return NULL;
 	}
 
 	player = player_new(game, name);
 	sm = player->sm;
-	sm_use_fd(sm, fd, TRUE);
+	sm_set_session(sm, ses);
+	net_set_check_connection_alive(ses, 30);
 	g_free(player->location);
 	player->location = g_strdup(location);
 
-	/* Cache messages of the game in progress until all intial 
+	/* Cache messages of the game in progress until all initial 
 	 * messages have been sent
 	 */
 	sm_set_use_cache(sm, TRUE);
 
-
 	sm_goto(sm, (StateFunc) mode_check_version);
 
 	driver->player_change(game);
@@ -529,7 +535,7 @@ static void player_set_name_real(Player * player, gchar * name,
 	driver->player_change(game);
 }
 
-static void player_setup(Player * player, int playernum,
+static void player_setup(Player * player, gint playernum,
 			 const gchar * name, gboolean force_spectator)
 {
 	gchar nm[MAX_NAME_LENGTH + 1];
@@ -571,7 +577,7 @@ static void player_setup(Player * player, int playernum,
 	/* if the new name exists, try padding it with underscores */
 	other = player_by_name(game, nm);
 	if (other != player && other != NULL) {
-		gint i;
+		size_t i;
 		/* add underscores until the name is unique */
 		for (i = strlen(nm); i < G_N_ELEMENTS(nm) - 1; ++i) {
 			if (player_by_name(game, nm) == NULL)
@@ -591,7 +597,7 @@ static void player_setup(Player * player, int playernum,
 				/* Digit will be: 0..10 */
 				--i;
 				digit = g_ascii_digit_value(nm[i]) + 1;
-				nm[i] = '0' + digit % 10;
+				nm[i] = (gchar) ('0' + digit % 10);
 			}
 		}
 	}
@@ -643,6 +649,8 @@ void player_archive(Player * player)
 {
 	StateFunc state;
 	Game *game = player->game;
+	gboolean human_player_present;
+	GList *pl;
 
 	/* If this was a spectator, forget about him */
 	if (player_is_spectator(game, player->num)) {
@@ -650,7 +658,7 @@ void player_archive(Player * player)
 		return;
 	}
 	/* If this game can't be started, forget old players */
-	if (game_is_unstartable(game)) {
+	if (params_game_is_unstartable(game->params)) {
 		player_free(player);
 		return;
 	}
@@ -692,8 +700,8 @@ void player_archive(Player * player)
 
 	/* if no human players are present, start timer */
 	playerlist_inc_use_count(game);
-	gboolean human_player_present = FALSE;
-	GList *pl = game->player_list;
+	human_player_present = FALSE;
+	pl = game->player_list;
 	for (pl = game->player_list;
 	     pl != NULL && !human_player_present; pl = g_list_next(pl)) {
 		Player *p = pl->data;
@@ -744,7 +752,8 @@ void player_revive(Player * newp, char *name)
 		     current = g_list_next(current)) {
 			p = current->data;
 			if (!strcmp(name, p->name))
-				if (p->disconnected && !p->sm->use_cache
+				if (p->disconnected
+				    && !sm_get_use_cache(p->sm)
 				    && p != newp)
 					break;
 		}
@@ -755,7 +764,7 @@ void player_revive(Player * newp, char *name)
 		gint num;
 
 		num = next_free_player_num(game, FALSE);
-		if (num < game->params->num_players) {
+		if (num < (gint) game->params->num_players) {
 			player_setup(newp, -1, name, FALSE);
 			return;
 		}
@@ -766,7 +775,7 @@ void player_revive(Player * newp, char *name)
 		for (current = game->player_list; current != NULL;
 		     current = g_list_next(current)) {
 			p = current->data;
-			if (p->disconnected && !p->sm->use_cache
+			if (p->disconnected && !sm_get_use_cache(p->sm)
 			    && p != newp)
 				break;
 		}
@@ -803,7 +812,7 @@ void player_revive(Player * newp, char *name)
 	memcpy(newp->assets, p->assets, sizeof(newp->assets));
 	newp->gold = p->gold;
 	/* take over the development deck */
-	deck_free(newp->devel);
+	deck_free(newp->devel, NULL);
 	newp->devel = p->devel;
 	p->devel = NULL;
 
@@ -826,13 +835,7 @@ void player_revive(Player * newp, char *name)
 	newp->market_played = p->market_played;
 	/* Not copied: sm, game, location, num, client_version */
 
-	/* copy over the state */
-	memcpy(newp->sm->stack, p->sm->stack, sizeof(newp->sm->stack));
-	memcpy(newp->sm->stack_name, p->sm->stack_name,
-	       sizeof(newp->sm->stack_name));
-	newp->sm->stack_ptr = p->sm->stack_ptr;
-	newp->sm->current_state = p->sm->current_state;
-
+	sm_copy_stack(newp->sm, p->sm);
 	if (sm_current(newp->sm) != (StateFunc) mode_pre_game)
 		sm_push(newp->sm, (StateFunc) mode_pre_game);
 	else
@@ -884,14 +887,15 @@ gboolean mode_spectator(Player * player, gint event)
 	if (sm_recv(sm, "play")) {
 		/* try to be the first available player */
 		num = next_free_player_num(game, FALSE);
-		if (num >= game->params->num_players) {
+		if (num >= (gint) game->params->num_players) {
 			player_send(player, FIRST_VERSION, LATEST_VERSION,
 				    "ERR game-full");
 			return TRUE;
 		}
 	} else if (sm_recv(sm, "play %d", &num)) {
 		/* try to be the specified player number */
-		if (num >= game->params->num_players
+		if (num >= (gint) game->params->num_players
+		    || num < 0
 		    || !player_by_num(game, num)->disconnected) {
 			player_send(player, FIRST_VERSION, LATEST_VERSION,
 				    "ERR invalid-player");
@@ -951,19 +955,13 @@ static gboolean mode_check_version(Player * player, gint event)
 			    client_version_type_from_string(version);
 			player->version = cvt;
 			if (can_client_connect_to_server
-			    (cvt,
-			     client_version_type_from_string
-			     (PROTOCOL_VERSION))) {
+			    (cvt, LATEST_VERSION)) {
 				sm_goto(sm, (StateFunc) mode_check_status);
 			} else {
-				/* PROTOCOL_VERSION is the current version
-				 * of the client when building this server.
-				 * Although it's not the only supported
-				 * version, it's the best the user can have.
-				 */
 				gchar *mismatch =
 				    g_strdup_printf("%s <-> %s",
-						    PROTOCOL_VERSION,
+						    client_version_type_to_string
+						    (LATEST_VERSION),
 						    version);
 				/* Make sure the argument does not contain the separator */
 				g_strdelimit(mismatch, "|", '_');
@@ -1075,7 +1073,7 @@ GList *player_next_real(GList * last)
 {
 	Player *player;
 	Game *game;
-	gint numplayers;
+	guint numplayers;
 	gint nextnum;
 	GList *list;
 
@@ -1087,7 +1085,7 @@ GList *player_next_real(GList * last)
 	numplayers = game->params->num_players;
 	nextnum = player->num + 1;
 
-	if (nextnum >= numplayers)
+	if (nextnum >= (gint) numplayers)
 		return NULL;
 
 	playerlist_inc_use_count(game);
@@ -1142,7 +1140,7 @@ Player *player_by_num(Game * game, gint num)
 
 gboolean player_is_spectator(Game * game, gint player_num)
 {
-	return game->params->num_players <= player_num;
+	return (gint) game->params->num_players <= player_num;
 }
 
 /* Returns a player that's not part of the game.
@@ -1183,7 +1181,7 @@ static void player_broadcast_internal(Player * player, BroadcastType type,
 	for (list = game->player_list; list != NULL;
 	     list = g_list_next(list)) {
 		Player *scan = list->data;
-		if ((scan->disconnected && !scan->sm->use_cache)
+		if ((scan->disconnected && !sm_get_use_cache(scan->sm))
 		    || scan->num < 0
 		    || scan->version < first_supported_version
 		    || scan->version > last_supported_version)
