@@ -21,6 +21,7 @@
 
 #include "config.h"
 #include "server.h"
+#include "random.h"
 
 static Hex *previous_robber_hex;
 
@@ -64,7 +65,7 @@ static void move_robber(Player * player, Hex * hex, gboolean is_undo)
 static void steal_card_from(Player * player, Player * victim)
 {
 	Game *game = player->game;
-	gint idx;
+	guint idx;
 	gint num;
 	gint steal;
 	GList *list;
@@ -82,7 +83,7 @@ static void steal_card_from(Player * player, Player * victim)
 
 	/* Work out which card to steal from the victim
 	 */
-	steal = get_rand(num);
+	steal = random_guint(num);
 	for (idx = 0; idx < G_N_ELEMENTS(victim->assets); idx++) {
 		steal -= victim->assets[idx];
 		if (steal < 0)
@@ -137,7 +138,7 @@ static void do_select_robbed(Player * player, Hex * hex, gint victim_num)
 	Game *game = player->game;
 	Player *owner;
 	Resource resource;
-	gint idx;
+	guint idx;
 
 	/* Check if the victim has any resources
 	 */
@@ -213,7 +214,7 @@ static void do_select_pirated(Player * player, Hex * hex, gint victim_num)
 	Game *game = player->game;
 	Player *owner;
 	Resource resource;
-	gint idx;
+	guint idx;
 
 	/* Check if the victim has any resources
 	 */
@@ -292,7 +293,7 @@ gboolean mode_place_robber(Player * player, gint event)
 	Map *map = game->params->map;
 	gint x, y;
 	Hex *hex;
-	gint idx;
+	guint idx;
 	gint one_victim;
 	gint victim_num = -1;
 	gboolean old_style;
