@@ -20,18 +20,13 @@
  */
 
 #include "config.h"
-#include <fcntl.h>
 #include <stdlib.h>
-#include <unistd.h>
-#include <string.h>
-#include <time.h>
-#include <signal.h>
 
 #include "server.h"
-
-static GameParams *load_game_desc(const gchar * fname);
-
-static GSList *_game_list = NULL;	/* The list of GameParams, ordered by title */
+#include "network.h"
+#include "avahi.h"
+#include "game-list.h"
+#include "random.h"
 
 #define TERRAIN_DEFAULT	0
 #define TERRAIN_RANDOM	1
@@ -62,22 +57,17 @@ void stop_timeout(Game * game)
 	}
 }
 
-gint get_rand(gint range)
-{
-	return g_rand_int_range(g_rand_ctx, 0, range);
-}
-
 Game *game_new(const GameParams * params)
 {
 	Game *game;
-	gint idx;
+	guint idx;
 
 	game = g_malloc0(sizeof(*game));
 
-	game->accept_tag = 0;
-	game->accept_fd = -1;
+	game->service = NULL;
 	game->is_running = FALSE;
 	game->is_game_over = FALSE;
+	game->is_manipulated = FALSE;
 	game->params = params_copy(params);
 	game->curr_player = -1;
 
@@ -101,31 +91,11 @@ void game_free(Game * game)
 	if (game->server_port != NULL)
 		g_free(game->server_port);
 	params_free(game->params);
+	net_service_free(game->service);
+	game->service = NULL;
 	g_free(game);
 }
 
-gint accept_connection(gint in_fd, gchar ** location)
-{
-	int fd;
-	gchar *error_message;
-	gchar *port;
-
-	fd = net_accept(in_fd, &error_message);
-	if (fd < 0) {
-		log_message(MSG_ERROR, "%s\n", error_message);
-		g_free(error_message);
-		return -1;
-	}
-
-	g_assert(location != NULL);
-	if (!net_get_peer_name(fd, location, &port, &error_message)) {
-		log_message(MSG_ERROR, "%s\n", error_message);
-		g_free(error_message);
-	}
-	g_free(port);
-	return fd;
-}
-
 gint add_computer_player(Game * game, gboolean want_chat)
 {
 	gchar *child_argv[10];
@@ -134,8 +104,8 @@ gint add_computer_player(Game * game, gboolean want_chat)
 	gint n = 0;
 	gint i;
 
-	child_argv[n++] = g_strdup(PIONEERS_AI_PATH);
-	child_argv[n++] = g_strdup(PIONEERS_AI_PATH);
+	child_argv[n++] = g_strdup(PIONEERS_AI_PROGRAM_NAME);
+	child_argv[n++] = g_strdup(PIONEERS_AI_PROGRAM_NAME);
 	child_argv[n++] = g_strdup("-s");
 	child_argv[n++] = g_strdup(PIONEERS_DEFAULT_GAME_HOST);
 	child_argv[n++] = g_strdup("-p");
@@ -147,11 +117,11 @@ gint add_computer_player(Game * game, gboolean want_chat)
 	child_argv[n] = NULL;
 	g_assert(n < 10);
 
-	if (!g_spawn_async(NULL, child_argv, NULL, 0, NULL, NULL,
-			   NULL, &error)) {
-		log_message(MSG_ERROR,
-			    _("Error starting %s: %s\n"),
-			    PIONEERS_AI_PATH, error->message);
+	if (!g_spawn_async
+	    (NULL, child_argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL,
+	     &error)) {
+		log_message(MSG_ERROR, _("Error starting %s: %s\n"),
+			    child_argv[0], error->message);
 		g_error_free(error);
 		ret = -1;
 	}
@@ -160,27 +130,45 @@ gint add_computer_player(Game * game, gboolean want_chat)
 	return ret;
 }
 
-
-static void player_connect(Game * game)
+static void player_connect(Session * ses, NetEvent event,
+			   G_GNUC_UNUSED const gchar * line,
+			   gpointer user_data)
 {
-	gchar *location;
-	gint fd = accept_connection(game->accept_fd, &location);
+	Game *game = (Game *) user_data;
 
-	if (fd > 0) {
-		if (player_new_connection(game, fd, location) != NULL)
+	switch (event) {
+	case NET_READ:
+		/* there is data to be read */
+		break;
+	case NET_CLOSE:
+		/* connection has been closed */
+		net_free(&ses);
+		break;
+	case NET_CONNECT:
+		/* new connection was made */
+		if (player_new_connection(game, ses) != NULL) {
 			stop_timeout(game);
+		} else {
+			net_close(ses);
+		}
+		break;
+	case NET_CONNECT_FAIL:
+		/* connect failed */
+		net_free(&ses);
+		break;
 	}
-	g_free(location);
 }
 
 static gboolean game_server_start(Game * game, gboolean register_server,
-				  const gchar * meta_server_name)
+				  const gchar * metaserver_name)
 {
 	gchar *error_message;
 
-	game->accept_fd =
-	    net_open_listening_socket(game->server_port, &error_message);
-	if (game->accept_fd == -1) {
+	game->service =
+	    net_service_new(atoi(game->server_port), player_connect, game,
+			    &error_message);
+
+	if (game->service == NULL) {
 		log_message(MSG_ERROR, "%s\n", error_message);
 		g_free(error_message);
 		return FALSE;
@@ -189,51 +177,49 @@ static gboolean game_server_start(Game * game, gboolean register_server,
 
 	start_timeout(game);
 
-	game->accept_tag = driver->input_add_read(game->accept_fd,
-						  (InputFunc)
-						  player_connect, game);
-
 	if (register_server) {
-		g_assert(meta_server_name != NULL);
-		meta_register(meta_server_name, PIONEERS_DEFAULT_META_PORT,
-			      game);
+		g_assert(metaserver_name != NULL);
+		meta_register(metaserver_name, game);
 	}
+	avahi_register_game(game);
 	return TRUE;
 }
 
 /** Try to start a new server.
  * @param params The parameters of the game
- * @param hostname The hostname that will be visible in the meta server
+ * @param hostname The hostname that will be visible in the metaserver
  * @param port The port to listen to
- * @param register_server Register at the meta server
- * @param meta_server_name The hostname of the meta server
+ * @param register_server Register at the metaserver
+ * @param metaserver_name The hostname of the metaserver
  * @param random_order Randomize the player number
  * @return A pointer to the new game, or NULL
 */
 Game *server_start(const GameParams * params, const gchar * hostname,
 		   const gchar * port, gboolean register_server,
-		   const gchar * meta_server_name, gboolean random_order)
+		   const gchar * metaserver_name, gboolean random_order)
 {
 	Game *game;
-	guint32 randomseed = time(NULL);
+	guint32 randomseed;
 
 	g_return_val_if_fail(params != NULL, NULL);
 	g_return_val_if_fail(port != NULL, NULL);
 
 #ifdef PRINT_INFO
 	g_print("game type: %s\n", params->title);
-	g_print("num players: %d\n", params->num_players);
-	g_print("victory points: %d\n", params->victory_points);
+	g_print("num players: %u\n", params->num_players);
+	g_print("victory points: %u\n", params->victory_points);
 	g_print("terrain type: %s\n",
 		(params->random_terrain) ? "random" : "default");
-	g_print("Tournament time: %d\n", params->tournament_time);
+	g_print("Tournament time: %u\n", params->tournament_time);
 	g_print("Quit when done: %d\n", params->quit_when_done);
 #endif
 
-	g_rand_ctx = g_rand_new_with_seed(randomseed);
-	log_message(MSG_INFO, "%s #%" G_GUINT32_FORMAT ".%s.%03d\n",
+	/* create new random seed, to be able to reproduce games */
+	randomseed = random_init();
+	log_message(MSG_INFO, "%s #%" G_GUINT32_FORMAT ".%s.%03u\n",
 		    /* Server: preparing game #..... */
-		    _("Preparing game"), randomseed, "G", get_rand(1000));
+		    _("Preparing game"), randomseed, "G",
+		    random_guint(1000));
 
 	game = game_new(params);
 	g_assert(game->server_port == NULL);
@@ -243,7 +229,7 @@ Game *server_start(const GameParams * params, const gchar * hostname,
 		game->hostname = g_strdup(hostname);
 	}
 	game->random_order = random_order;
-	if (!game_server_start(game, register_server, meta_server_name)) {
+	if (!game_server_start(game, register_server, metaserver_name)) {
 		game_free(game);
 		game = NULL;
 	}
@@ -262,22 +248,19 @@ gboolean server_stop(Game * game)
 		return FALSE;
 
 	meta_unregister();
+	avahi_unregister_game();
 
 	game->is_running = FALSE;
-	if (game->accept_tag) {
-		driver->input_remove(game->accept_tag);
-		game->accept_tag = 0;
-	}
-	if (game->accept_fd >= 0) {
-		close(game->accept_fd);
-		game->accept_fd = -1;
-	}
+	net_service_free(game->service);
+	game->service = NULL;
 
 	playerlist_inc_use_count(game);
 	current = game->player_list;
 	while (current != NULL) {
 		Player *player = current->data;
-		player_remove(player);
+		if (!player->disconnected) {
+			player_remove(player);
+		}
 		player_free(player);
 		current = g_list_next(current);
 	}
@@ -294,146 +277,6 @@ gboolean server_is_running(Game * game)
 	return FALSE;
 }
 
-static gint sort_function(gconstpointer a, gconstpointer b)
-{
-	return (strcmp(((const GameParams *) a)->title,
-		       ((const GameParams *) b)->title));
-}
-
-static gboolean game_list_add_item(GameParams * item)
-{
-	/* check for name collisions */
-	if (item->title && game_list_find_item(item->title)) {
-
-		gchar *nt;
-		gint i;
-
-		if (params_is_equal
-		    (item, game_list_find_item(item->title))) {
-			return FALSE;
-		}
-
-		/* append a number */
-		for (i = 1; i <= INT_MAX; i++) {
-			nt = g_strdup_printf("%s%d", item->title, i);
-			if (!game_list_find_item(nt)) {
-				g_free(item->title);
-				item->title = nt;
-				break;
-			}
-			g_free(nt);
-		}
-		/* give up and skip this game */
-		if (item->title != nt) {
-			g_free(nt);
-			return FALSE;
-		}
-	}
-
-	_game_list =
-	    g_slist_insert_sorted(_game_list, item, sort_function);
-	return TRUE;
-}
-
-/** Returns TRUE if the game list is empty */
-static gboolean game_list_is_empty(void)
-{
-	return _game_list == NULL;
-}
-
-static gint game_list_locate(gconstpointer param, gconstpointer argument)
-{
-	const GameParams *data = param;
-	const gchar *title = argument;
-	return strcmp(data->title, title);
-}
-
-const GameParams *game_list_find_item(const gchar * title)
-{
-	GSList *result;
-	if (!_game_list) {
-		return NULL;
-	}
-
-	result = g_slist_find_custom(_game_list, title, game_list_locate);
-	if (result)
-		return result->data;
-	else
-		return NULL;
-}
-
-void game_list_foreach(GFunc func, gpointer user_data)
-{
-	if (_game_list) {
-		g_slist_foreach(_game_list, func, user_data);
-	}
-}
-
-GameParams *load_game_desc(const gchar * fname)
-{
-	GameParams *params;
-
-	params = params_load_file(fname);
-	if (params == NULL)
-		g_warning("Skipping: %s", fname);
-	return params;
-}
-
-static void game_list_prepare_directory(const gchar * directory)
-{
-	GDir *dir;
-	const gchar *fname;
-	gchar *fullname;
-
-	log_message(MSG_INFO, _("Looking for games in '%s'\n"), directory);
-	if ((dir = g_dir_open(directory, 0, NULL)) == NULL) {
-		log_message(MSG_INFO, _("Game directory '%s' not found\n"),
-			    directory);
-		return;
-	}
-
-	while ((fname = g_dir_read_name(dir))) {
-		GameParams *params;
-		gint len = strlen(fname);
-
-		if (len < 6 || strcmp(fname + len - 5, ".game") != 0)
-			continue;
-		fullname = g_build_filename(directory, fname, NULL);
-		params = load_game_desc(fullname);
-		g_free(fullname);
-		if (params) {
-			if (!game_list_add_item(params))
-				params_free(params);
-		}
-	}
-	g_dir_close(dir);
-}
-
-void game_list_prepare(void)
-{
-	gchar *directory;
-
-	directory =
-	    g_build_filename(g_get_user_data_dir(), "pioneers", NULL);
-	game_list_prepare_directory(directory);
-	g_free(directory);
-
-	game_list_prepare_directory(get_pioneers_dir());
-
-	if (game_list_is_empty())
-		g_error("No games available");
-}
-
-void game_list_cleanup(void)
-{
-	GSList *games = _game_list;
-	while (games) {
-		params_free(games->data);
-		games = g_slist_next(games);
-	}
-	g_slist_free(_game_list);
-}
-
 /* game configuration functions / callbacks */
 void cfg_set_num_players(GameParams * params, gint num_players)
 {
@@ -441,7 +284,7 @@ void cfg_set_num_players(GameParams * params, gint num_players)
 	g_print("cfg_set_num_players: %d\n", num_players);
 #endif
 	g_return_if_fail(params != NULL);
-	params->num_players = CLAMP(num_players, 2, MAX_PLAYERS);
+	params->num_players = (guint) CLAMP(num_players, 2, MAX_PLAYERS);
 }
 
 void cfg_set_sevens_rule(GameParams * params, gint sevens_rule)
@@ -453,75 +296,42 @@ void cfg_set_sevens_rule(GameParams * params, gint sevens_rule)
 	params->sevens_rule = CLAMP(sevens_rule, 0, 2);
 }
 
+void cfg_set_use_dice_deck(GameParams * params, gboolean use_dice_deck)
+{
+#ifdef PRINT_INFO
+	g_print("cfg_set_dice_deck: %d\n", use_dice_deck);
+#endif
+	g_return_if_fail(params != NULL);
+	params->use_dice_deck = use_dice_deck;
+}
+
+void cfg_set_num_dice_decks(GameParams * params, gint num_dice_decks)
+{
+#ifdef PRINT_INFO
+	g_print("cfg_set_num-dice-decks: %d\n", num_dice_decks);
+#endif
+	g_return_if_fail(params != NULL);
+	params->num_dice_decks = CLAMP(num_dice_decks, 1, 5);
+}
+
+void cfg_set_num_removed_dice_cards(GameParams * params,
+				    gint num_removed_dice_cards)
+{
+#ifdef PRINT_INFO
+	g_print("cfg_set_dice_deck: %d\n", num_removed_dice_cards);
+#endif
+	g_return_if_fail(params != NULL);
+	params->num_removed_dice_cards =
+	    CLAMP(num_removed_dice_cards, 0, 30);
+}
+
 void cfg_set_victory_points(GameParams * params, gint victory_points)
 {
 #ifdef PRINT_INFO
 	g_print("cfg_set_victory_points: %d\n", victory_points);
 #endif
 	g_return_if_fail(params != NULL);
-	params->victory_points = MAX(3, victory_points);
-}
-
-/** Attempt to find a game with @a title in @a directory.
- *  @param title The game must match this title
- *  @param directory Look in this directory for *.game files
- *  @return The GameParams, or NULL if the title was not found
- */
-static GameParams *server_find_title_in_directory(const gchar * title,
-						  const gchar * directory)
-{
-	GDir *dir;
-	const gchar *fname;
-	gchar *fullname;
-
-	log_message(MSG_INFO, _("Looking for games in '%s'\n"), directory);
-	if ((dir = g_dir_open(directory, 0, NULL)) == NULL) {
-		log_message(MSG_INFO, _("Game directory '%s' not found\n"),
-			    directory);
-		return NULL;
-	}
-
-	while ((fname = g_dir_read_name(dir))) {
-		GameParams *params;
-		gint len = strlen(fname);
-
-		if (len < 6 || strcmp(fname + len - 5, ".game") != 0)
-			continue;
-		fullname = g_build_filename(directory, fname, NULL);
-		params = load_game_desc(fullname);
-		g_free(fullname);
-		if (params) {
-			if (strcmp(params->title, title) == 0) {
-				g_dir_close(dir);
-				return params;
-			}
-			params_free(params);
-		}
-	}
-	g_dir_close(dir);
-	return NULL;
-}
-
-/** Find a game with @a title in the default locations.
- *  @param title The title of the game_new
- *  @return The GameParams or NULL if the title was not found
- */
-static GameParams *server_find_title(const gchar * title)
-{
-	GameParams *result;
-	gchar *directory;
-
-	directory =
-	    g_build_filename(g_get_user_data_dir(), "pioneers", NULL);
-	result = server_find_title_in_directory(title, directory);
-	g_free(directory);
-
-	if (result == NULL) {
-		result =
-		    server_find_title_in_directory(title,
-						   get_pioneers_dir());
-	}
-	return result;
+	params->victory_points = (guint) MAX(3, victory_points);
 }
 
 GameParams *cfg_set_game(const gchar * game)
@@ -530,7 +340,10 @@ GameParams *cfg_set_game(const gchar * game)
 	g_print("cfg_set_game: %s\n", game);
 #endif
 	if (game_list_is_empty()) {
-		return server_find_title(game);
+		game_list_prepare();
+		GameParams *param = params_copy(game_list_find_item(game));
+		game_list_cleanup();
+		return param;
 	} else {
 		return params_copy(game_list_find_item(game));
 	}
@@ -559,7 +372,7 @@ void cfg_set_tournament_time(GameParams * params, gint tournament_time)
 	g_print("cfg_set_tournament_time: %d\n", tournament_time);
 #endif
 	g_return_if_fail(params != NULL);
-	params->tournament_time = tournament_time;
+	params->tournament_time = (guint) MAX(0, tournament_time);
 }
 
 void cfg_set_quit(GameParams * params, gboolean quitdone)
@@ -593,8 +406,3 @@ void server_init(void)
 	sigaction(SIGPIPE, &sa, NULL);
 #endif				/* G_OS_WIN32 */
 }
-
-gboolean game_is_unstartable(Game * game)
-{
-	return game->params->num_build_type[BUILD_SETTLEMENT] < 2;
-}
