@@ -27,6 +27,7 @@
 #include "map.h"
 #include "quoteinfo.h"
 #include "state.h"
+#include "network.h"
 
 #define TERRAIN_DEFAULT	0
 #define TERRAIN_RANDOM	1
@@ -45,8 +46,8 @@ typedef struct {
 	GList *build_list;	/* list of building that can be undone */
 	gint prev_assets[NO_RESOURCE];	/* remember previous resources */
 	gint assets[NO_RESOURCE];	/* our resources */
-	gint gold;		/* how much gold will we recieve? */
-	DevelDeck *devel;	/* development cards we own */
+	gint gold;		/* how much gold will we receive? */
+	Deck *devel;		/* development cards we own */
 	GList *special_points;	/* points from special actions */
 	gint special_points_next_id;	/* Next id for the special points */
 	gint discard_num;	/* number of resources we must discard */
@@ -65,7 +66,7 @@ typedef struct {
 	gint gov_played;	/* number of Governors cards played */
 	gint libr_played;	/* number of Library cards played */
 	gint market_played;	/* number of Market cards played */
-	gint islands_discovered;	/* number of islands discovered */
+	guint islands_discovered;	/* number of islands discovered */
 	gboolean disconnected;
 } Player;
 
@@ -73,15 +74,14 @@ struct Game {
 	GameParams *params;	/* game parameters */
 	gchar *hostname;	/* reported hostname */
 
-	int accept_fd;		/* socket for accepting new clients */
-	int accept_tag;		/* Gdk event tag for accept socket */
+	Service *service;	/* network service */
 
 	GList *player_list;	/* all players in the game */
 	GList *dead_players;	/* all players that should be removed when player_list_use_count == 0 */
 	gint player_list_use_count;	/* # functions is in use by */
-	gint num_players;	/* current number of players in the game */
+	guint num_players;	/* current number of players in the game */
 
-	gint tournament_countdown;	/* number of remaining minutes before AIs are added */
+	guint tournament_countdown;	/* number of remaining minutes before AIs are added */
 	guint tournament_timer;	/* timer id */
 
 	gboolean double_setup;
@@ -100,8 +100,10 @@ struct Game {
 	gint curr_turn;		/* current turn number */
 	gboolean rolled_dice;	/* has dice been rolled in turn yet? */
 	gint die1, die2;	/* latest dice values */
-	gboolean played_develop;	/* has devel. card been played in turn? */
+	guint dice_cards[36];	/* remaining dice cards */
+	guint num_dice_cards;	/* total remaining dice cards */
 	gboolean bought_develop;	/* has devel. card been bought in turn? */
+	guint num_playable_cards;	/* number of playable development cards */
 
 	gint bank_deck[NO_RESOURCE];	/* resource card bank */
 	gint num_develop;	/* number of development cards */
@@ -111,8 +113,9 @@ struct Game {
 	gboolean is_running;	/* is the server currently running? */
 	gchar *server_port;	/* port to run game on */
 	gboolean random_order;	/* is turn order randomized? */
+	gboolean is_manipulated;	/* has the game been manipulated by the admin? */
 
-	gint no_player_timeout;	/* time to wait for players */
+	guint no_player_timeout;	/* time to wait for players */
 	guint no_player_timer;	/* glib timer identifier */
 
 	guint no_humans_timer;	/* timer id: no human players are present */
@@ -131,7 +134,7 @@ gboolean perform_undo(Player * player);
 /* develop.c */
 void develop_shuffle(Game * game);
 void develop_buy(Player * player);
-void develop_play(Player * player, gint idx);
+void develop_play(Player * player, guint idx);
 gboolean mode_road_building(Player * player, gint event);
 gboolean mode_plenty_resources(Player * player, gint event);
 gboolean mode_monopoly(Player * player, gint event);
@@ -144,11 +147,10 @@ gboolean mode_discard_resources_place_robber(Player * player, gint event);
 
 /* meta.c */
 gchar *get_server_name(void);
-void meta_register(const gchar * server, const gchar * port, Game * game);
+void meta_register(const gchar * server, Game * game);
 void meta_unregister(void);
 void meta_start_game(void);
-void meta_report_num_players(gint num_players);
-void meta_send_details(Game * game);
+void meta_report_num_players(guint num_players);
 
 /* player.c */
 typedef enum {
@@ -159,7 +161,7 @@ typedef enum {
 } BroadcastType;
 gchar *player_new_computer_player(Game * game);
 Player *player_new(Game * game, const gchar * name);
-Player *player_new_connection(Game * game, int fd, const gchar * location);
+Player *player_new_connection(Game * game, Session * ses);
 Player *player_by_num(Game * game, gint num);
 void player_set_name(Player * player, gchar * name);
 Player *player_none(Game * game);
@@ -219,29 +221,25 @@ void robber_undo(Player * player);
 /* server.c */
 void start_timeout(Game * game);
 void stop_timeout(Game * game);
-gint get_rand(gint range);
 Game *game_new(const GameParams * params);
 void game_free(Game * game);
 gint add_computer_player(Game * game, gboolean want_chat);
 Game *server_start(const GameParams * params, const gchar * hostname,
 		   const gchar * port, gboolean register_server,
-		   const gchar * meta_server_name, gboolean random_order);
+		   const gchar * metaserver_name, gboolean random_order);
 gboolean server_stop(Game * game);
 gboolean server_is_running(Game * game);
 gint accept_connection(gint in_fd, gchar ** location);
-gboolean game_is_unstartable(Game * game);
-
-/**** game list control functions ****/
-void game_list_prepare(void);
-const GameParams *game_list_find_item(const gchar * title);
-void game_list_foreach(GFunc func, gpointer user_data);
-void game_list_cleanup(void);
 
 /**** callbacks to set parameters ****/
 GameParams *cfg_set_game(const gchar * game);
 GameParams *cfg_set_game_file(const gchar * game_filename);
 void cfg_set_num_players(GameParams * params, gint num_players);
 void cfg_set_sevens_rule(GameParams * params, gint sevens_rule);
+void cfg_set_use_dice_deck(GameParams * params, gboolean use_dice_deck);
+void cfg_set_num_dice_decks(GameParams * params, gint num_dice_decks);
+void cfg_set_num_removed_dice_cards(GameParams * params,
+				    gint num_removed_dice_cards);
 void cfg_set_victory_points(GameParams * params, gint victory_points);
 void cfg_set_terrain_type(GameParams * params, gint terrain_type);
 void cfg_set_tournament_time(GameParams * params, gint tournament_time);
