@@ -26,6 +26,7 @@
 #include "cost.h"
 #include "server.h"
 #include "admin.h"
+#include "random.h"
 
 static void build_add(Player * player, BuildType type, gint x, gint y,
 		      gint pos)
@@ -296,7 +297,7 @@ static void build_move(Player * player, gint sx, gint sy, gint spos,
 	}
 
 	if (map->pirate_hex != NULL) {
-		gint idx;
+		guint idx;
 		/* check that the pirate is not on the from hexes */
 		for (idx = 0; idx < G_N_ELEMENTS(from->hexes); ++idx) {
 			if (map->pirate_hex == from->hexes[idx]) {
@@ -358,7 +359,7 @@ typedef struct {
 
 static gboolean distribute_resources(const Hex * hex, gpointer closure)
 {
-	int idx;
+	guint idx;
 	GameRoll *data = closure;
 
 	if (hex->roll != data->roll || hex->robber)
@@ -395,7 +396,7 @@ gboolean check_victory(Player * player)
 {
 	Game *game = player->game;
 	GList *list;
-	gint points;
+	gint points;		/* can be negative, due to island bonuses */
 
 	if (player->num != game->curr_player)
 		/* Only the player that has the turn can win */
@@ -416,7 +417,7 @@ gboolean check_victory(Player * player)
 		list = g_list_next(list);
 	}
 
-	if (points >= game->params->victory_points) {
+	if (points >= (gint) game->params->victory_points) {
 		player_broadcast(player, PB_ALL, FIRST_VERSION,
 				 LATEST_VERSION, "won with %d\n", points);
 		game->is_game_over = TRUE;
@@ -436,41 +437,64 @@ gboolean check_victory(Player * player)
 	return FALSE;
 }
 
-/* Handle all actions that a player may perform in a turn
+/** Rolls the dice.
+ * @param player The player who rolled the dice.
  */
-gboolean mode_turn(Player * player, gint event)
+static void roll_dice(Player * player)
 {
-	StateMachine *sm = player->sm;
 	Game *game = player->game;
 	const Map *map = game->params->map;
-	BuildType build_type;
-	DevelType devel_type;
-	gint x, y, pos;
-	gint idx, ratio;
-	Resource supply_type, receive_type;
-	gint supply[NO_RESOURCE], receive[NO_RESOURCE];
-	gint sx, sy, spos, dx, dy, dpos;
+	GameRoll data;
+	gint roll;
 
-	sm_state_name(sm, "mode_turn");
-	if (event != SM_RECV)
-		return FALSE;
-	if (sm_recv(sm, "roll")) {
-		GameRoll data;
-		gint roll;
-
-		if (game->rolled_dice) {
-			player_send(player, FIRST_VERSION, LATEST_VERSION,
-				    "ERR already-rolled\n");
-			return TRUE;
-		}
+	if (game->rolled_dice) {
+		player_send(player, FIRST_VERSION, LATEST_VERSION,
+			    "ERR already-rolled\n");
+		return;
+	}
 
+	roll = admin_get_dice_roll();
+	if (roll == 0) {
 		/* roll the dice until we like it */
 		while (TRUE) {
-			game->die1 = get_rand(6) + 1;
-			game->die2 = get_rand(6) + 1;
-			roll = game->die1 + game->die2;
-			game->rolled_dice = TRUE;
+			if (game->params->use_dice_deck) {
+				/* dice deck */
+				gint i;
+				gint card;
+				if (game->num_dice_cards <=
+				    game->params->num_removed_dice_cards) {
+					/* shuffle deck */
+					for (i = 0; i < 36; i++) {
+						game->dice_cards[i] =
+						    game->
+						    params->num_dice_decks;
+					}
+					game->num_dice_cards =
+					    game->params->num_dice_decks *
+					    36;
 
+					player_broadcast(player, PB_ALL,
+							 V15,
+							 LATEST_VERSION,
+							 "shuffled-dice-deck\n");
+				}
+				card = random_guint(game->num_dice_cards);
+
+				i = -1;
+				while (card >= 0) {
+					i++;
+					card -= game->dice_cards[i];
+				}
+				game->dice_cards[i]--;
+				game->num_dice_cards--;
+				game->die1 = i % 6 + 1;
+				game->die2 = i / 6 + 1;
+			} else {
+				/* two dice */
+				game->die1 = random_guint(6) + 1;
+				game->die2 = random_guint(6) + 1;
+			}
+			roll = game->die1 + game->die2;
 			/* sevens_rule == 1: reroll first two turns */
 			if (game->params->sevens_rule == 1)
 				if (roll == 7 && game->curr_turn <= 2)
@@ -482,37 +506,62 @@ gboolean mode_turn(Player * player, gint event)
 			/* sevens_rule == 0: don't reroll anything */
 			break;
 		}
+	} else {
 		/* The administrator can override the dice */
-		if (admin_dice_roll >= 2) {
-			game->die1 = admin_dice_roll > 6 ? 6 : 1;
-			game->die2 = admin_dice_roll - game->die1;
-			roll = admin_dice_roll;
-			player_broadcast(player, PB_SILENT, FIRST_VERSION,
-					 LATEST_VERSION, "NOTE %s\n",
-					 /* Cheat mode has been activated */
-					 N_(""
-					    "The dice roll has been determined by the administrator."));
-		}
+		game->die1 = roll > 6 ? 6 : 1;
+		game->die2 = roll - game->die1;
+		player_broadcast(player, PB_SILENT, FIRST_VERSION,
+				 LATEST_VERSION, "NOTE %s\n",
+				 /* Cheat mode has been activated */
+				 N_(""
+				    "The dice roll has been determined by the administrator."));
+	}
+	game->rolled_dice = TRUE;
+	g_assert(roll == game->die1 + game->die2);
 
-		/* let people know what we rolled */
-		player_broadcast(player, PB_RESPOND, FIRST_VERSION,
-				 LATEST_VERSION, "rolled %d %d\n",
-				 game->die1, game->die2);
+	/* let people know what we rolled */
+	player_broadcast(player, PB_RESPOND, FIRST_VERSION,
+			 LATEST_VERSION, "rolled %d %d\n",
+			 game->die1, game->die2);
 
-		if (roll == 7) {
-			/* Find all players with more than 7 cards -
-			 * they must discard half (rounded down)
-			 */
-			discard_resources(game);
-			/* there are no resources to distribute on a 7 */
-			return TRUE;
-		}
-		resource_start(game);
-		data.game = game;
-		data.roll = roll;
-		map_traverse_const(map, distribute_resources, &data);
-		/* distribute resources and gold (includes resource_end) */
-		distribute_first(list_from_player(player));
+	if (roll == 7) {
+		/* Find all players with more than 7 cards -
+		 * they must discard half (rounded down)
+		 */
+		discard_resources(game);
+		/* there are no resources to distribute on a 7 */
+		return;
+	}
+	resource_start(game);
+	data.game = game;
+	data.roll = roll;
+	map_traverse_const(map, distribute_resources, &data);
+	/* distribute resources and gold (includes resource_end) */
+	distribute_first(list_from_player(player));
+	return;
+
+}
+
+/* Handle all actions that a player may perform in a turn
+ */
+gboolean mode_turn(Player * player, gint event)
+{
+	StateMachine *sm = player->sm;
+	Game *game = player->game;
+	BuildType build_type;
+	DevelType devel_type;
+	gint x, y, pos;
+	guint idx;
+	gint ratio;
+	Resource supply_type, receive_type;
+	gint supply[NO_RESOURCE], receive[NO_RESOURCE];
+	gint sx, sy, spos, dx, dy, dpos;
+
+	sm_state_name(sm, "mode_turn");
+	if (event != SM_RECV)
+		return FALSE;
+	if (sm_recv(sm, "roll")) {
+		roll_dice(player);
 		return TRUE;
 	}
 	/* try to end a turn */
@@ -535,7 +584,7 @@ gboolean mode_turn(Player * player, gint event)
 		develop_buy(player);
 		return TRUE;
 	}
-	if (sm_recv(sm, "play-develop %d", &idx, &devel_type)) {
+	if (sm_recv(sm, "play-develop %u", &idx, &devel_type)) {
 		develop_play(player, idx);
 		if (!game->params->check_victory_at_end_of_turn)
 			check_victory(player);
@@ -615,14 +664,19 @@ void turn_next_player(Game * game)
 	/* reset variables */
 	game->curr_player = player->num;
 	game->rolled_dice = FALSE;
-	game->played_develop = FALSE;
 	game->bought_develop = FALSE;
+	game->num_playable_cards = deck_count(player->devel);
 	player->build_list = buildrec_free(player->build_list);
 	game->params->map->has_moved_ship = FALSE;
 
 	/* tell everyone what's happening */
 	player_broadcast(player, PB_RESPOND, FIRST_VERSION, LATEST_VERSION,
 			 "turn %d\n", game->curr_turn);
+	if (game->is_manipulated) {
+		/* Intentionally not translated */
+		admin_broadcast(game, "The administrator has "
+				"manipulated this game.");
+	}
 
 	/* put the player in the right state */
 	sm_push(player->sm, (StateFunc) mode_turn);
